@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: acconfig.h,v 1.117 2004/05/14 08:51:04 grubba Exp $
+|| $Id: acconfig.h,v 1.142 2005/05/30 13:23:22 grubba Exp $
 */
 
 #ifndef MACHINE_H
@@ -29,6 +29,9 @@
 /* Define this if you want run time self tests */
 #undef PIKE_DEBUG
 
+/* Define to make Pike do a full cleanup at exit to detect leaks. */
+#undef DO_PIKE_CLEANUP
+
 /* Define this if you want pike to interact with valgrind. */
 #undef USE_VALGRIND
 
@@ -65,9 +68,6 @@
 /* Define this to get portable dumped bytecode. */
 #undef PIKE_PORTABLE_BYTECODE
 
-/* Define this to use the new multiset implementation. */
-#undef PIKE_NEW_MULTISETS
-
 /* Enable profiling */
 #undef PROFILING
 
@@ -93,6 +93,9 @@
 /* Define this if your ld uses -R, but your cc wants -Wl,-R */
 #undef USE_Wl_R
 
+/* Define this if your ld uses -rpath, but your cc -Qoption,ld,-rpath (icc) */
+#undef USE_Qoption
+
 /* Define this if your ld uses -YP, , but your cc wants -Xlinker -YP, */
 #undef USE_XLINKER_YP_
 
@@ -165,9 +168,6 @@
 /* define this if igonoring SIGFPE helps with core dumps */
 #undef IGNORE_SIGFPE
 
-/* define this to disable use of setlocale(3). */
-#undef DONT_USE_SYSTEM_LOCALE
-
 /* define if you want to use double precision floats instead of single */
 #undef WITH_DOUBLE_PRECISION_SVALUE
 
@@ -183,7 +183,6 @@
 /* force this type upon ints */
 #undef WITH_LONG_INT
 #undef WITH_LONG_LONG_INT
-#undef WITH_SHORT_INT
 #undef WITH_INT_INT
 
 /* define to the type of pike primitive ints */
@@ -248,12 +247,24 @@
 /* Define if you have ualarm. */
 #undef HAVE_UALARM
 
-/* Define if your ualarm takes two args.. */
+/* Define if your ualarm takes two args. */
 #undef UALARM_TAKES_TWO_ARGS
 
+/* Define if your ptrace takes four args. */
+#undef PTRACE_TAKES_FOUR_ARGS
+
+/* Define if argument 3 to ptrace is a pointer type. */
+#undef PTRACE_ADDR_TYPE_IS_POINTER
+
 /* Define if gettimeofday takes to arguments */
 #undef GETTIMEOFDAY_TAKES_TWO_ARGS
 
+/* Define if realloc(NULL, SZ) works. */
+#undef HAVE_WORKING_REALLOC_NULL
+
+/* Define if gethrvtime works (i.e. even without ptime). */
+#undef HAVE_WORKING_GETHRVTIME
+
 /* Define if you have gethrtime */
 #undef HAVE_GETHRTIME
 
@@ -266,6 +277,9 @@
 /* Define if you have a working, 8-bit-clean memcmp */
 #undef HAVE_MEMCMP
 
+/* Define if it is possible to allocate PROT_EXEC memory with mmap */
+#undef MEXEC_USES_MMAP
+
 /* Define if you have gethostname */
 #undef HAVE_GETHOSTNAME
 
@@ -316,9 +330,12 @@
 /* You have cl-type __declspec? */
 #undef HAVE_DECLSPEC
 
-/* Do your compiler grock 'volatile' */
+/* Does your compiler grock 'volatile' */
 #define VOLATILE volatile
 
+/* Define to empty if your compiler doesn't support C99's restrict keyword. */
+#undef restrict
+
 /* Define this if your compiler doesn't allow cast of void * to function pointer */
 #undef NO_CAST_TO_FUN
 
@@ -364,8 +381,8 @@
 /* The byteorder your machine use, most use 4321, PC use 1234 */
 #define PIKE_BYTEORDER 0
 
-/* What alignment do 32-bit integers need */
-#define PIKE_INT32_ALIGNMENT 4
+/* What alignment do pointers need */
+#define PIKE_POINTER_ALIGNMENT 4
 
 /* Assembler prefix for general purpose registers */
 #undef PIKE_CPU_REG_PREFIX
@@ -385,6 +402,9 @@
 /* Define if you have struct timeval */
 #undef HAVE_STRUCT_TIMEVAL
 
+/* Define if you have struct sockaddr_in6 */
+#undef HAVE_STRUCT_SOCKADDR_IN6
+
 /* Define this to the max value of an unsigned short unles <limits.h> does.. */
 #undef USHRT_MAX
 
@@ -406,6 +426,9 @@
 /* Define this if you have Windows NT threads */
 #undef NT_THREADS
 
+/* Define this if your THREAD_T type is a pointer type. */
+#undef PIKE_THREAD_T_IS_POINTER
+
 /* Define to the flag to get an error checking mutex, if supported. */
 #undef PIKE_MUTEX_ERRORCHECK
 
@@ -457,6 +480,12 @@
 /* Use poll() instead of select() ? */
 #undef HAVE_AND_USE_POLL
 
+/* Enable use of /dev/epoll on Linux. */
+#undef WITH_EPOLL
+
+/* Define to the poll device (eg "/dev/poll") */
+#undef PIKE_POLL_DEVICE
+
 /* This works on Solaris or any UNIX where
  * waitpid can report ECHILD when running more than one at once
  * (or any UNIX where waitpid actually works)
@@ -469,12 +498,6 @@
  */
 #undef USE_SIGCHILD
 
-/* Enable code to handle Out-Of-Band data */
-#undef WITH_OOB
-
-/* Enable individual tracing of threads */
-#undef THREAD_TRACE
-
 /* Enable tracing of the compiler */
 #undef YYDEBUG
 
@@ -499,6 +522,9 @@
 /* set this to the modifier type string to print ptrdiff_t, like "" or "l" */
 #undef PRINTPTRDIFFT
 
+/* set this to the modifier type string to print INT64 if that type exists */
+#undef PRINTINT64
+
 /* Define if the compiler understand union initializations. */
 #undef HAVE_UNION_INIT
 
@@ -512,11 +538,16 @@
 /* Define to the page size (handled efficiently by malloc). */
 #undef PIKE_MALLOC_PAGE_SIZE
 
-/* Define to the size of the c-stack for new threads */
-#undef PIKE_THREAD_C_STACK_SIZE
+/* PIKE_YES if the number reported by get_cpu_time (rusage.c) is
+ * thread local, PIKE_NO if it isn't, PIKE_UNKNOWN if it couldn't be
+ * established. */
+#undef CPU_TIME_IS_THREAD_LOCAL
 
 @BOTTOM@
 
+/* Define to the size of the c-stack for new threads */
+#undef PIKE_THREAD_C_STACK_SIZE
+
 /* NT stuff */
 #undef HAVE_GETSYSTEMTIMEASFILETIME
 #undef HAVE_LOADLIBRARY
@@ -579,13 +610,9 @@
 #define DECLSPEC(X)
 #endif /* HAVE_DECLSPEC */
 
-#ifndef HAVE_WORKING___FUNC__
-#ifdef HAVE_WORKING___FUNCTION__
-#define __func__	__FUNCTION__
-#else /* !HAVE_WORKING___FUNCTION__ */
-#define __func__	"unknown"
-#endif /* HAVE_WORKING___FUNCTION__ */
-#endif /* !HAVE_WORKING___FUNC__ */
+#ifndef HAVE_WORKING_REALLOC_NULL
+#define realloc(PTR, SZ)	pike_realloc(PTR,SZ)
+#endif
 
 /* NOTE:
  *    PIKE_CONCAT doesn't get defined if there isn't any way to
@@ -606,4 +633,9 @@
 #define TOSTR(X)	#X
 #define DEFINETOSTR(X)	TOSTR(X)
 
+/* Some identifiers used as flags in the defines above. */
+#define PIKE_YES	1
+#define PIKE_NO		2
+#define PIKE_UNKNOWN	3
+
 #endif /* MACHINE_H */
