@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: array.c,v 1.158 2004/04/06 15:37:54 nilsson Exp $
+|| $Id: array.c,v 1.164 2005/05/18 12:36:53 mast Exp $
 */
 
 #include "global.h"
@@ -25,8 +25,9 @@
 #include "bignum.h"
 #include "cyclic.h"
 #include "multiset.h"
+#include "mapping.h"
 
-RCSID("$Id: array.c,v 1.158 2004/04/06 15:37:54 nilsson Exp $");
+RCSID("$Id: array.c,v 1.164 2005/05/18 12:36:53 mast Exp $");
 
 PMOD_EXPORT struct array empty_array=
 {
@@ -90,10 +91,13 @@ PMOD_EXPORT struct array *low_allocate_array(ptrdiff_t size, ptrdiff_t extra_spa
     return &empty_array;
   }
 
+  if( (size_t) (size+extra_space-1) >
+      (ULONG_MAX-sizeof(struct array))/sizeof(struct svalue) )
+    Pike_error("Too large array (memory size exceeds size of size_t)\n");
   v=(struct array *)malloc(sizeof(struct array)+
 			   (size+extra_space-1)*sizeof(struct svalue));
   if(!v)
-    Pike_error("Couldn't allocate array, out of memory.\n");
+    Pike_error(msg_out_of_mem);
 
   GC_ALLOC(v);
 
@@ -172,7 +176,7 @@ PMOD_EXPORT struct array *array_set_flags(struct array *a, int flags)
     a->flags = flags;
   else {
     free_array(a);
-    switch (flags) {
+    switch (flags & (ARRAY_WEAK_FLAG|ARRAY_WEAK_SHRINK)) {
       case 0:
 	add_ref(a = &empty_array); break;
       case ARRAY_WEAK_FLAG:
@@ -266,8 +270,7 @@ PMOD_EXPORT void simple_array_index_no_free(struct svalue *s,
 	if (a->size) {
 	  index_error(0,0,0,&tmp,ind,
 		      "Index %"PRINTPIKEINT"d is out of array range "
-		      "%"PRINTPTRDIFFT"d..%"PRINTPTRDIFFT"d.\n",
-		      p, -a->size, a->size-1);
+		      "%d..%d.\n", p, -a->size, a->size-1);
 	} else {
 	  index_error(0,0,0,&tmp,ind,
 		      "Attempt to index the empty array with %"PRINTPIKEINT"d.\n", p);
@@ -319,8 +322,7 @@ PMOD_EXPORT void simple_set_index(struct array *a,struct svalue *ind,struct sval
       if(i<0 || i>=a->size) {
 	if (a->size) {
 	  Pike_error("Index %"PRINTPIKEINT"d is out of array range "
-		     "%"PRINTPTRDIFFT"d..%"PRINTPTRDIFFT"d.\n",
-		     p, -a->size, a->size-1);
+		     "%d..%d.\n", p, -a->size, a->size-1);
 	} else {
 	  Pike_error("Attempt to index the empty array with %"PRINTPIKEINT"d.\n", p);
 	}
@@ -378,7 +380,8 @@ PMOD_EXPORT struct array *array_insert(struct array *v,struct svalue *s,INT32 in
   }else{
     struct array *ret;
 
-    ret=allocate_array_no_init(v->size+1, (v->size >> 3) + 1);
+    ret = array_set_flags(allocate_array_no_init(v->size+1, (v->size >> 3) + 1),
+			  v->flags);
     ret->type_field = v->type_field;
 
     MEMCPY(ITEM(ret), ITEM(v), sizeof(struct svalue) * index);
@@ -398,6 +401,40 @@ PMOD_EXPORT struct array *array_insert(struct array *v,struct svalue *s,INT32 in
   return v;
 }
 
+/*
+ * Shrink an array destructively
+ */
+PMOD_EXPORT struct array *array_shrink(struct array *v, ptrdiff_t size)
+{
+  struct array *a;
+
+#ifdef PIKE_DEBUG
+  if(v->refs>2) /* Odd, but has to be two */
+    Pike_fatal("Array shrink on array with many references.\n");
+
+  if(size > v->size)
+    Pike_fatal("Illegal argument to array_shrink.\n");
+#endif
+
+  if(!size || (size*4 < v->malloced_size + 4)) /* Should we realloc it? */
+  {
+    a = array_set_flags(allocate_array_no_init(size, 0), v->flags);
+    if (a->size) {
+      a->type_field = v->type_field;
+    }
+
+    free_svalues(ITEM(v) + size, v->size - size, v->type_field);
+    MEMCPY(ITEM(a), ITEM(v), size*sizeof(struct svalue));
+    v->size=0;
+    free_array(v);
+    return a;
+  }else{
+    free_svalues(ITEM(v) + size, v->size - size, v->type_field);
+    v->size=size;
+    return v;
+  }
+}
+
 /*
  * resize array, resize an array destructively
  */
@@ -422,52 +459,18 @@ PMOD_EXPORT struct array *resize_array(struct array *a, INT32 size)
       }
       a->type_field |= BIT_INT;
       return a;
-    }else{
+    } else {
       struct array *ret;
-      ret=low_allocate_array(size, (size>>1) + 4);
-      MEMCPY(ITEM(ret),ITEM(a),sizeof(struct svalue)*a->size);
+      ret = array_set_flags(low_allocate_array(size, (size>>1) + 4),
+			    a->flags);
+      MEMCPY(ITEM(ret), ITEM(a), sizeof(struct svalue)*a->size);
       ret->type_field = DO_NOT_WARN((TYPE_FIELD)(a->type_field | BIT_INT));
       a->size=0;
       free_array(a);
       return ret;
     }
-  }else{
-    /* We should shrink the array */
-    free_svalues(ITEM(a)+size, a->size - size, a->type_field);
-    a->size = size;
-    return a;
-  }
-}
-
-/*
- * Shrink an array destructively
- */
-PMOD_EXPORT struct array *array_shrink(struct array *v, ptrdiff_t size)
-{
-  struct array *a;
-
-#ifdef PIKE_DEBUG
-  if(v->refs>2) /* Odd, but has to be two */
-    Pike_fatal("Array shrink on array with many references.\n");
-
-  if(size > v->size)
-    Pike_fatal("Illegal argument to array_shrink.\n");
-#endif
-
-  if(size*2 < v->malloced_size + 4) /* Should we realloc it? */
-  {
-    a=allocate_array_no_init(size,0);
-    a->type_field = v->type_field;
-
-    free_svalues(ITEM(v) + size, v->size - size, v->type_field);
-    MEMCPY(ITEM(a), ITEM(v), size*sizeof(struct svalue));
-    v->size=0;
-    free_array(v);
-    return a;
-  }else{
-    free_svalues(ITEM(v) + size, v->size - size, v->type_field);
-    v->size=size;
-    return v;
+  } else {
+    return array_shrink(a, size);
   }
 }
 
@@ -484,10 +487,15 @@ PMOD_EXPORT struct array *array_remove(struct array *v,INT32 index)
 #endif
 
   array_free_index(v, index);
-  if(v->size!=1 &&
-     v->size*2 + 4 < v->malloced_size ) /* Should we realloc it? */
+  if (v->size == 1) {
+    v->size = 0;
+    /* NOTE: The following uses the fact that array_set_flags()
+     *       will reallocate the array if it has zero size!
+     */
+    return array_set_flags(v, v->flags);
+  } else if(v->size*4 + 4 < v->malloced_size ) /* Should we realloc it? */
   {
-    a=allocate_array_no_init(v->size-1, 0);
+    a = array_set_flags(allocate_array_no_init(v->size-1, 0), v->flags);
     a->type_field = v->type_field;
 
     if(index>0)
@@ -499,7 +507,7 @@ PMOD_EXPORT struct array *array_remove(struct array *v,INT32 index)
     v->size=0;
     free_array(v);
     return a;
-  }else{
+  } else {
     if(v->size-index>1)
     {
       MEMMOVE((char *)(ITEM(v)+index),
@@ -556,7 +564,7 @@ PMOD_EXPORT ptrdiff_t array_search(struct array *v, struct svalue *s,
 }
 
 /*
- * Slice a pice of an array (nondestructively)
+ * Slice a piece of an array (nondestructively)
  * return an array consisting of v[start..end-1]
  */
 PMOD_EXPORT struct array *slice_array(struct array *v, ptrdiff_t start,
@@ -594,15 +602,17 @@ PMOD_EXPORT struct array *slice_array(struct array *v, ptrdiff_t start,
 #endif
 
   a=allocate_array_no_init(end-start,0);
-  a->type_field = v->type_field;
+  if (end-start) {
+    a->type_field = v->type_field;
 
-  assign_svalues_no_free(ITEM(a), ITEM(v)+start, end-start, v->type_field);
+    assign_svalues_no_free(ITEM(a), ITEM(v)+start, end-start, v->type_field);
+  }
 
   return a;
 }
 
 /*
- * Slice a pice of an array (nondestructively)
+ * Slice a piece of an array (nondestructively)
  * return an array consisting of v[start..end-1]
  */
 PMOD_EXPORT struct array *friendly_slice_array(struct array *v,
@@ -1184,18 +1194,15 @@ PMOD_EXPORT struct array *reorder_and_copy_array(struct array *v, INT32 *order)
 }
 
 /* Maybe I should have a 'clean' flag for this computation */
-PMOD_EXPORT void array_fix_type_field(struct array *v)
+PMOD_EXPORT TYPE_FIELD array_fix_type_field(struct array *v)
 {
   int e;
   TYPE_FIELD t;
 
-  t=0;
-
   if(v->flags & ARRAY_LVALUE)
-  {
-    v->type_field=BIT_MIXED|BIT_UNFINISHED;
-    return;
-  }
+    return v->type_field=BIT_MIXED|BIT_UNFINISHED;
+
+  t=0;
 
   for(e=0; e<v->size; e++) {
     check_svalue (ITEM(v) + e);
@@ -1210,7 +1217,7 @@ PMOD_EXPORT void array_fix_type_field(struct array *v)
 	       v->type_field, t);
   }
 #endif
-  v->type_field = t;
+  return v->type_field = t;
 }
 
 #ifdef PIKE_DEBUG
@@ -1253,8 +1260,8 @@ void array_check_type_field(struct array *v)
  * type. The 'union anything' may be changed, but not the type.
  */
 PMOD_EXPORT union anything *low_array_get_item_ptr(struct array *a,
-				       INT32 ind,
-				       TYPE_T t)
+						   INT32 ind,
+						   TYPE_T t)
 {
   if(ITEM(a)[ind].type == t) return & (ITEM(a)[ind].u);
   return 0;
@@ -1267,8 +1274,8 @@ PMOD_EXPORT union anything *low_array_get_item_ptr(struct array *a,
  * the index as an svalue.
  */
 PMOD_EXPORT union anything *array_get_item_ptr(struct array *a,
-				   struct svalue *ind,
-				   TYPE_T t)
+					       struct svalue *ind,
+					       TYPE_T t)
 {
   INT_TYPE i, p;
   if(ind->type != T_INT)
@@ -1279,8 +1286,7 @@ PMOD_EXPORT union anything *array_get_item_ptr(struct array *a,
   if(i<0 || i>=a->size) {
     if (a->size) {
       Pike_error("Index %"PRINTPIKEINT"d is out of array range "
-		 "%"PRINTPTRDIFFT"d..%"PRINTPTRDIFFT"d.\n",
-		 p, -a->size, a->size-1);
+		 "%d..%d.\n", p, -a->size, a->size-1);
     } else {
       Pike_error("Attempt to index the empty array with %"PRINTPIKEINT"d.\n", p);
     }
@@ -2106,6 +2112,11 @@ PMOD_EXPORT struct array *copy_array_recursively(struct array *a,struct processi
   if(d_flag > 1)  array_check_type_field(a);
 #endif
 
+  if (!a->size) {
+    add_ref(&empty_array);
+    return array_set_flags(&empty_array, a->flags);
+  }
+
   doing.next=p;
   doing.pointer_a=(void *)a;
   for(;p;p=p->next)
@@ -2141,7 +2152,7 @@ PMOD_EXPORT void apply_array(struct array *a, INT32 args)
   check_stack(120 + args + 1);
 
   /* FIXME: Ought to use a better key on the arguments below. */
-  if (!(cycl = (struct array *)BEGIN_CYCLIC(a, args))) {
+  if (!(cycl = (struct array *)BEGIN_CYCLIC(a, (ptrdiff_t) args))) {
     BEGIN_AGGREGATE_ARRAY(a->size) {
       SET_CYCLIC_RET(Pike_sp[-1].u.array);
       for (e=0;e<a->size;e++) {
