@@ -2,10 +2,11 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: block_alloc.h,v 1.60 2003/03/17 13:57:16 grubba Exp $
+|| $Id: block_alloc.h,v 1.71 2004/04/03 15:55:15 mast Exp $
 */
 
 #undef PRE_INIT_BLOCK
+#undef DO_PRE_INIT_BLOCK
 #undef INIT_BLOCK
 #undef EXIT_BLOCK
 #undef BLOCK_ALLOC
@@ -14,6 +15,7 @@
 #undef PTR_HASH_ALLOC
 #undef COUNT_BLOCK
 #undef COUNT_OTHER
+#undef DMALLOC_DESCRIBE_BLOCK
 #undef BLOCK_ALLOC_HSIZE_SHIFT
 #undef MAX_EMPTY_BLOCKS
 #undef BLOCK_ALLOC_FILL_PAGES
@@ -26,6 +28,7 @@
 #define EXIT_BLOCK(X)
 #define COUNT_BLOCK(X)
 #define COUNT_OTHER()
+#define DMALLOC_DESCRIBE_BLOCK(X)
 #define BLOCK_ALLOC_HSIZE_SHIFT 2
 #define MAX_EMPTY_BLOCKS 4
 
@@ -33,6 +36,17 @@
 #define BLOCK_ALLOC_USED DO_IF_DMALLOC(real_used) DO_IF_NOT_DMALLOC(used)
 #endif
 
+/* Invalidate the block as far as possible if running with dmalloc.
+ */
+#define DO_PRE_INIT_BLOCK(X)	do {				\
+    DO_IF_DMALLOC(MEMSET((X), 0x55, sizeof(*(X))));		\
+    PRE_INIT_BLOCK(X);						\
+  } while (0)
+
+#ifndef PIKE_HASH_T
+#define PIKE_HASH_T	size_t
+#endif /* !PIKE_HASH_T */
+
 #ifdef PIKE_RUN_UNLOCKED
 #include "threads.h"
 
@@ -111,12 +125,12 @@ static void PIKE_CONCAT(alloc_more_,DATA)(void)				\
   PIKE_CONCAT(DATA,_blocks)=n;						\
   PIKE_CONCAT(DATA,_free_blocks)=n;					\
 									\
+  DO_PRE_INIT_BLOCK( n->x );						\
   n->x[0].BLOCK_ALLOC_NEXT=NULL;					\
-  PRE_INIT_BLOCK( n->x );						\
   for(e=1;e<(BSIZE);e++)						\
   {									\
+    DO_PRE_INIT_BLOCK( (n->x+e) );					\
     n->x[e].BLOCK_ALLOC_NEXT=(void *)&n->x[e-1];			\
-    PRE_INIT_BLOCK( (n->x+e) );						\
   }									\
   n->PIKE_CONCAT3(free_,DATA,s)=&n->x[(BSIZE)-1];			\
   /* Mark the new blocks as unavailable for now... */			\
@@ -179,8 +193,13 @@ static void PIKE_CONCAT(check_free_,DATA)(struct DATA *d)               \
     return;                                                             \
   }                                                                     \
   Pike_fatal("really_free_%s called on non-block_alloc region (%p).\n",	\
-         #DATA, d);                                                     \
+	     #DATA, d);							\
 }                                                                       \
+									\
+static void PIKE_CONCAT (dmalloc_describe_, DATA) (struct DATA *d)	\
+{									\
+  DMALLOC_DESCRIBE_BLOCK (d);						\
+}									\
 )									\
 									\
 void PIKE_CONCAT(really_free_,DATA)(struct DATA *d)			\
@@ -240,11 +259,11 @@ void PIKE_CONCAT(really_free_,DATA)(struct DATA *d)			\
       PIKE_CONCAT(DATA,_free_blocks) = blk;				\
   }									\
 									\
+  DO_PRE_INIT_BLOCK(d);							\
   d->BLOCK_ALLOC_NEXT = (void *)blk->PIKE_CONCAT3(free_,DATA,s);	\
-  PRE_INIT_BLOCK(d);							\
+  blk->PIKE_CONCAT3(free_,DATA,s)=d;					\
   /* Mark block as unavailable. */					\
   PIKE_MEM_NA(*d);							\
-  blk->PIKE_CONCAT3(free_,DATA,s)=d;					\
 									\
   if(!--blk->BLOCK_ALLOC_USED &&					\
      ++PIKE_CONCAT3(num_empty_,DATA,_blocks) > MAX_EMPTY_BLOCKS) {	\
@@ -259,9 +278,10 @@ void PIKE_CONCAT(really_free_,DATA)(struct DATA *d)			\
 									\
     DO_IF_DMALLOC({							\
 	size_t i;							\
-	extern void dmalloc_check_block_free(void *p, char *loc);	\
 	for (i = 0; i < (BSIZE); i++) {					\
-	  dmalloc_check_block_free(blk->x + i, DMALLOC_LOCATION());	\
+	  dmalloc_check_block_free(					\
+	    blk->x + i, DMALLOC_LOCATION(), #DATA,			\
+	    (describe_block_fn *) PIKE_CONCAT (dmalloc_describe_, DATA)); \
 	  dmalloc_unregister(blk->x + i, 1);				\
 	}								\
       });								\
@@ -283,10 +303,11 @@ static void PIKE_CONCAT3(free_all_,DATA,_blocks_unlocked)(void)		\
    for(tmp=PIKE_CONCAT(DATA,_blocks);tmp;tmp=tmp->next)                 \
    {                                                                    \
      size_t tmp2;							\
-     extern void dmalloc_check_block_free(void *p, char *loc);          \
      for(tmp2=0;tmp2<(BSIZE);tmp2++)					\
      {                                                                  \
-       dmalloc_check_block_free(tmp->x+tmp2, DMALLOC_LOCATION());       \
+       dmalloc_check_block_free(					\
+	 tmp->x+tmp2, DMALLOC_LOCATION(), #DATA,			\
+	 (describe_block_fn *) PIKE_CONCAT (dmalloc_describe_, DATA));	\
        dmalloc_unregister(tmp->x+tmp2, 1);                              \
      }                                                                  \
    }                                                                    \
@@ -347,7 +368,8 @@ size_t PIKE_CONCAT(DATA,_hash_table_magnitude)=0;			     \
 static size_t PIKE_CONCAT(num_,DATA)=0;					     \
 									     \
 static inline struct DATA *						     \
- PIKE_CONCAT3(really_low_find_,DATA,_unlocked)(void *ptr, size_t hval)	     \
+ PIKE_CONCAT3(really_low_find_,DATA,_unlocked)(void *ptr,		     \
+					       PIKE_HASH_T hval)	     \
 {									     \
   struct DATA *p,**pp;							     \
   p=PIKE_CONCAT(DATA,_hash_table)[hval];                                     \
@@ -371,20 +393,21 @@ static inline struct DATA *						     \
 struct DATA *PIKE_CONCAT(find_,DATA)(void *ptr)				     \
 {									     \
   struct DATA *p;                                                            \
-  size_t hval = (size_t)ptr;						     \
+  PIKE_HASH_T hval = (PIKE_HASH_T)PTR_TO_INT(ptr);			     \
   DO_IF_RUN_UNLOCKED(mt_lock(&PIKE_CONCAT(DATA,_mutex)));                    \
   if(!PIKE_CONCAT(DATA,_hash_table_size)) {                                  \
     DO_IF_RUN_UNLOCKED(mt_unlock(&PIKE_CONCAT(DATA,_mutex)));                \
     return 0;                                                                \
   }                                                                          \
-  hval%=PIKE_CONCAT(DATA,_hash_table_size);				     \
+  hval %= (PIKE_HASH_T)PIKE_CONCAT(DATA,_hash_table_size);		     \
   p=PIKE_CONCAT3(really_low_find_,DATA,_unlocked)(ptr, hval);		     \
   DO_IF_RUN_UNLOCKED(mt_unlock(&PIKE_CONCAT(DATA,_mutex)));                  \
   return p;								     \
 }									     \
 									     \
 static inline struct DATA *						     \
- PIKE_CONCAT3(just_find_,DATA,_unlocked)(void *ptr, size_t hval)	     \
+ PIKE_CONCAT3(just_find_,DATA,_unlocked)(void *ptr,			     \
+					 PIKE_HASH_T hval)		     \
 {									     \
   struct DATA *p,**pp;							     \
   p=PIKE_CONCAT(DATA,_hash_table)[hval];                                     \
@@ -402,13 +425,13 @@ static inline struct DATA *						     \
 static struct DATA *PIKE_CONCAT(just_find_,DATA)(void *ptr)		     \
 {									     \
   struct DATA *p;                                                            \
-  size_t hval = (size_t)ptr;						     \
+  PIKE_HASH_T hval = (PIKE_HASH_T)PTR_TO_INT(ptr);			     \
   DO_IF_RUN_UNLOCKED(mt_lock(&PIKE_CONCAT(DATA,_mutex)));                    \
   if(!PIKE_CONCAT(DATA,_hash_table_size)) {                                  \
     DO_IF_RUN_UNLOCKED(mt_unlock(&PIKE_CONCAT(DATA,_mutex)));                \
     return 0;                                                                \
   }                                                                          \
-  hval %= PIKE_CONCAT(DATA,_hash_table_size);				     \
+  hval %= (PIKE_HASH_T)PIKE_CONCAT(DATA,_hash_table_size);		     \
   p=PIKE_CONCAT3(just_find_,DATA,_unlocked)(ptr, hval);			     \
   DO_IF_RUN_UNLOCKED(mt_unlock(&PIKE_CONCAT(DATA,_mutex)));                  \
   return p;								     \
@@ -419,9 +442,9 @@ static struct DATA *PIKE_CONCAT(just_find_,DATA)(void *ptr)		     \
 struct DATA *PIKE_CONCAT(make_,DATA)(void *ptr)				     \
 {									     \
   struct DATA *p;							     \
-  size_t hval=(size_t)ptr;						     \
+  PIKE_HASH_T hval = (PIKE_HASH_T)PTR_TO_INT(ptr);			     \
   DO_IF_RUN_UNLOCKED(mt_lock(&PIKE_CONCAT(DATA,_mutex)));                    \
-  hval%=PIKE_CONCAT(DATA,_hash_table_size);				     \
+  hval %= (PIKE_HASH_T)PIKE_CONCAT(DATA,_hash_table_size);		     \
   p=PIKE_CONCAT3(make_,DATA,_unlocked)(ptr,hval);                            \
   DO_IF_RUN_UNLOCKED(mt_unlock(&PIKE_CONCAT(DATA,_mutex)));                  \
   return p;								     \
@@ -430,9 +453,9 @@ struct DATA *PIKE_CONCAT(make_,DATA)(void *ptr)				     \
 struct DATA *PIKE_CONCAT(get_,DATA)(void *ptr)			 	     \
 {									     \
   struct DATA *p;							     \
-  size_t hval=(size_t)ptr;						     \
+  PIKE_HASH_T hval = (PIKE_HASH_T)PTR_TO_INT(ptr);			     \
   DO_IF_RUN_UNLOCKED(mt_lock(&PIKE_CONCAT(DATA,_mutex)));                    \
-  hval%=PIKE_CONCAT(DATA,_hash_table_size);				     \
+  hval %= (PIKE_HASH_T)PIKE_CONCAT(DATA,_hash_table_size);		     \
   if(!(p=PIKE_CONCAT3(really_low_find_,DATA,_unlocked)(ptr, hval)))          \
     p=PIKE_CONCAT3(make_,DATA,_unlocked)(ptr, hval);		             \
   DO_IF_RUN_UNLOCKED(mt_unlock(&PIKE_CONCAT(DATA,_mutex)));                  \
@@ -442,9 +465,9 @@ struct DATA *PIKE_CONCAT(get_,DATA)(void *ptr)			 	     \
 int PIKE_CONCAT3(check_,DATA,_semafore)(void *ptr)			     \
 {									     \
   struct DATA *p;							     \
-  size_t hval=(size_t)ptr;						     \
+  PIKE_HASH_T hval = (PIKE_HASH_T)PTR_TO_INT(ptr);			     \
   DO_IF_RUN_UNLOCKED(mt_lock(&PIKE_CONCAT(DATA,_mutex)));                    \
-  hval%=PIKE_CONCAT(DATA,_hash_table_size);				     \
+  hval %= (PIKE_HASH_T)PIKE_CONCAT(DATA,_hash_table_size);		     \
   if((p=PIKE_CONCAT3(really_low_find_,DATA,_unlocked)(ptr, hval)))	     \
   {                                                                          \
     DO_IF_RUN_UNLOCKED(mt_unlock(&PIKE_CONCAT(DATA,_mutex)));                \
@@ -458,9 +481,10 @@ int PIKE_CONCAT3(check_,DATA,_semafore)(void *ptr)			     \
 									     \
 void PIKE_CONCAT(move_,DATA)(struct DATA *block, void *new_ptr)		     \
 {									     \
-  size_t hval = (size_t) block->PTR_HASH_ALLOC_DATA;			     \
+  PIKE_HASH_T hval =							     \
+    (PIKE_HASH_T)PTR_TO_INT(block->PTR_HASH_ALLOC_DATA);		     \
   DO_IF_RUN_UNLOCKED(mt_lock(&PIKE_CONCAT(DATA,_mutex)));		     \
-  hval %= PIKE_CONCAT(DATA,_hash_table_size);				     \
+  hval %= (PIKE_HASH_T)PIKE_CONCAT(DATA,_hash_table_size);		     \
   if (!PIKE_CONCAT3(really_low_find_,DATA,_unlocked)(			     \
 	block->PTR_HASH_ALLOC_DATA, hval))				     \
     Pike_fatal("The block to move wasn't found.\n");			     \
@@ -470,7 +494,8 @@ void PIKE_CONCAT(move_,DATA)(struct DATA *block, void *new_ptr)		     \
   );									     \
   PIKE_CONCAT(DATA,_hash_table)[hval] = block->BLOCK_ALLOC_NEXT;	     \
   block->PTR_HASH_ALLOC_DATA = new_ptr;					     \
-  hval = (size_t) new_ptr % PIKE_CONCAT(DATA,_hash_table_size);		     \
+  hval = (PIKE_HASH_T)PTR_TO_INT(new_ptr) %				     \
+    (PIKE_HASH_T)PIKE_CONCAT(DATA,_hash_table_size);			     \
   block->BLOCK_ALLOC_NEXT = PIKE_CONCAT(DATA,_hash_table)[hval];	     \
   PIKE_CONCAT(DATA,_hash_table)[hval] = block;				     \
   DO_IF_RUN_UNLOCKED(mt_unlock(&PIKE_CONCAT(DATA,_mutex)));		     \
@@ -479,14 +504,14 @@ void PIKE_CONCAT(move_,DATA)(struct DATA *block, void *new_ptr)		     \
 int PIKE_CONCAT(remove_,DATA)(void *ptr)				     \
 {									     \
   struct DATA *p;							     \
-  size_t hval=(size_t)ptr;						     \
+  PIKE_HASH_T hval = (PIKE_HASH_T)PTR_TO_INT(ptr);			     \
   DO_IF_RUN_UNLOCKED(mt_lock(&PIKE_CONCAT(DATA,_mutex)));                    \
   if(!PIKE_CONCAT(DATA,_hash_table))                                         \
   {                                                                          \
     DO_IF_RUN_UNLOCKED(mt_unlock(&PIKE_CONCAT(DATA,_mutex)));                \
     return 0;				                                     \
   }                                                                          \
-  hval%=PIKE_CONCAT(DATA,_hash_table_size);				     \
+  hval %= (PIKE_HASH_T)PIKE_CONCAT(DATA,_hash_table_size);		     \
   if((p=PIKE_CONCAT3(really_low_find_,DATA,_unlocked)(ptr, hval)))	     \
   {									     \
     PIKE_CONCAT(num_,DATA)--;						     \
@@ -539,10 +564,10 @@ void PIKE_CONCAT3(exit_,DATA,_hash)(void)				     \
 }
 
 #define PTR_HASH_ALLOC_FIXED(DATA,BSIZE)				     \
-struct DATA *PIKE_CONCAT3(make_,DATA,_unlocked)(void *ptr, size_t hval);     \
+struct DATA *PIKE_CONCAT3(make_,DATA,_unlocked)(void *ptr, PIKE_HASH_T hval);\
 LOW_PTR_HASH_ALLOC(DATA,BSIZE)                                               \
 									     \
-struct DATA *PIKE_CONCAT3(make_,DATA,_unlocked)(void *ptr, size_t hval)	     \
+struct DATA *PIKE_CONCAT3(make_,DATA,_unlocked)(void *ptr, PIKE_HASH_T hval) \
 {									     \
   struct DATA *p;							     \
 									     \
@@ -559,7 +584,8 @@ struct DATA *PIKE_CONCAT3(make_,DATA,_unlocked)(void *ptr, size_t hval)	     \
 
 
 #define PTR_HASH_ALLOC(DATA,BSIZE)					     \
-struct DATA *PIKE_CONCAT3(make_,DATA,_unlocked)(void *ptr, size_t hval);     \
+struct DATA *PIKE_CONCAT3(make_,DATA,_unlocked)(void *ptr,		     \
+						PIKE_HASH_T hval);	     \
 LOW_PTR_HASH_ALLOC(DATA,BSIZE)                                               \
                                                                              \
 static void PIKE_CONCAT(DATA,_rehash)()					     \
@@ -568,7 +594,7 @@ static void PIKE_CONCAT(DATA,_rehash)()					     \
   extern INT32 hashprimes[32];						     \
   struct DATA **old_hash;	                                 	     \
   struct DATA *p;							     \
-  size_t hval;								     \
+  PIKE_HASH_T hval;							     \
   size_t e;								     \
                                                                              \
   old_hash= PIKE_CONCAT(DATA,_hash_table);		                     \
@@ -588,8 +614,8 @@ static void PIKE_CONCAT(DATA,_rehash)()					     \
       while((p=old_hash[e]))						     \
       {									     \
 	old_hash[e]=p->BLOCK_ALLOC_NEXT;				     \
-        hval=(size_t)(p->PTR_HASH_ALLOC_DATA);				     \
-	hval%=PIKE_CONCAT(DATA,_hash_table_size);			     \
+	hval = (PIKE_HASH_T)PTR_TO_INT(p->PTR_HASH_ALLOC_DATA);		     \
+	hval %= (PIKE_HASH_T)PIKE_CONCAT(DATA,_hash_table_size);	     \
 	p->BLOCK_ALLOC_NEXT=PIKE_CONCAT(DATA,_hash_table)[hval];	     \
 	PIKE_CONCAT(DATA,_hash_table)[hval]=p;				     \
       }									     \
@@ -601,7 +627,8 @@ static void PIKE_CONCAT(DATA,_rehash)()					     \
   }									     \
 }									     \
 									     \
-struct DATA *PIKE_CONCAT3(make_,DATA,_unlocked)(void *ptr, size_t hval)	     \
+struct DATA *PIKE_CONCAT3(make_,DATA,_unlocked)(void *ptr,		     \
+						PIKE_HASH_T hval)	     \
 {									     \
   struct DATA *p;							     \
 									     \
@@ -613,8 +640,8 @@ struct DATA *PIKE_CONCAT3(make_,DATA,_unlocked)(void *ptr, size_t hval)	     \
      PIKE_CONCAT(DATA,_hash_table_size))				     \
   {									     \
     PIKE_CONCAT(DATA,_rehash)();					     \
-    hval=(size_t)ptr;							     \
-    hval%=PIKE_CONCAT(DATA,_hash_table_size);				     \
+    hval = (PIKE_HASH_T)PTR_TO_INT(ptr);				     \
+    hval %= (PIKE_HASH_T)PIKE_CONCAT(DATA,_hash_table_size);		     \
   }									     \
 									     \
   p=BA_UL(PIKE_CONCAT(alloc_,DATA))();					     \
