@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 1.875a.  */
+/* A Bison parser, made by GNU Bison 2.0.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -194,7 +194,7 @@
 
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 176 "/tmp/pikedeb.a8fa1cdb35/7.6/src/language.yacc"
+#line 176 "language.yacc"
 typedef union YYSTYPE {
   int number;
   FLOAT_TYPE fnum;
@@ -202,7 +202,7 @@ typedef union YYSTYPE {
   char *str;
   void *ptr;
 } YYSTYPE;
-/* Line 1240 of yacc.c.  */
+/* Line 1318 of yacc.c.  */
 #line 207 "y.tab.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
