@@ -1,27 +1,30 @@
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: png.c,v 1.64 2005/05/09 11:23:17 nilsson Exp $
+*/
+
 #include "global.h"
-RCSID("$Id: png.c,v 1.41 2000/12/05 21:08:27 per Exp $");
+RCSID("$Id: png.c,v 1.64 2005/05/09 11:23:17 nilsson Exp $");
 
 #include "image_machine.h"
 
-#include "pike_macros.h"
 #include "object.h"
-#include "constants.h"
 #include "interpret.h"
 #include "svalue.h"
 #include "threads.h"
-#include "array.h"
 #include "mapping.h"
 #include "pike_error.h"
 #include "stralloc.h"
-#include "dynamic_buffer.h"
 #include "builtin_functions.h"
 #include "operators.h"
 
 #include "image.h"
 #include "colortable.h"
 
-/* MUST BE INCLUDED LAST */
-#include "module_magic.h"
+
+#define sp Pike_sp
 
 extern struct program *image_colortable_program;
 extern struct program *image_program;
@@ -38,14 +41,16 @@ static struct pike_string *param_type;
 static struct pike_string *param_bpp;
 static struct pike_string *param_background;
 
+/*! @module Image
+ */
 
-/*
-**! module Image
-**! submodule PNG
-**!
-**! note
-**!	This module uses <tt>zlib</tt>.
-*/
+/*! @module PNG
+ *!   Support for encoding and decoding the Portable Network Graphics
+ *!   format, PNG.
+ *!
+ *! @note
+ *!   This module uses zlib.
+ */
 
 static INLINE void push_nbo_32bit(size_t x)
 {
@@ -72,13 +77,12 @@ static INLINE COLORTYPE _png_c16(unsigned long z,int bpp)
 {
    switch (bpp)
    {
-      case 16: return z>>8;
-      case 4:  return z*17;
-      case 2:  return z*0x55;
-      case 1:  return z*255;
-      default: return z;
+      case 16: return DO_NOT_WARN((COLORTYPE)(z>>8));
+      case 4:  return DO_NOT_WARN((COLORTYPE)(z*17));
+      case 2:  return DO_NOT_WARN((COLORTYPE)(z*0x55));
+      case 1:  return DO_NOT_WARN((COLORTYPE)(z*255));
+      default: return DO_NOT_WARN((COLORTYPE)z);
    }
-
 }
 
 static INLINE INT32 call_gz_crc32(INT32 args)
@@ -86,7 +90,8 @@ static INLINE INT32 call_gz_crc32(INT32 args)
    INT32 z;
    apply_svalue(&gz_crc32,args);
    if (sp[-1].type!=T_INT)
-      Pike_error("Image.PNG: internal error (not integer from Gz.crc32)\n");
+      PIKE_ERROR("Image.PNG", "Internal error (not integer from Gz.crc32).\n",
+		 sp, args);
    z=sp[-1].u.integer;
    pop_stack();
    return z;
@@ -150,18 +155,12 @@ static void png_compress(int style)
    free_object(o);
 }
 
-/*
-49 48 44 52
-00 00 01 e0  00 00 01 68  08 06 00 00  00 8f 37 28  20
--> 00 00 00  04
-*/
-/*
-**! method string _chunk(string type,string data)
-**! 	Encodes a PNG chunk.
-**!
-**! note
-**!	Please read about the PNG file format.
-*/
+/*! @decl string _chunk(string type, string data)
+ *! 	Encodes a PNG chunk.
+ *!
+ *! @note
+ *!	Please read about the PNG file format.
+ */
 
 static void image_png__chunk(INT32 args)
 {
@@ -183,28 +182,28 @@ static void image_png__chunk(INT32 args)
 }
 
 
-/*
-**! method array __decode(string data)
-**! method array __decode(string data, int dontcheckcrc)
-**! 	Splits a PNG file into chunks.
-**!
-**!     Result is an array of arrays,
-**!	<tt>({ ({ string chunk_type, string data, int crc_ok }), 
-**!            ({ string chunk_type, string data, int crc_ok }) ... })</tt>
-**!
-**!	<tt>chunk_type</tt> is the type of the chunk, like
-**!	<tt>"IHDR"</tt> or <tt>"IDAT"</tt>.
-**!
-**!	<tt>data</tt> is the actual chunk data.
-**!	
-**!	<tt>crcok</tt> is set to 1 if the checksum is ok and
-**!	<tt>dontcheckcrc</tt> parameter isn't set.
-**!
-**!	Returns 0 if it isn't a PNG file.
-**!
-**! note
-**!	Please read about the PNG file format.
-*/
+/*! @decl array __decode(string data)
+ *! @decl array __decode(string data, int dontcheckcrc)
+ *! 	Splits a PNG file into chunks.
+ *!
+ *! @returns
+ *!   Result is an array of arrays, or 0 if data isn't a PNG file.
+ *!   Each element in the array is constructed as follows.
+ *!   @array
+ *!     @elem string 0
+ *!       The type of the chunk, e.g. "IHDR" or "IDAT".
+ *!     @elem string 1
+ *!       The actual chunk data.
+ *!     @elem int(0..1) 2
+ *!       Set to 1 if the checksum is ok and @[dontcheckcrc]
+ *!       isn't set.
+ *!   @endarray
+ *!
+ *! @note
+ *!   Please read about the PNG file format.
+ *!   Support for decoding cHRM, gAMA, sBIT, hIST, pHYs, tIME,
+ *!   tEXt and zTXt chunks are missing.
+ */
 
 static void image_png___decode(INT32 args)
 {
@@ -215,12 +214,12 @@ static void image_png___decode(INT32 args)
    int n=0;
    ONERROR uwp;
 
-   if (args<1) 
-      Pike_error("Image.PNG.__decode: too few arguments\n");
+   if (args<1)
+     SIMPLE_TOO_FEW_ARGS_ERROR("Image.PNG.__decode", 1);
    if (sp[-args].type!=T_STRING)
-      Pike_error("Image.PNG.__decode: illegal argument 1\n");
-   
-   if (args==2 &&
+     SIMPLE_BAD_ARG_ERROR("Image.PNG.__decode", 1, "string");
+
+   if (args>1 &&
        (sp[1-args].type!=T_INT ||
 	sp[1-args].u.integer!=0))
       nocrc=1;
@@ -283,99 +282,110 @@ static void image_png___decode(INT32 args)
    f_aggregate(n);
 }
 
+/*! @decl mapping _decode(string|array data)
+ *! @decl mapping _decode(string|array data, mapping options)
+ *!   Decode a PNG image file.
+ *!
+ *! @param options
+ *!    @mapping
+ *!      @member string|array|Image.Colortable "colortable"
+ *!        A replacement color table to be used instead of the one
+ *!        in the PNG file, if any.
+ *!    @endmapping
+ *!
+ *! @returns
+ *!   @mapping
+ *!     @member Image.Image "image"
+ *!       The decoded image.
+ *!     @member int "bpp"
+ *!       Number of bitplanes in the image. One of 1, 2, 4, 8 and 16.
+ *!     @member int "type"
+ *!       Image color type. Bit values are:
+ *!       @int
+ *!         @value 1
+ *!           Palette used.
+ *!         @value 2
+ *!           Color used.
+ *!         @value 4
+ *!           Alpha channel used.
+ *!       @endint
+ *!       Valid values are 0, 2, 3, 4 and 6.
+ *!     @member int "xsize"
+ *!     @member int "ysize"
+ *!       Image dimensions.
+ *!     @member array(int) "background"
+ *!       The background color, if any. An array of size three with
+ *!       the RGB values.
+ *!     @member Image.Image "alpha"
+ *!       The alpha channel, if any.
+ *!   @endmapping
+ *!
+ *! @throws
+ *!   Throws an error if the image data is erroneous.
+ *!
+ *! @note
+ *!	Please read about the PNG file format.
+ *!	This function ignores any checksum errors in the file.
+ *!	A PNG of higher color resolution than the Image module
+ *!	supports (8 bit) will lose that information in the conversion.
+ */
+
 /*
-**! method array _decode(string|array data)
-**! method array _decode(string|array data,mapping options)
-**! 	Decode a PNG image file.
-**!
-**!     Result is a mapping,
-**!	<pre>
-**!	([
-**!	   "image": object image,
-**!
-**!        ... options ...
-**!     ])
-**!	</pre>
-**!
-**!	<tt>image</tt> is the stored image.
-**!
-**!	Valid entries in <tt>options</tt> is a superset
-**!	of the one given to <ref>encode</ref>:
-**!
-**!	<pre>
-**!     basic options:
-**!
-**!	    "alpha": object alpha,            - alpha channel
-**!
-**!	    "palette": object colortable,     - image palette
-**!                                             (if non-truecolor)
-**!         
-**!     advanced options:
-**! 
-**!	    "background": array(int) color,   - suggested background color
-**!	    "background_index": int index,    - what index in colortable
-**!
-**!	    "chroma": ({ float white_point_x,
-**!	                 float white_point_y,
-**!			 float red_x,
-**!			 float red_y,         - CIE x,y chromaticities
-**!			 float green_x,         
-**!			 float green_y,
-**!			 float blue_x,
-**!			 float blue_y })  
-**!
-**!	    "gamma":  float gamma,            - gamma
-**!
-**!	    "spalette": object colortable,    - suggested palette, 
-**!                                             for truecolor images
-**!	    "histogram": array(int) hist,     - histogram for the image,
-**!	                                        corresponds to palette index
-**!	
-**!	    "physical": ({ int unit,          - physical pixel dimension
-**!	                   int x,y })           unit 0 means pixels/meter
-**!
-**!	    "sbit": array(int) sbits          - significant bits
-**!
-**!	    "text": array(array(string)) text - text information, 
-**!                 ({ ({ keyword, data }), ... })
-**!
-**!                 Standard keywords:
-**!
-**!                 Title          Short (one line) title or caption for image
-**!                 Author         Name of image's creator
-**!                 Description    Description of image (possibly long)
-**!                 Copyright      Copyright notice
-**!                 Creation Time  Time of original image creation
-**!                 Software       Software used to create the image
-**!                 Disclaimer     Legal disclaimer
-**!                 Warning        Warning of nature of content
-**!                 Source         Device used to create the image
-**!                 Comment        Miscellaneous comment
-**!
-**!	    "time": ({ int year, month, day,  - time of last modification
-**!	               hour, minute, second })  
-**!
-**!      wizard options:
-**!	    "compression": int method         - compression method (0)
-**!
-**!      </pre>
-**!	
-**!	This method can also take options, 
-**! 	as a mapping:
-**!	<pre>
-**!     advanced options:
-**!	    "palette": colortable object
-**!		- replace the decoded palette with this when
-**!		  unpacking the image data, if applicable
-**!	</pre>
-**!
-**! note
-**!	Please read about the PNG file format.
-**!	This function ignores any checksum errors in the file.
-**!	A PNG of higher color resolution than the Image module
-**!	supports (8 bit) will lose that information in the conversion.
-**!	It throws an error if the image data is erroneous.
-*/
+ *     basic options:
+ *
+ *	    "alpha": object alpha,            - alpha channel
+ *
+ *	    "palette": object colortable,     - image palette
+ *                                             (if non-truecolor)
+ *         
+ *     advanced options:
+ * 
+ *	    "background": array(int) color,   - suggested background color
+ *	    "background_index": int index,    - what index in colortable
+ *
+ *	    "chroma": ({ float white_point_x,
+ *	                 float white_point_y,
+ *			 float red_x,
+ *			 float red_y,         - CIE x,y chromaticities
+ *			 float green_x,         
+ *			 float green_y,
+ *			 float blue_x,
+ *			 float blue_y })  
+ *
+ *	    "gamma":  float gamma,            - gamma
+ *
+ *	    "spalette": object colortable,    - suggested palette, 
+ *                                             for truecolor images
+ *	    "histogram": array(int) hist,     - histogram for the image,
+ *	                                        corresponds to palette index
+ *	
+ *	    "physical": ({ int unit,          - physical pixel dimension
+ *	                   int x,y })           unit 0 means pixels/meter
+ *
+ *	    "sbit": array(int) sbits          - significant bits
+ *
+ *	    "text": array(array(string)) text - text information, 
+ *                 ({ ({ keyword, data }), ... })
+ *
+ *                 Standard keywords:
+ *
+ *                 Title          Short (one line) title or caption for image
+ *                 Author         Name of image's creator
+ *                 Description    Description of image (possibly long)
+ *                 Copyright      Copyright notice
+ *                 Creation Time  Time of original image creation
+ *                 Software       Software used to create the image
+ *                 Disclaimer     Legal disclaimer
+ *                 Warning        Warning of nature of content
+ *                 Source         Device used to create the image
+ *                 Comment        Miscellaneous comment
+ *
+ *	    "time": ({ int year, month, day,  - time of last modification
+ *	               hour, minute, second })  
+ *
+ *      wizard options:
+ *	    "compression": int method         - compression method (0)
+ */
 
 static struct pike_string *_png_unfilter(unsigned char *data,
 					 size_t len,
@@ -390,12 +400,15 @@ static struct pike_string *_png_unfilter(unsigned char *data,
    int x;
    int sbb;
 
-   switch (type)
+   if(filter!=0)
+     Pike_error("Unknown filter type %d.\n", filter);
+
+   switch (type) /* Each pixel is ... */
    {
-      case 2: x=3; break;
-      case 4: x=2; break;
-      case 6: x=4; break;
-      default: x=1; 
+      case 2: x=3; break; /* an R,G,B triple */
+      case 4: x=2; break; /* a grayscale sample, followed by an alpha sample */
+      case 6: x=4; break; /* an R,G,B triple, followed by an alpha sample */
+      default: x=1;       /* a palette index / a grayscale sample */
    }
 
    bpp*=x; /* multipy for units/pixel (rgba=4, grey=1, etc) */
@@ -462,7 +475,7 @@ static struct pike_string *_png_unfilter(unsigned char *data,
 	       a=(a+b)>>1;
 
 	       *d=*s+a;
-       
+
 	       d++;
 	       s++;
 	    }
@@ -505,7 +518,7 @@ static struct pike_string *_png_unfilter(unsigned char *data,
 	    
 	    break;
 	 default:
-	    Pike_error("Image.PNG._decode: unsupported filter %d\n",s[-1]);
+	    Pike_error("Unsupported subfilter %d (filter %d)\n", s[-1],type);
       }
    }
 }
@@ -524,7 +537,8 @@ static int _png_write_rgb(rgb_group *w1,
    /* w1, wa1 will be freed upon error */
 
    static rgb_group white={255,255,255};
-   static rgb_group grey4[4]={{0,0,0},{85,85,85},{170,170,170},{255,255,255}};
+   static rgb_group grey4[4]={{0,0,0},{85,85,85},
+			      {170,170,170},{255,255,255}};
    static rgb_group black={0,0,0};
 
    rgb_group *d1=w1;
@@ -543,9 +557,8 @@ static int _png_write_rgb(rgb_group *w1,
 	 switch (bpp)
 	 {
 	    case 1:
-	       if (n>len*8) n=len*8;
 	       x=width;
-	       while (n)
+	       while (len--)
 	       {
 		  if (x) x--,*(d1++)=((*s)&128)?white:black;
 		  if (x) x--,*(d1++)=((*s)&64)?white:black;
@@ -555,25 +568,21 @@ static int _png_write_rgb(rgb_group *w1,
 		  if (x) x--,*(d1++)=((*s)&4)?white:black;
 		  if (x) x--,*(d1++)=((*s)&2)?white:black;
 		  if (x) x--,*(d1++)=((*s)&1)?white:black;
-		  if (n<8) break;
-		  n-=8;
-		  s++;
 		  if (!x) x=width;
+		  s++;
 	       }
 	       break;
 	    case 2:
-	       if (n>len*4) n=len*4;
 	       x=width;
-	       while (n)
+	       if(len>(n/4)) len=n/4;
+	       while (len--)
 	       {
 		  if (x) x--,*(d1++)=grey4[((*s)>>6)&3];
 		  if (x) x--,*(d1++)=grey4[((*s)>>4)&3];
 		  if (x) x--,*(d1++)=grey4[((*s)>>2)&3];
 		  if (x) x--,*(d1++)=grey4[(*s)&3];
-		  if (n<4) break;
-		  n-=4;
-		  s++;
 		  if (!x) x=width;
+		  s++;
 	       }
 	       break;
 	    case 4:
@@ -616,7 +625,7 @@ static int _png_write_rgb(rgb_group *w1,
 	       break;
 	    default:
 	       free(wa1); free(w1);
-	       Pike_error("Image.PNG->_decode: Unsupported color type/bit depth %d (grey)/%d bit.\n",
+	       Pike_error("Image.PNG._decode: Unsupported color type/bit depth %d (grey)/%d bit.\n",
 		     type,bpp);
 	 }
 	 if (trns && trns->len==2)
@@ -647,17 +656,19 @@ static int _png_write_rgb(rgb_group *w1,
 	       if (n>len/6) n=len/6;
 	       while (n)
 	       {
-		  d1->r=*(s++);
-		  d1->g=*(s++);
-		  d1->b=*(s++);
+		  d1->r=*s;
+		  s += 2;
+		  d1->g=*s;
+		  s += 2;
+		  d1->b=*s;
+		  s += 2;
 		  d1++;
-		  s++;
 		  n--;
 	       }
 	       break;
 	    default:
 	       free(wa1); free(w1);
-	       Pike_error("Image.PNG->_decode: Unsupported color type/bit depth %d (rgb)/%d bit.\n",
+	       Pike_error("Image.PNG._decode: Unsupported color type/bit depth %d (rgb)/%d bit.\n",
 		     type,bpp);
 	 }
 	 if (trns && trns->len==6)
@@ -673,25 +684,25 @@ static int _png_write_rgb(rgb_group *w1,
 	 }
 	 return 0; /* no alpha channel */
 
-      case 3: /* 1,2,4,8 bit palette index */
+      case 3: /* 1,2,4,8 bit palette index. Alpha might be in palette */
 	 if (!ct)
 	 {
 	    free(w1);
 	    free(wa1);
-	    Pike_error("Image.PNG->decode: No palette (PLTE entry), but color type (3) needs one\n");
+	    Pike_error("Image.PNG.decode: No palette (PLTE entry), but color type (3) needs one\n");
 	 }
 	 if (ct->type!=NCT_FLAT)
 	 {
 	    free(w1);
 	    free(wa1);
-	    Pike_error("Image.PNG->decode: Internal error (created palette isn't flat)\n");
+	    Pike_error("Image.PNG.decode: Internal error (created palette isn't flat)\n");
 	 }
 	 mz=ct->u.flat.numentries;
 	 if (mz==0)
 	 {
 	    free(w1);
 	    free(wa1);
-	    Pike_error("Image.PNG->decode: palette is zero entries long; need at least one color.\n");
+	    Pike_error("Image.PNG.decode: palette is zero entries long; need at least one color.\n");
 	 }
 
 #define CUTPLTE(X,Z) (((X)>=(Z))?0:(X))
@@ -800,7 +811,7 @@ static int _png_write_rgb(rgb_group *w1,
 		  while (n)
 		  {
 		     int i;
-		     for (i=8; i>4;)
+		     for (i=8; i>=4;)
 		     {
 			i-=4;
 			if (x) 
@@ -858,7 +869,7 @@ static int _png_write_rgb(rgb_group *w1,
 	       
 	    default:
 	       free(w1); free(wa1);
-	       Pike_error("Image.PNG->_decode: Unsupported color type/bit depth %d (palette)/%d bit.\n",
+	       Pike_error("Image.PNG._decode: Unsupported color type/bit depth %d (palette)/%d bit.\n",
 		     type,bpp);
 	 }
 	 return !!trns; /* alpha channel if trns chunk */
@@ -867,7 +878,7 @@ static int _png_write_rgb(rgb_group *w1,
 	 switch (bpp)
 	 {
 	    case 8:
-	       if (n>len/3) n=len/3;
+	       if (n>len/2) n=len/2;
 	       while (n)
 	       {
 		  d1->r=d1->g=d1->b=*(s++);
@@ -878,21 +889,21 @@ static int _png_write_rgb(rgb_group *w1,
 	       }
 	       break;
 	    case 16:
-	       if (n>len/6) n=len/6;
+	       if (n>len/4) n=len/4;
 	       while (n)
 	       {
-		  d1->r=d1->g=d1->b=*(s++);
+		  d1->r=d1->g=d1->b=*s;
+		  s += 2;
 		  d1++;
-		  s++;
-		  da1->r=da1->g=da1->b=*(s++);
-		  s++;
+		  da1->r=da1->g=da1->b=*s;
+		  s += 2;
 		  da1++;
 		  n--;
 	       }
 	       break;
 	    default:
 	       free(wa1); free(w1);
-	       Pike_error("Image.PNG->_decode: Unsupported color type/bit depth %d (grey+a)/%d bit.\n",
+	       Pike_error("Image.PNG._decode: Unsupported color type/bit depth %d (grey+a)/%d bit.\n",
 		     type,bpp);
 	 }
 	 return 1; /* alpha channel */
@@ -917,29 +928,31 @@ static int _png_write_rgb(rgb_group *w1,
 	       if (n>len/8) n=len/8;
 	       while (n)
 	       {
-		  d1->r=*(s++);
-		  d1->g=*(s++);
-		  d1->b=*(s++);
+		  d1->r=*s;
+		  s += 2;
+		  d1->g=*s;
+		  s += 2;
+		  d1->b=*s;
+		  s += 2;
 		  d1++;
-		  s++;
-		  da1->r=da1->g=da1->b=*(s++);
-		  s++;
+		  da1->r=da1->g=da1->b=*s;
+		  s += 2;
 		  da1++;
 		  n--;
 	       }
 	       break;
 	    default:
 	       free(wa1); free(w1);
-	       Pike_error("Image.PNG->_decode: Unsupported color type/bit depth %d(rgba)/%d bit.\n",
+	       Pike_error("Image.PNG._decode: Unsupported color type/bit depth %d(rgba)/%d bit.\n",
 		     type,bpp);
 	 }
 	 return 1; /* alpha channel */
       default:
 	 free(wa1); free(w1);
-	 Pike_error("Image.PNG->_decode: Unknown color type %d (bit depth %d).\n",
+	 Pike_error("Image.PNG._decode: Unknown color type %d (bit depth %d).\n",
 	       type,bpp);
    }
-   Pike_error("Image.PNG->_decode: illegal state\n");
+   Pike_error("Image.PNG._decode: illegal state\n");
    return 0; /* stupid */
 }
 
@@ -978,8 +991,8 @@ static void img_png_decode(INT32 args,int header_only)
       int interlace;
    } ihdr={-1,-1,-1,0,-1,-1,-1};
 
-   if (args<1) 
-      Pike_error("Image.PNG._decode: too few arguments\n");
+   if (args<1)
+     SIMPLE_TOO_FEW_ARGS_ERROR("Image.PNG._decode", 1);
 
    m=allocate_mapping(10);
    push_mapping(m);
@@ -987,7 +1000,7 @@ static void img_png_decode(INT32 args,int header_only)
    if (args>=2)
    {
       if (sp[1-args-1].type!=T_MAPPING)
-	 Pike_error("Image.PNG._decode: illegal argument 2\n");
+	SIMPLE_BAD_ARG_ERROR("Image.PNG._decode", 2, "mapping");
 
       push_svalue(sp+1-args-1);
       ref_push_string(param_palette);
@@ -995,11 +1008,11 @@ static void img_png_decode(INT32 args,int header_only)
       switch (sp[-1].type)
       {
 	 case T_OBJECT:
-	    push_string(make_shared_string("cast"));
+	    push_text("cast");
 	    if (sp[-1].type==T_INT)
 	       Pike_error("Image.PNG._decode: illegal value of option \"palette\"\n");
 	    f_index(2);
-	    push_string(make_shared_string("array"));
+	    push_text("array");
 	    f_call_function(2);
 	 case T_ARRAY:
 	 case T_STRING:
@@ -1015,6 +1028,7 @@ static void img_png_decode(INT32 args,int header_only)
 	    break;
 	 case T_INT:
 	    pop_n_elems(1);
+	    break;
 	 default:
 	    Pike_error("Image.PNG._decode: illegal value of option \"palette\"\n");
       }
@@ -1033,7 +1047,7 @@ static void img_png_decode(INT32 args,int header_only)
 	 Pike_error("Image.PNG._decode: Not PNG data\n");
    }
    else if (sp[-1].type!=T_ARRAY)
-      Pike_error("Image.PNG._decode: Illegal argument 1\n");
+     SIMPLE_BAD_ARG_ERROR("Image.PNG._decode", 1, "string");
 
    a=sp[-1].u.array;
 
@@ -1123,6 +1137,12 @@ static void img_png_decode(INT32 args,int header_only)
          case 0x67414d41: /* gAMA */
 	    break;
 
+         case 0x73524742: /* sRGB */
+	   break;
+
+         case 0x69434350: /* iCCP */
+	   break;
+
          case 0x73424954: /* sBIT */
 	    break;
 
@@ -1194,6 +1214,28 @@ static void img_png_decode(INT32 args,int header_only)
 
          case 0x7a545874: /* zTXt */
 	    break;
+
+         case 0x69545874: /* iTXt */
+	   break;
+
+         /* Extensions */
+         case 0x6f464673: /* oFFs */
+	   break;
+
+         case 0x7043414c: /* pCAL */
+	   break;
+
+         case 0x7343414c: /* sCAL */
+	   break;
+
+         case 0x67494667: /* gIFg */
+	   break;
+
+         case 0x67494678: /* gIFg */
+	   break;
+
+         case 0x66524163: /* fRAc */
+	   break;
       }
    }
 
@@ -1203,7 +1245,7 @@ static void img_png_decode(INT32 args,int header_only)
 
    /* IDAT stuff on stack, now */
    if (!n) 
-      push_string(make_shared_binary_string("",0));
+      push_text("");
    else
       f_add(n);
 
@@ -1215,18 +1257,18 @@ static void img_png_decode(INT32 args,int header_only)
    {
       Pike_error("Image.PNG._decode: missing palette (PLTE chunk)\n");
    }
-   
+
    if (ihdr.compression==0)
    {
       png_decompress(ihdr.compression);
       if (sp[-1].type!=T_STRING)
-	 Pike_error("Image.PNG._decode: got wierd stuff from decompression\n");
+	 Pike_error("Image.PNG._decode: got weird stuff from decompression\n");
    }
    else
       Pike_error("Image.PNG._decode: illegal compression type 0x%02x\n",
 	    ihdr.compression);
 
-   fs=sp[-1].u.string; 
+   fs=sp[-1].u.string;
    push_int(-1);
    mapping_insert(m,sp-1,sp-2);
 
@@ -1278,8 +1320,8 @@ static void img_png_decode(INT32 args,int header_only)
 	    free(w1); 
 	    if (wa1) free(wa1); 
 	    if (ta1) free(ta1); 
-	    if (ta1) free(t1); 
-	    Pike_error("Image.PNG->_decode: out of memory (close one)\n");
+	    if (t1) free(t1);
+	    Pike_error("Image.PNG._decode: out of memory (close one)\n");
 	 }
 	 /* loop over adam7 interlace's 
 	    and write them to the arena */
@@ -1288,39 +1330,40 @@ static void img_png_decode(INT32 args,int header_only)
 	 for (i=0; i<7; i++)
 	 {
 	    struct pike_string *ds;
+	    unsigned int x0 = adam7[i].x0;
+	    unsigned int xd = adam7[i].xd;
+	    unsigned int y0 = adam7[i].y0;
+	    unsigned int yd = adam7[i].yd;
+	    unsigned int iwidth = (ihdr.width+xd-1-x0)/xd;
+	    unsigned int iheight = (ihdr.height+yd-1-y0)/yd;
+
+	    if(!iwidth || !iheight) continue;
 
 	    ds=_png_unfilter(s0,fs->len-(s0-(unsigned char*)fs->str),
-			     (ihdr.width+adam7[i].xd-1-adam7[i].x0)/
-			     adam7[i].xd,			     
-			     (ihdr.height+adam7[i].yd-1-adam7[i].y0)/
-			     adam7[i].yd,
+			     iwidth, iheight,
 			     ihdr.filter,ihdr.type,ihdr.bpp,
 			     &s0);
 
 	    push_string(ds);
 	    if (!_png_write_rgb(w1,wa1,ihdr.type,ihdr.bpp,
 				(unsigned char*)ds->str,ds->len,
-				(ihdr.width+adam7[i].xd-1-adam7[i].x0)/
-				adam7[i].xd,
-				(ihdr.width+adam7[i].xd-1-adam7[i].x0)/
-				adam7[i].xd*
-				(ihdr.height+adam7[i].yd-1-adam7[i].y0)/
-				adam7[i].yd,
+				iwidth,
+				iwidth*iheight,
 				ct,trns))
 	    {
 	       if (wa1) free(wa1);
 	       wa1=NULL;
 	    }
 	    d1=w1;
-	    for (y=adam7[i].y0;y<ihdr.height;y+=adam7[i].yd)
-	       for (x=adam7[i].x0;x<ihdr.width;x+=adam7[i].xd)
+	    for (y=y0; y<ihdr.height; y+=yd)
+	       for (x=x0; x<ihdr.width; x+=xd)
 		  t1[x+y*ihdr.width]=*(d1++);
 
 	    if (wa1)
 	    {
 	       da1=wa1;
-	       for (y=adam7[i].y0;y<ihdr.height;y+=adam7[i].yd)
-		  for (x=adam7[i].x0;x<ihdr.width;x+=adam7[i].xd)
+	       for (y=y0; y<ihdr.height; y+=yd)
+		  for (x=x0; x<ihdr.width; x+=xd)
 		     ta1[x+y*ihdr.width]=*(da1++);
 	    }
 
@@ -1376,11 +1419,11 @@ header_stuff:
    mapping_insert(m,sp-2,sp-1);
    pop_n_elems(2);
 
-   push_string(make_shared_string("xsize"));
+   push_constant_text("xsize");
    push_int(ihdr.width);
    mapping_insert(m,sp-2,sp-1);
    pop_n_elems(2);
-   push_string(make_shared_string("ysize"));
+   push_constant_text("ysize");
    push_int(ihdr.height);
    mapping_insert(m,sp-2,sp-1);
    pop_n_elems(2);
@@ -1396,31 +1439,27 @@ header_stuff:
 }
 
 
-/*
-**! method string encode(object image)
-**! method string encode(object image, mapping options)
-**! 	Encodes a PNG image. 
-**!
-**!     The <tt>options</tt> argument may be a mapping
-**!	containing zero or more encoding options:
-**!
-**!	<pre>
-**!	normal options:
-**!	    "alpha":image object
-**!		Use this image as alpha channel 
-**!		(Note: PNG alpha channel is grey.
-**!		 The values are calculated by (r+2g+b)/4.)
-**!
-**!	    "palette":colortable object
-**!		Use this as palette for pseudocolor encoding
-**!		(Note: encoding with alpha channel and pseudocolor
-**!		 at the same time are not supported)
-**!
-**!	</pre>
-**!
-**! note
-**!	Please read some about PNG files. 
-*/
+/*! @decl string encode(Image.Image image)
+ *! @decl string encode(Image.Image image, mapping options)
+ *! 	Encodes a PNG image. 
+ *!
+ *! @param options
+ *!   @mapping
+ *!     @member Image.Image "alpha"
+ *!       Use this image as alpha channel (Note: PNG alpha
+ *!       channel is grey. The values are calculated by (r+2g+b)/4.)
+ *!     @member Image.Colortable "palette"
+ *!       Use this as palette for pseudocolor encoding
+ *!       (Note: encoding with alpha channel and pseudocolor
+ *!       at the same time are not supported)
+ *!   @endmapping
+ *!
+ *! @seealso
+ *!   @[__decode]
+ *!
+ *! @note
+ *!	Please read some about PNG files. 
+ */
 
 static void image_png_encode(INT32 args)
 {
@@ -1432,37 +1471,42 @@ static void image_png_encode(INT32 args)
    char buf[20];
    
    if (!args)
-      Pike_error("Image.PNG.encode: too few arguments\n");
-   
+     SIMPLE_TOO_FEW_ARGS_ERROR("Image.PNG.encode", 1);
+
    if (sp[-args].type!=T_OBJECT ||
        !(img=(struct image*)
 	 get_storage(sp[-args].u.object,image_program)))
-      Pike_error("Image.PNG.encode: illegal argument 1\n");
-   
+     SIMPLE_BAD_ARG_ERROR("Image.PNG.encode", 1, "Image.Image");
+
    if (!img->img)
       Pike_error("Image.PNG.encode: no image\n");
 
    if (args>1)
    {
       if (sp[1-args].type!=T_MAPPING)
-	 Pike_error("Image.PNG.encode: illegal argument 2\n");
-      
+	SIMPLE_BAD_ARG_ERROR("Image.PNG.encode", 2, "mapping");
+
       push_svalue(sp+1-args);
       ref_push_string(param_alpha);
       f_index(2);
-      if ( !IS_ZERO( sp - 1 ) )
+      if ( !UNSAFE_IS_ZERO( sp - 1 ) )
 	 if (sp[-1].type!=T_OBJECT ||
 	     !(alpha=(struct image*)
 	       get_storage(sp[-1].u.object,image_program)))
-	    Pike_error("Image.PNG.encode: option (arg 2) \"alpha\" has illegal type\n");
+	    PIKE_ERROR("Image.PNG.encode",
+		       "Option (arg 2) \"alpha\" has illegal type.\n",
+		       sp, args);
       pop_stack();
 
       if (alpha &&
 	  (alpha->xsize!=img->xsize ||
 	   alpha->ysize!=img->ysize))
-	 Pike_error("Image.PNG.encode option (arg 2) \"alpha\"; images differ in size\n");
+	 PIKE_ERROR("Image.PNG.encode",
+		    "Option (arg 2) \"alpha\"; images differ in size.\n",
+		    sp, args);
       if (alpha && !alpha->img)
-	 Pike_error("Image.PNG.encode option (arg 2) \"alpha\"; no image\n");
+	PIKE_ERROR("Image.PNG.encode", "Option (arg 2) \"alpha\"; no image\n",
+		   sp, args);
 
       push_svalue(sp+1-args);
       ref_push_string(param_palette); 
@@ -1472,9 +1516,10 @@ static void image_png_encode(INT32 args)
 	 if (sp[-1].type!=T_OBJECT ||
 	     !(ct=(struct neo_colortable*)
 	       get_storage(sp[-1].u.object,image_colortable_program)))
-	    Pike_error("Image.PNG.encode: option (arg 2) \"palette\" has illegal type\n");
+	   PIKE_ERROR("Image.PNG.encode",
+		      "Option (arg 2) \"palette\" has illegal type.\n",
+		      sp, args);
       pop_stack();
-
    }
    
    sprintf(buf,"%c%c%c%c%c%c%c%c",
@@ -1487,8 +1532,9 @@ static void image_png_encode(INT32 args)
       ptrdiff_t sz;
       sz = image_colortable_size(ct);
       if (sz>256)
-	 Pike_error("Image.PNG.encode: palette size to large; "
-	       "PNG doesn't support bigger palettes then 256 colors\n");
+	 PIKE_ERROR("Image.PNG.encode", "Palette size to large; "
+		    "PNG doesn't support bigger palettes then 256 colors.\n",
+		    sp, args);
       if (sz>16) bpp=8;
       else if (sz>4) bpp=4;
       else if (sz>2) bpp=2;
@@ -1498,10 +1544,10 @@ static void image_png_encode(INT32 args)
       bpp=8;
 
    sprintf(buf,"%c%c%c%c%c%c%c%c%c%c%c%c%c",
-	   (img->xsize>>24)&255,(img->xsize>>16)&255,
-	   (img->xsize>>8)&255,(img->xsize)&255,
-	   (img->ysize>>24)&255,(img->ysize>>16)&255,
-	   (img->ysize>>8)&255,(img->ysize)&255,
+	   (char)((img->xsize>>24)&255),(char)((img->xsize>>16)&255),
+	   (char)((img->xsize>>8)&255), (char)((img->xsize)&255),
+	   (char)((img->ysize>>24)&255),(char)((img->ysize>>16)&255),
+	   (char)((img->ysize>>8)&255), (char)((img->ysize)&255),
 	   bpp /* bpp */,
 	   ct?3:(alpha?6:2) /* type (P/(RGBA/RGB)) */,
 	   0 /* compression */,
@@ -1526,15 +1572,18 @@ static void image_png_encode(INT32 args)
    y=img->ysize;
    s=img->img;
    if (alpha) sa=alpha->img;
-   if (ct)
-      if (alpha)
-	 Pike_error("Image.PNG.encode: colortable and alpha channel not supported at the same time\n");
+   if (ct) {
+      if (alpha) {
+	 PIKE_ERROR("Image.PNG.encode",
+		    "Colortable and alpha channel not supported "
+		    "at the same time.\n", sp, args);
+      }
       else
       {
 	 unsigned char *tmp=malloc(img->xsize*img->ysize),*ts;
 
 	 if (!tmp)
-	    Pike_error("Image.PNG.encode: out of memory\n");
+	    PIKE_ERROR("Image.PNG.encode", "Out of memory.\n", sp, args);
 	 image_colortable_index_8bit_image(ct,img->img,tmp,
 					   img->xsize*img->ysize,img->xsize);
 	 ts=tmp;
@@ -1567,6 +1616,7 @@ static void image_png_encode(INT32 args)
 	 }
 	 free(tmp);
       }
+   }
    else
       while (y--)
       {
@@ -1601,7 +1651,7 @@ static void image_png_encode(INT32 args)
    push_png_chunk("IDAT",NULL);
    n++;
 
-   push_string(make_shared_binary_string("",0));
+   push_text("");
    push_png_chunk("IEND",NULL);
    n++;
 
@@ -1617,54 +1667,63 @@ static void image_png__decode(INT32 args)
    img_png_decode(args,0);
 }
 
+/*! @decl mapping decode_header(string data)
+ *!   Decodes only the PNG headers.
+ *! @seealso
+ *!   @[_decode]
+ */
+
 static void image_png_decode_header(INT32 args)
 {
    img_png_decode(args,1);
 }
 
-/*
-**! method object decode(string data)
-**! method object decode(string data, mapping options)
-**! 	Decodes a PNG image. 
-**!
-**!     The <tt>options</tt> argument may be a mapping
-**!	containing zero or more encoding options:
-**!
-**!	<pre>
-**!	</pre>
-**!
-**! note
-**!	Throws upon error in data.
-*/
+/*! @decl Image.Image decode(string data)
+ *! @decl Image.Image decode(string data, mapping(string:mixed) options)
+ *!   Decodes a PNG image. The @[options] mapping is the
+ *!   as for @[_decode].
+ *!
+ *! @throws
+ *!   Throws upon error in data.
+ */
 
 static void image_png_decode(INT32 args)
 {
    if (!args)
-      Pike_error("Image.PNG.decode: missing argument(s)\n");
+     SIMPLE_TOO_FEW_ARGS_ERROR("Image.PNG.decode", 1);
    
    image_png__decode(args);
-   push_string(make_shared_string("image"));
+   push_constant_text("image");
    f_index(2);
 }
 
+/*! @decl Image.Image decode_alpha(string data, @
+ *!     void|mapping(string:mixed) options)
+ *!   Decodes the alpha channel in a PNG file. The
+ *!   @[options] mapping is the same as for @[_decode].
+ *!
+ *! @throws
+ *!   Throws upon error in data.
+ */
+
 static void image_png_decode_alpha(INT32 args)
 {
    struct svalue s;
    if (!args)
-      Pike_error("Image.PNG.decode: missing argument(s)\n");
-   
+     SIMPLE_TOO_FEW_ARGS_ERROR("Image.PNG.decode_alpha", 1);
+
    image_png__decode(args);
    assign_svalue_no_free(&s,sp-1);
-   push_string(make_shared_string("alpha"));
+   push_constant_text("alpha");
    f_index(2);
 
    if (sp[-1].type==T_INT)
    {
       push_svalue(&s);
-      push_string(make_shared_string("xsize"));
+      push_constant_text("xsize");
       f_index(2);
       push_svalue(&s);
-      ref_push_string(make_shared_string("ysize"));
+      push_constant_text("ysize");
       f_index(2);
       push_int(255);
       push_int(255);
@@ -1674,6 +1733,12 @@ static void image_png_decode_alpha(INT32 args)
    free_svalue(&s);
 }
 
+/*! @endmodule
+ */
+
+/*! @endmodule
+ */
+
 /*** module init & exit & stuff *****************************************/
 
 void exit_image_png(void)
@@ -1697,8 +1762,7 @@ void exit_image_png(void)
 void init_image_png(void)
 {
    push_text("Gz");
-   push_int(0);
-   SAFE_APPLY_MASTER("resolv",2);
+   SAFE_APPLY_MASTER("resolv",1);
    if (sp[-1].type==T_OBJECT) 
    {
      stack_dup();
