@@ -1,34 +1,36 @@
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: xcf.c,v 1.49 2005/05/18 12:36:54 mast Exp $
+*/
+
 #include "global.h"
-RCSID("$Id: xcf.c,v 1.35 2000/12/05 21:08:28 per Exp $");
+RCSID("$Id: xcf.c,v 1.49 2005/05/18 12:36:54 mast Exp $");
 
 #include "image_machine.h"
 
 #include "pike_macros.h"
 #include "object.h"
-#include "constants.h"
 #include "module_support.h"
 #include "interpret.h"
 #include "object.h"
 #include "svalue.h"
 #include "threads.h"
-#include "array.h"
 #include "interpret.h"
 #include "svalue.h"
 #include "mapping.h"
 #include "pike_error.h"
 #include "stralloc.h"
 #include "builtin_functions.h"
-#include "operators.h"
-#include "dynamic_buffer.h"
-#include "signal_handler.h"
 #include "bignum.h"
 
 #include "image.h"
 #include "colortable.h"
 
-/* MUST BE INCLUDED LAST */
-#include "module_magic.h"
 
+#define sp Pike_sp
+#define fp Pike_fp
 
 extern struct program *image_colortable_program;
 extern struct program *image_program;
@@ -77,14 +79,14 @@ static void f_substring_cast( INT32 args )
 
 static void f_substring_index( INT32 args )
 {
-  int i = sp[-1].u.integer;
+  ptrdiff_t i = sp[-1].u.integer;
   struct substring *s = SS(fp->current_object);
   pop_n_elems( args );
 
   if( i < 0 ) i = s->len + i;
   if( i >= s->len ) {
-    Pike_error("Index out of bounds, %d > %ld\n", i,
-	  DO_NOT_WARN((long)s->len-1) );
+    Pike_error("Index out of bounds, %"PRINTPTRDIFFT"d > %"PRINTPTRDIFFT"d\n",
+	       i, s->len-1);
   }
   push_int( ((unsigned char *)s->s->str)[s->offset+i] );
 }
@@ -112,8 +114,8 @@ static void f_substring__sprintf( INT32 args )
      push_constant_text("SubString( %O /* [+%d .. %d] */ )" );
      push_text("string"); f_substring_cast( 1 );
 
-     push_int( s->len );
-     push_int( s->offset );
+     push_int64( s->len );
+     push_int64( s->offset );
      f_sprintf( 4 );
      return;
    default: 
@@ -129,9 +131,9 @@ static void f_substring_get_int( INT32 args )
   unsigned char *p;
   int x = sp[-1].u.integer;
   if( x > s->len>>2 )
-    Pike_error("Index %d out of range", x );
+    Pike_error("Index %d out of range.\n", x );
 
-  p = s->s->str + s->offset + x*4;
+  p = ((unsigned char *)s->s->str) + s->offset + x*4;
   res = (p[0]<<24) | (p[1]<<16) | (p[2]<<8) | p[3];
   push_int( res );
 }
@@ -144,9 +146,9 @@ static void f_substring_get_uint( INT32 args )
   unsigned char *p;
   int x = sp[-1].u.integer;
   if( x > s->len>>2 )
-    Pike_error("Index %d out of range", x );
+    Pike_error("Index %d out of range.\n", x );
 
-  p = s->s->str + s->offset + x*4;
+  p = ((unsigned char *)s->s->str) + s->offset + x*4;
   res = (p[0]<<24) | (p[1]<<16) | (p[2]<<8) | p[3];
   push_int64( res );
 }
@@ -158,9 +160,9 @@ static void f_substring_get_ushort( INT32 args )
   unsigned char *p;
   int x = sp[-1].u.integer;
   if( x > s->len>>1 )
-    Pike_error("Index %d out of range", x );
+    Pike_error("Index %d out of range.\n", x );
 
-  p = s->s->str + s->offset + x*2;
+  p = ((unsigned char *)s->s->str) + s->offset + x*2;
   res = (p[2]<<8) | p[3];
   push_int( res );
 }
@@ -172,9 +174,9 @@ static void f_substring_get_short( INT32 args )
   unsigned char *p;
   int x = sp[-1].u.integer;
   if( x > s->len>>1 )
-    Pike_error("Index %d out of range", x );
+    Pike_error("Index %d out of range.\n", x );
 
-  p = s->s->str + s->offset + x*2;
+  p = ((unsigned char *)s->s->str) + s->offset + x*2;
   res = (p[2]<<8) | p[3];
   push_int( res );
 }
@@ -340,7 +342,6 @@ static struct buffer read_string( struct buffer *data )
 
 static struct property read_property( struct buffer * data )
 {
-  int i;
   struct property res;
   res.type = read_uint( data );
   if(res.type == PROP_COLORMAP)
@@ -566,7 +567,6 @@ static struct layer_mask read_layer_mask( struct buffer *buff,
   ONERROR err;
   int offset;
   struct property tmp;
-  struct buffer ob;
 
   MEMSET(&res, 0, sizeof(res));
   res.width = read_uint( buff );
@@ -604,7 +604,6 @@ static struct channel read_channel( struct buffer *buff,
   ONERROR err;
   int offset;
   struct property tmp;
-  struct buffer ob;
 
   MEMSET(&res, 0, sizeof(res));
   res.width = read_uint( buff );
@@ -780,7 +779,7 @@ static void push_properties( struct property *p )
     f_aggregate_mapping( 4 );
     p = p->next;
   }
-  f_aggregate(DO_NOT_WARN(sp - osp));
+  f_aggregate(DO_NOT_WARN((INT32)(sp - osp)));
 }
 
 static void push_tile( struct tile *t )
@@ -807,8 +806,8 @@ static void push_hierarchy( struct hierarchy * h )
     push_tile( t );
     t=t->next;
   }
-  f_aggregate(DO_NOT_WARN(sp - tsp));
-  f_aggregate_mapping(DO_NOT_WARN(sp - osp));
+  f_aggregate(DO_NOT_WARN((INT32)(sp - tsp)));
+  f_aggregate_mapping(DO_NOT_WARN((INT32)(sp - osp)));
 }
 
 static void push_layer_mask(struct layer_mask *i)
@@ -821,7 +820,7 @@ static void push_layer_mask(struct layer_mask *i)
   ref_push_string( s_name );  push_buffer( &i->name );
   ref_push_string( s_image_data );
   push_hierarchy( &i->image_data );
-  f_aggregate_mapping(DO_NOT_WARN(sp-osp));
+  f_aggregate_mapping(DO_NOT_WARN((INT32)(sp-osp)));
 }
 
 static void push_channel(struct channel *i)
@@ -834,7 +833,7 @@ static void push_channel(struct channel *i)
   ref_push_string( s_name );  push_buffer( &i->name );
   ref_push_string( s_image_data );
   push_hierarchy( &i->image_data );
-  f_aggregate_mapping(DO_NOT_WARN(sp-osp));
+  f_aggregate_mapping(DO_NOT_WARN((INT32)(sp-osp)));
 }
 
 static void push_layer(struct layer *i)
@@ -853,7 +852,7 @@ static void push_layer(struct layer *i)
     ref_push_string( s_mask );
     push_layer_mask( i->mask );
   }
-  f_aggregate_mapping(DO_NOT_WARN(sp - osp));
+  f_aggregate_mapping(DO_NOT_WARN((INT32)(sp - osp)));
 }
 
 
@@ -888,7 +887,7 @@ static void push_image( struct gimp_image *i )
     c = c->next;
   }
   f_aggregate( nitems );
-  f_aggregate_mapping(DO_NOT_WARN(sp-osp));
+  f_aggregate_mapping(DO_NOT_WARN((INT32)(sp-osp)));
 }
 
 
@@ -932,6 +931,7 @@ static unsigned char read_char( struct buffer *from )
 
 /*
 **! method array(object) decode_layers( string data )
+**! method array(object) decode_layers( string data, mapping options )
 **!     Decodes a XCF image to an array of Image.Layer objects
 **!
 **!     The layer object have the following extra variables (to be queried
@@ -939,6 +939,9 @@ static unsigned char read_char( struct buffer *from )
 **!
 **!      image_xres, image_yres, image_colormap, image_guides, image_parasites,
 **!      name, parasites, visible, active
+**!
+**!	Takes the same argument mapping as <ref>_decode</ref>,
+**!	note especially "draw_all_layers":1.
 */
 
 /*
@@ -1182,14 +1185,14 @@ void image_xcf_f__decode_tiles( INT32 args )
   struct array *tiles;
   struct image *i=NULL, *a=NULL;
   struct neo_colortable *cmap = NULL;
-  int rxs, rys;
+  INT32 rxs, rys;
   rgb_group *colortable=NULL;
   rgb_group pix = {0,0,0};
   rgb_group apix= {255,255,255}; /* avoid may use uninitialized warnings */
 
   INT_TYPE rle, bpp, span, shrink;
   unsigned int l, x=0, y=0, cx, cy;
-  get_all_args( "_decode_tiles", args, "%o%O%a%i%i%O%d%d%d",
+  get_all_args( "_decode_tiles", args, "%o%O%a%i%i%O%i%d%d",
                 &io, &ao, &tiles, &rle, &bpp, &cmapo, &shrink, &rxs, &rys);
 
 
@@ -1232,7 +1235,7 @@ void image_xcf_f__decode_tiles( INT32 args )
     if(!tile_ss)
       continue;
 
-    tile.str = (tile_ss->s->str + tile_ss->offset);
+    tile.str = (unsigned char *)(tile_ss->s->str + tile_ss->offset);
     tile.len = tile_ss->len;
 
     ewidth = MINIMUM(TILE_WIDTH, (rxs-x));
@@ -1243,7 +1246,7 @@ void image_xcf_f__decode_tiles( INT32 args )
       struct buffer s = tile, od, d;
       int i;
       od.len = eheight*ewidth*bpp;  /* Max and only size, really */
-      df = od.str = (unsigned char *)xalloc( eheight*ewidth*bpp+1 );
+      df = (char *)(od.str = (unsigned char *)xalloc( eheight*ewidth*bpp+1 ));
       d = od;
 
       for(i=0; i<bpp; i++)
