@@ -1,4 +1,4 @@
-#line 1 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 1 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 /* -*- c -*-
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
@@ -47,8 +47,8 @@
 
 #undef class_FFT_defined
 #define class_FFT_defined
-struct program *FFT_program=0;
-int FFT_program_fun_num=-1;
+struct program *FFT_program=NULL;
+static int FFT_program_fun_num=-1;
 
 #undef var_r_in_FFT_defined
 #define var_r_in_FFT_defined
@@ -83,32 +83,32 @@ static ptrdiff_t FFT_storage_offset;
 struct FFT_struct {
 
 #ifdef var_r_in_FFT_defined
-#line 48 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 48 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 fftw_real *r_in;
 #endif /* var_r_in_FFT_defined */
 
 #ifdef var_r_out_FFT_defined
-#line 49 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 49 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 fftw_real *r_out;
 #endif /* var_r_out_FFT_defined */
 
 #ifdef var_r_cr_plan_FFT_defined
-#line 50 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 50 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 rfftw_plan r_cr_plan;
 #endif /* var_r_cr_plan_FFT_defined */
 
 #ifdef var_r_rc_plan_FFT_defined
-#line 51 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 51 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 rfftw_plan r_rc_plan;
 #endif /* var_r_rc_plan_FFT_defined */
 
 #ifdef var_r_cr_plan_size_FFT_defined
-#line 52 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 52 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 int r_cr_plan_size;
 #endif /* var_r_cr_plan_size_FFT_defined */
 
 #ifdef var_r_rc_plan_size_FFT_defined
-#line 53 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 53 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 int r_rc_plan_size;
 #endif /* var_r_rc_plan_size_FFT_defined */
 };
@@ -132,13 +132,13 @@ int r_rc_plan_size;
   #define f_FFT_rFFT_defined
 ptrdiff_t f_FFT_rFFT_fun_num = 0;
 void f_FFT_rFFT(INT32 args) {
-#line 72 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 72 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 struct array * myarray;
-#line 72 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 72 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 if(args != 1) wrong_number_of_args_error("rFFT",args,1);
-#line 72 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 72 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 if(Pike_sp[0-1].type != PIKE_T_ARRAY) SIMPLE_BAD_ARG_ERROR("rFFT",1,"array");
-#line 72 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 72 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 debug_malloc_pass(myarray=Pike_sp[0-1].u.array);
 {
       int i, j;
@@ -234,13 +234,13 @@ debug_malloc_pass(myarray=Pike_sp[0-1].u.array);
   #define f_FFT_rIFFT_defined
 ptrdiff_t f_FFT_rIFFT_fun_num = 0;
 void f_FFT_rIFFT(INT32 args) {
-#line 163 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 163 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 struct array * myarray;
-#line 163 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 163 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 if(args != 1) wrong_number_of_args_error("rIFFT",args,1);
-#line 163 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 163 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 if(Pike_sp[0-1].type != PIKE_T_ARRAY) SIMPLE_BAD_ARG_ERROR("rIFFT",1,"array");
-#line 163 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 163 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 debug_malloc_pass(myarray=Pike_sp[0-1].u.array);
 {
       int i, j=1;
@@ -324,21 +324,25 @@ debug_malloc_pass(myarray=Pike_sp[0-1].u.array);
   #define f_FFT_create_defined
 ptrdiff_t f_FFT_create_fun_num = 0;
 void f_FFT_create(INT32 args) {
-#line 242 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 242 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 struct svalue * no;
-#line 242 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 242 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 struct svalue * exact;
-#line 242 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 242 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 if(args > 2) wrong_number_of_args_error("create",args,2);
 if (args > 0) {
-#line 242 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 242 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
+if(Pike_sp[0-args].type != PIKE_T_INT) SIMPLE_BAD_ARG_ERROR("create",1,"void|int");
+#line 242 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 no=Pike_sp+0-args; dmalloc_touch_svalue(Pike_sp+0-args);
 } else no=0;
 if (args > 1) {
-#line 242 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 242 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
+if(Pike_sp[1-args].type != PIKE_T_INT) SIMPLE_BAD_ARG_ERROR("create",2,"void|int(0..1)");
+#line 242 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 exact=Pike_sp+1-args; dmalloc_touch_svalue(Pike_sp+1-args);
 } else exact=0;
-#line 245 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 245 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 {
     int t = FFTW_ESTIMATE;
     int n = 0;
@@ -374,7 +378,7 @@ exact=Pike_sp+1-args; dmalloc_touch_svalue(Pike_sp+1-args);
 #undef FFT_event_handler_defined
 #define FFT_event_handler_defined
 static void init_FFT_struct(void)
-#line 273 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 273 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 {
       THIS->r_in=NULL;
       THIS->r_out=NULL;
@@ -387,7 +391,7 @@ static void init_FFT_struct(void)
 #undef FFT_event_handler_defined
 #define FFT_event_handler_defined
 static void exit_FFT_struct(void)
-#line 279 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 279 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 {
       if (THIS->r_cr_plan_size>0)
 	rfftw_destroy_plan(THIS->r_cr_plan);
@@ -426,7 +430,7 @@ static void FFT_event_handler(int ev) {
 /*! @endmodule
  */
 
-#line 300 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 300 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 #endif /* HAVE_RFFTW */
 
 void init_math_transforms(void)
@@ -436,8 +440,10 @@ void init_math_transforms(void)
 #ifdef class_FFT_defined
 
 #ifdef PROG_FFT_ID
+#line 46 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
   START_NEW_PROGRAM_ID(FFT);
 #else
+#line 46 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
   start_new_program();
 
 #endif /* PROG_FFT_ID */
@@ -461,30 +467,32 @@ void init_math_transforms(void)
 #endif /* FFT_event_handler_defined */
 
 #ifdef f_FFT_rFFT_defined
-#line 72 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
   f_FFT_rFFT_fun_num =
+#line 72 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
     ADD_FUNCTION2("rFFT", f_FFT_rFFT, tFunc(tArray,tArray), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_FFT_rFFT_defined */
 
 #ifdef f_FFT_rIFFT_defined
-#line 163 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
   f_FFT_rIFFT_fun_num =
+#line 163 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
     ADD_FUNCTION2("rIFFT", f_FFT_rIFFT, tFunc(tArray,tArray), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_FFT_rIFFT_defined */
 
 #ifdef f_FFT_create_defined
-#line 242 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
   f_FFT_create_fun_num =
+#line 242 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
     ADD_FUNCTION2("create", f_FFT_create, tFunc(tOr(tVoid,"\10\200\0\0\0\177\377\377\377") tOr(tVoid,"\10\0\0\0\0\0\0\0\1"),tVoid), ID_STATIC, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_FFT_create_defined */
+#line 46 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
   FFT_program=end_program();
+#line 46 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
   FFT_program_fun_num=add_program_constant("FFT",FFT_program,0);
 
 #endif /* class_FFT_defined */
-#line 306 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 306 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 #endif /* HAVE_RFFTW */
 }
 
@@ -494,12 +502,13 @@ void exit_math_transforms(void)
   
 #ifdef class_FFT_defined
   if(FFT_program) {
+#line 46 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
     free_program(FFT_program);
     FFT_program=0;
   }
 
 #endif /* class_FFT_defined */
-#line 313 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/Math/transforms.cmod"
+#line 313 "/tmp/pikedeb.470504af38/7.6/src/modules/Math/transforms.cmod"
 #endif /* HAVE_RFFTW */
 }
 
