@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: mysql.c,v 1.81 2004/03/19 14:49:44 nilsson Exp $
+|| $Id: mysql.c,v 1.82 2004/07/15 08:39:09 grubba Exp $
 */
 
 /*
@@ -97,7 +97,7 @@
  * Globals
  */
 
-RCSID("$Id: mysql.c,v 1.81 2004/03/19 14:49:44 nilsson Exp $");
+RCSID("$Id: mysql.c,v 1.82 2004/07/15 08:39:09 grubba Exp $");
 
 /*! @module Mysql
  *!
@@ -1098,7 +1098,12 @@ static void f_shutdown(INT32 args)
   if (socket) {
     MYSQL_ALLOW();
   
+#ifdef HAVE_SHUTDOWN_DEFAULT
+    /* Mysql 4.1.3 added an extra shutdown_level argument. */
+    tmp = mysql_shutdown(socket, SHUTDOWN_DEFAULT);
+#else /* !HAVE_SHUTDOWN_DEFAULT */
     tmp = mysql_shutdown(socket);
+#endif /* HAVE_SHUTDOWN_DEFAULT */
 
     MYSQL_DISALLOW();
   }
@@ -1110,7 +1115,12 @@ static void f_shutdown(INT32 args)
 
     MYSQL_ALLOW();
   
+#ifdef HAVE_SHUTDOWN_DEFAULT
+    /* Mysql 4.1.3 added an extra shutdown_level argument. */
+    tmp = mysql_shutdown(socket, SHUTDOWN_DEFAULT);
+#else /* !HAVE_SHUTDOWN_DEFAULT */
     tmp = mysql_shutdown(socket);
+#endif /* HAVE_SHUTDOWN_DEFAULT */
 
     MYSQL_DISALLOW();
   }
