@@ -1,13 +1,14 @@
-/*\
-||| This file a part of Pike, and is copyright by Fredrik Hubinette
-||| Pike is distributed as GPL (General Public License)
-||| See the files COPYING and DISCLAIMER for more information.
-\*/
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: pike_regexp.h,v 1.9 2002/10/11 01:39:49 nilsson Exp $
+*/
+
 #ifndef REGEXP_H
 #define REGEXP_H
+
 /*
- * $Id: pike_regexp.h,v 1.3 1998/03/28 14:35:36 grubba Exp $
- *
  * Definitions etc. for regexp(3) routines.
  *
  * Caveat:  this is V8 regexp(3) [actually, a reimplementation thereof],
@@ -22,7 +23,7 @@ typedef struct regexp
   char regstart;		/* Internal use only. */
   char reganch;			/* Internal use only. */
   char *regmust;		/* Internal use only. */
-  int regmlen;			/* Internal use only. */
+  size_t regmlen;		/* Internal use only. */
   char program[1];		/* Unwarranted chumminess with compiler. */
 } regexp;
 
