@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: sane.c,v 1.15 2002/10/21 17:06:22 marcus Exp $
+|| $Id: sane.c,v 1.19 2004/02/03 10:14:21 nilsson Exp $
 */
 
 #include "config.h"
@@ -39,7 +39,7 @@
 
 #define sp Pike_sp
 
-RCSID("$Id: sane.c,v 1.15 2002/10/21 17:06:22 marcus Exp $");
+RCSID("$Id: sane.c,v 1.19 2004/02/03 10:14:21 nilsson Exp $");
 
 /*! @module SANE
  *!
@@ -334,7 +334,7 @@ static void f_scanner_set_option( INT32 args )
      case SANE_TYPE_BOOL:
      case SANE_TYPE_INT:
      case SANE_TYPE_BUTTON:
-       sp++;get_all_args( "set_option", args, "%D", &int_value );sp--;
+       sp++;get_all_args( "set_option", args, "%I", &int_value );sp--;
        sane_control_option( THIS->h, no, SANE_ACTION_SET_VALUE,
                             &int_value, &tmp );
        break;
@@ -722,7 +722,7 @@ static void f_scanner_nonblocking_row_scan( INT32 args )
   push_int( 1 );
   rsp->o = clone_object( image_program, 2 );
   rsp->t = Pike_fp->current_object;
-  Pike_fp->current_object->refs++;
+  add_ref(Pike_fp->current_object);
   rsp->r = ((struct image *)rsp->o->storage)->img;
   rsp->h = THIS->h;
   rsp->p = p;
@@ -741,7 +741,8 @@ static void f_scanner_nonblocking_row_scan( INT32 args )
     free( rsp );
     Pike_error("Failed to get select fd for scanning device!\n");
   }
-  set_read_callback( fd, nonblocking_row_scan_callback, (void*)rsp );
+  set_read_callback( fd, (file_callback)nonblocking_row_scan_callback,
+		     (void*)rsp );
   push_int( 0 );
 }
 
@@ -759,7 +760,7 @@ static void f_scanner_cancel_scan( INT32 args )
  *! @decl constant FrameRGB
  *! @decl constant FrameRed
  *! @decl constant FrameGreen
- *! @decl cosntant FrameBlue
+ *! @decl constant FrameBlue
  */
 
 /*! @endmodule
