@@ -1,12 +1,15 @@
 /*
- * $Id: image_ttf.c,v 1.40 2001/06/12 13:21:10 grubba Exp $
- */
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: image_ttf.c,v 1.51 2004/04/14 12:12:50 grubba Exp $
+*/
 
 #include "config.h"
 
-
 #include "global.h"
-RCSID("$Id: image_ttf.c,v 1.40 2001/06/12 13:21:10 grubba Exp $");
+RCSID("$Id: image_ttf.c,v 1.51 2004/04/14 12:12:50 grubba Exp $");
+#include "module.h"
 
 #ifdef HAVE_LIBTTF
 #if defined(HAVE_FREETYPE_FREETYPE_H) && defined(HAVE_FREETYPE_FTXKERN_H)
@@ -50,8 +53,8 @@ static TT_Engine engine;
 
 #endif /* HAVE_LIBTTF */
 
-/* This must be included last! */
-#include "module_magic.h"
+
+#define sp Pike_sp
 
 static struct pike_string *param_baseline;
 static struct pike_string *param_quality;
@@ -271,14 +274,14 @@ static void image_ttf_make(INT32 args)
 
 static void image_ttf_face_exit(struct object *o)
 {
-  if (THISf)
+  if(THISf)
     TT_Close_Face(THISf->face);
 }
 
 static void image_ttf_faceinstance_exit(struct object *o)
 {
   if (THISi) {
-    if (THISi->faceobj->prog)
+    if(THISi->faceobj->prog)
       TT_Done_Instance(THISi->instance);
     free_object(THISi->faceobj);
   }
@@ -457,7 +460,7 @@ static void image_ttf_face_properties(INT32 args)
 **!	Might be used to save memory - the face
 **!	information is read back from disk upon need.
 **!
-**! returns the called object
+**! returns the object being called
 */
 
 static void image_ttf_face_flush(INT32 args)
@@ -555,7 +558,7 @@ static void image_ttf_face_names(INT32 args)
    image_ttf_face__names(args);
 
    if (sp[-1].type!=T_ARRAY)
-      Pike_error("Image.TTF.Face->names(): internal error, wierd _names()\n");
+      Pike_error("Image.TTF.Face->names(): internal error, weird _names()\n");
 
    a=sp[-1].u.array;
 
@@ -567,7 +570,7 @@ static void image_ttf_face_names(INT32 args)
       b=a->item[i].u.array;
 
       what=b->item[3].u.integer;
-      if (what>=8 || what<0) continue; /* wierd */
+      if (what>=8 || what<0) continue; /* weird */
       switch (b->item[0].u.integer*100+b->item[1].u.integer)
       {
 	 case 301: /* M$:  unicode */
@@ -582,7 +585,7 @@ static void image_ttf_face_names(INT32 args)
 
       push_text(hasname[what]);
 
-      if (ihas==30) /* unicode, M$ but wierd enough correct byteorder */
+      if (ihas==30) /* unicode, M$ but weird enough correct byteorder */
       {
 	 ptrdiff_t n = b->item[4].u.string->len/2;
 	 struct pike_string *ps=begin_wide_shared_string(n,1);
@@ -912,7 +915,7 @@ static void image_ttf_faceinstance_ponder(INT32 args)
 
 static int find_kerning( TT_Kerning kerning, int c1, int c2 )
 {
-  int i,j;
+  int j;
 /*   fprintf(stderr, "kern: %d %d\n", c1, c2); */
   for(j=0; j<kerning.nTables; j++)
   {
@@ -1119,7 +1122,6 @@ static void image_ttf_faceinstance_write(INT32 args)
       TT_Raster_Map rastermap;
       struct object *o;
       struct image *img;
-      unsigned char *s;
       rgb_group *d;
 
 
@@ -1239,7 +1241,7 @@ static void image_ttf_faceinstance_face(INT32 args)
 /*** module init & exit & stuff *****************************************/
 
 
-void pike_module_exit(void)
+PIKE_MODULE_EXIT
 {
    free_string(param_baseline);
    free_string(param_quality);
@@ -1257,7 +1259,7 @@ void pike_module_exit(void)
 #endif /* HAVE_LIBTTF */
 }
 
-void pike_module_init(void)
+PIKE_MODULE_INIT
 {
 #ifdef HAVE_LIBTTF
    unsigned char palette[5]={0,64,128,192,255};
@@ -1277,20 +1279,15 @@ void pike_module_init(void)
      return;
    }
 
-   TT_Set_Raster_Gray_Palette(engine,(char*)palette);
+   TT_Set_Raster_Gray_Palette(engine, palette);
    TT_Init_Kerning_Extension( engine );
 
 #ifdef DYNAMIC_MODULE
-   push_string(make_shared_string("Image"));
-   push_int(0);
-   SAFE_APPLY_MASTER("resolv",2);
-   if (sp[-1].type==T_OBJECT)
-   {
-      push_string(make_shared_string("image"));
-      f_index(2);
+   push_text("Image.Image");
+   SAFE_APPLY_MASTER("resolv",1);
+   if (sp[-1].type==T_PROGRAM)
       image_program=program_from_svalue(sp-1);
-   }
-   pop_n_elems(1);
+   pop_stack();
 #endif /* DYNAMIC_MODULE */
 
    if (image_program)
