@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: efuns.c,v 1.120 2004/05/13 20:38:14 mast Exp $
+|| $Id: efuns.c,v 1.143 2004/10/04 08:13:38 grubba Exp $
 */
 
 #include "global.h"
@@ -20,13 +20,13 @@
 #include "backend.h"
 #include "operators.h"
 #include "builtin_functions.h"
-#include "security.h"
+#include "pike_security.h"
 #include "bignum.h"
 
 #include "file_machine.h"
 #include "file.h"
 
-RCSID("$Id: efuns.c,v 1.120 2004/05/13 20:38:14 mast Exp $");
+RCSID("$Id: efuns.c,v 1.143 2004/10/04 08:13:38 grubba Exp $");
 
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -110,12 +110,14 @@ struct array *encode_stat(PIKE_STAT_T *s)
 {
   struct array *a;
   a=allocate_array(7);
+  a->type_field = BIT_INT;
   ITEM(a)[0].u.integer=s->st_mode;
   switch(S_IFMT & s->st_mode)
   {
   case S_IFREG:
     push_int64((INT64)s->st_size);
-    ITEM(a)[1] = *(--sp);
+    stack_pop_to_no_free (ITEM(a) + 1);
+    if (ITEM(a)[1].type == T_OBJECT) a->type_field |= BIT_OBJECT;
     break;
     
   case S_IFDIR: ITEM(a)[1].u.integer=-2; break;
@@ -141,10 +143,10 @@ struct array *encode_stat(PIKE_STAT_T *s)
  *!
  *! Stat a file.
  *!
- *! If the argument @[symlink] is @tt{1@} symlinks will not be followed.
+ *! If the argument @[symlink] is @expr{1@} symlinks will not be followed.
  *!
  *! @returns
- *!   If the path specified by @[path] doesn't exist @tt{0@} (zero) will
+ *!   If the path specified by @[path] doesn't exist @expr{0@} (zero) will
  *!   be returned.
  *!
  *!   Otherwise an object describing the properties of @[path] will be
@@ -153,12 +155,14 @@ struct array *encode_stat(PIKE_STAT_T *s)
  *! @note
  *!   In Pike 7.0 and earlier this function returned an array with 7 elements.
  *!   The old behaviour can be simulated with the following function:
- *!   @code{array(int) file_stat(string path, void|int(0..1) symlink)
- *!         {
- *!           File.Stat st = predef::file_stat(path, symlink);
- *!           if (!st) return 0;
- *!           return (array(int))st;
- *!         }@}
+ *! @code
+ *! array(int) file_stat(string path, void|int(0..1) symlink)
+ *! {
+ *!   File.Stat st = predef::file_stat(path, symlink);
+ *!   if (!st) return 0;
+ *!   return (array(int))st;
+ *! }
+ *! @endcode
  *!
  *! @seealso
  *!   @[Stdio.Stat], @[Stdio.File->stat()]
@@ -169,16 +173,16 @@ void f_file_stat(INT32 args)
   int i, l;
   struct pike_string *str;
   
+  VALID_FILE_IO("file_stat","read");
+
   if(args<1)
-    Pike_error("Too few arguments to file_stat()\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR("file_stat", 1);
   if(sp[-args].type != T_STRING)
-    Pike_error("Bad argument 1 to file_stat()\n");
+    SIMPLE_BAD_ARG_ERROR("file_stat", 1, "string");
 
   str = sp[-args].u.string;
   l = (args>1 && !UNSAFE_IS_ZERO(sp+1-args))?1:0;
 
-  VALID_FILE_IO("file_stat","read");
-
   if (strlen(str->str) != (size_t)str->len) {
     /* Filenames with NUL are not supported. */
     errno = ENOENT;
@@ -222,6 +226,8 @@ void f_file_truncate(INT32 args)
   struct pike_string *str;
   int res;
 
+  VALID_FILE_IO("file_truncate","write");
+
   if(args < 2)
     SIMPLE_TOO_FEW_ARGS_ERROR("file_truncate", 2);
   if(sp[-args].type != T_STRING)
@@ -251,8 +257,6 @@ void f_file_truncate(INT32 args)
     return;
   }
 
-  VALID_FILE_IO("file_truncate","write");
-
 #ifdef __NT__
   {
     HANDLE h = CreateFile(str->str, GENERIC_WRITE,
@@ -304,7 +308,7 @@ void f_file_truncate(INT32 args)
  *! containing the path specified by @[path].
  *!
  *! @returns
- *!   If a filesystem cannot be determined @tt{0@} (zero) will be returned.
+ *!   If a filesystem cannot be determined @expr{0@} (zero) will be returned.
  *!
  *!   Otherwise a mapping(string:int) with the following fields will be
  *!   returned:
@@ -317,7 +321,7 @@ void f_file_truncate(INT32 args)
  *!       Number of free blocks in the filesystem.
  *!     @member int "bavail"
  *!       Number of available blocks in the filesystem.
- *!       This is usually somewhat less than the @tt{"bfree"@} value, and
+ *!       This is usually somewhat less than the @expr{"bfree"@} value, and
  *!       can usually be adjusted with eg tunefs(1M).
  *!     @member int "files"
  *!       Total number of files (aka inodes) allowed by this filesystem.
@@ -325,12 +329,14 @@ void f_file_truncate(INT32 args)
  *!       Number of free files in the filesystem.
  *!     @member int "favail"
  *!       Number of available files in the filesystem.
- *!       This is usually the same as the @tt{"ffree"@} value, and can
+ *!       This is usually the same as the @expr{"ffree"@} value, and can
  *!       usually be adjusted with eg tunefs(1M).
- *!     @member int "fsname"
- *!       Name assigned to the filesystem.
- *!     @member int "fstype"
- *!       Type of filesystem (eg @tt{"nfs"@}).
+ *!     @member string "fsname"
+ *!       Name assigned to the filesystem. This item is not available
+ *!       on all systems.
+ *!     @member string "fstype"
+ *!       Type of filesystem (eg @expr{"nfs"@}). This item is not
+ *!       available on all systems.
  *!   @endmapping
  *!
  *! @note
@@ -421,9 +427,11 @@ void f_filesystem_stat( INT32 args )
 #ifdef HAVE_SYS_MOUNT_H
 #include <sys/mount.h>
 #endif /* HAVE_SYS_MOUNT_H */
+#if !defined(HAVE_STATVFS) && !defined(HAVE_STATFS)
 #ifdef HAVE_USTAT_H
 #include <ustat.h>
 #endif /* HAVE_USTAT_H */
+#endif /* !HAVE_STATVFS && !HAVE_STATFS */
 void f_filesystem_stat(INT32 args)
 {
 #ifdef HAVE_STATVFS
@@ -455,9 +463,9 @@ void f_filesystem_stat(INT32 args)
   struct pike_string *str;
 
   if(args<1)
-    Pike_error("Too few arguments to filesystem_stat()\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR("filesystem_stat", 1);
   if(sp[-args].type != T_STRING)
-    Pike_error("Bad argument 1 to filesystem_stat()\n");
+    SIMPLE_BAD_ARG_ERROR("filesystem_stat", 1, "string");
 
   str = sp[-args].u.string;
 
@@ -576,10 +584,12 @@ void f_filesystem_stat(INT32 args)
  */
 void f_werror(INT32 args)
 {
+  VALID_FILE_IO("werror","werror");
+
   if(!args)
-    Pike_error("Too few arguments to werror.\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR("werror", 1);
   if(sp[-args].type != T_STRING)
-    Pike_error("Bad argument 1 to werror().\n");
+    SIMPLE_BAD_ARG_ERROR("werror", 1, "string");
 
   if(args> 1)
   {
@@ -587,8 +597,9 @@ void f_werror(INT32 args)
     args=1;
   }
 
-  VALID_FILE_IO("werror","werror");
-
+  /* FIXME: Wide string handling. */
+  if(sp[-args].u.string->size_shift!=0)
+    Pike_error("Wide strings can not be written to stderr.\n");
   write_to_stderr(sp[-args].u.string->str, sp[-args].u.string->len);
   pop_n_elems(args);
 }
@@ -598,7 +609,7 @@ void f_werror(INT32 args)
  *! Remove a file or directory.
  *!
  *! @returns
- *!   Returns @tt{0@} (zero) on failure, @tt{1@} otherwise.
+ *!   Returns @expr{0@} (zero) on failure, @expr{1@} otherwise.
  *!
  *! @seealso
  *!   @[mkdir()], @[Stdio.recursive_rm()]
@@ -611,13 +622,13 @@ void f_rm(INT32 args)
 
   destruct_objects_to_destruct();
 
+  VALID_FILE_IO("rm","write");
+
   if(!args)
-    Pike_error("Too few arguments to rm()\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR("rm", 1);
 
   if(sp[-args].type != T_STRING)
-    Pike_error("Bad argument 1 to rm().\n");
-
-  VALID_FILE_IO("rm","write");
+    SIMPLE_BAD_ARG_ERROR("rm", 1, "string");
 
   str = sp[-args].u.string;
   
@@ -677,10 +688,10 @@ void f_rm(INT32 args)
  *! Create a directory.
  *!
  *! If @[mode] is specified, it's will be used for the new directory after
- *! being @tt{&@}'ed with the current umask (on OS'es that support this).
+ *! being @expr{&@}'ed with the current umask (on OS'es that support this).
  *!
  *! @returns
- *!   Returns @tt{0@} (zero) on failure, @tt{1@} otherwise.
+ *!   Returns @expr{0@} (zero) on failure, @expr{1@} otherwise.
  *!
  *! @seealso
  *!   @[rm()], @[cd()], @[Stdio.mkdirhier()]
@@ -692,11 +703,13 @@ void f_mkdir(INT32 args)
   int mode;
   int i;
 
+  VALID_FILE_IO("mkdir","write");
+
   if(!args)
-    Pike_error("Too few arguments to mkdir()\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR("mkdir", 1);
 
   if(sp[-args].type != T_STRING)
-    Pike_error("Bad argument 1 to mkdir().\n");
+    SIMPLE_BAD_ARG_ERROR("mkdir", 1, "string");
 
   mode = 0777;			/* &'ed with ~umask anyway. */
 
@@ -708,8 +721,6 @@ void f_mkdir(INT32 args)
     mode = sp[1-args].u.integer;
   }
 
-  VALID_FILE_IO("mkdir","write");
-
   str = sp[-args].u.string;
 
   if (strlen(str->str) != (size_t)str->len) {
@@ -768,9 +779,14 @@ void f_mkdir(INT32 args)
 	do {
 	  i = chmod(str->str, mode) != -1;
 	  if (i || errno != EINTR) break;
-	  THREADS_DISALLOW_UID();
-	  check_threads_etc();
-	  THREADS_ALLOW_UID();
+	  /* Must have do { ... } while(0) around these since
+	   * THREADS_DISALLOW_UID contains "} while (0)" and
+	   * THREADS_ALLOW_UID "do {". */
+	  do {
+	    THREADS_DISALLOW_UID();
+	    check_threads_etc();
+	    THREADS_ALLOW_UID();
+	  } while (0);
 	} while (1);
       }
       if (i) {
@@ -802,17 +818,14 @@ void f_mkdir(INT32 args)
 /*! @decl array(string) get_dir(string dirname)
  *!
  *! Returns an array of all filenames in the directory @[dirname], or
- *! @tt{0@} (zero) if the directory does not exist.
+ *! @expr{0@} (zero) if the directory does not exist.
  *!
  *! @seealso
  *!   @[mkdir()], @[cd()]
  */
 void f_get_dir(INT32 args)
 {
-  struct svalue *save_sp=sp;
   DIR *dir;
-  struct dirent *d;
-  struct array *a=0;
   struct pike_string *str;
 
   VALID_FILE_IO("get_dir","read");
@@ -827,34 +840,39 @@ void f_get_dir(INT32 args)
     return;
   }
 
-#if defined(_REENTRANT) && defined(HAVE_READDIR_R)
   THREADS_ALLOW_UID();
   dir = opendir(str->str);
   THREADS_DISALLOW_UID();
   if(dir)
   {
+    struct dirent *d;
+    struct dirent *tmp = NULL;
+#if defined(_REENTRANT) && defined(HAVE_READDIR_R)
 #define FPR 1024
     char buffer[MAXPATHLEN * 4];
     char *ptrs[FPR];
     ptrdiff_t lens[FPR];
-    struct dirent *tmp;
 
     if (!(tmp =
-#if defined(HAVE_SOLARIS_READDIR_R) || defined(_PC_NAME_MAX)
+#ifdef _PC_NAME_MAX
 	  malloc(sizeof(struct dirent) + 
 		 ((pathconf(str->str, _PC_NAME_MAX) < 1024)?1024:
 		  pathconf(str->str, _PC_NAME_MAX)) + 1)
-#else
+#else /* !_PC_NAME_MAX */
 #ifndef NAME_MAX
 #define NAME_MAX 1024
 #endif
 	  malloc(sizeof(struct dirent) + NAME_MAX+ 1024 + 1)
-#endif /* HAVE_SOLARIS_READDIR_R */
+#endif /* _PC_NAME_MAX */
       )) {
       closedir(dir);
       Pike_error("get_dir(): Out of memory\n");
     }
+#endif /* _REENTRANT && HAVE_READDIR_R */
 
+    BEGIN_AGGREGATE_ARRAY(10);
+
+#if defined(_REENTRANT) && defined(HAVE_READDIR_R)
     while(1)
     {
       int e;
@@ -935,10 +953,11 @@ void f_get_dir(INT32 args)
 #else
 #error Unknown readdir_r variant
 #endif
+	/* Filter "." and ".." from the list. */
 	if(d->d_name[0]=='.')
 	{
-	  if(!d->d_name[1]) continue;
-	  if(d->d_name[1]=='.' && !d->d_name[2]) continue;
+	  if(NAMLEN(d)==1) continue;
+	  if(d->d_name[1]=='.' && NAMLEN(d)==2) continue;
 	}
 	if(num_files >= FPR) break;
 	lens[num_files]=NAMLEN(d);
@@ -960,36 +979,32 @@ void f_get_dir(INT32 args)
 	push_string(make_shared_binary_string(d->d_name,NAMLEN(d)));
       else
 	break;
+      DO_AGGREGATE_ARRAY(120);
     }
-    THREADS_ALLOW();
+
     free(tmp);
-    closedir(dir);
-    THREADS_DISALLOW();
-    a = aggregate_array(DO_NOT_WARN((INT32)(sp - save_sp)));
-  }
 #else
-  dir = opendir(str->str);
-  if(dir)
-  {
     for(d=readdir(dir); d; d=readdir(dir))
     {
+      /* Filter "." and ".." from the list. */
       if(d->d_name[0]=='.')
       {
-	if(!d->d_name[1]) continue;
-	if(d->d_name[1]=='.' && !d->d_name[2]) continue;
+	if(NAMLEN(d)==1) continue;
+	if(d->d_name[1]=='.' && NAMLEN(d)==2) continue;
       }
       push_string(make_shared_binary_string(d->d_name, NAMLEN(d)));
-    }
-    closedir(dir);
-    a=aggregate_array(sp-save_sp);
-  }
-#endif
 
-  pop_n_elems(args);
-  if(a)
-    push_array(a);
-  else
+      DO_AGGREGATE_ARRAY(120);
+    }
+#endif
+    closedir(dir);
+
+    END_AGGREGATE_ARRAY;
+    stack_pop_n_elems_keep_top(args);
+  } else {
+    pop_n_elems(args);
     push_int(0);
+  }
 }
 
 /*! @decl int cd(string s)
@@ -997,7 +1012,7 @@ void f_get_dir(INT32 args)
  *! Change the current directory for the whole Pike process.
  *!
  *! @returns
- *!   Returns @tt{1@} for success, @tt{0@} (zero) otherwise.
+ *!   Returns @expr{1@} for success, @expr{0@} (zero) otherwise.
  *!
  *! @seealso
  *!   @[getcwd()]
@@ -1007,13 +1022,13 @@ void f_cd(INT32 args)
   INT32 i;
   struct pike_string *str;
 
+  VALID_FILE_IO("cd","status");
+
   if(!args)
-    Pike_error("Too few arguments to cd()\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR("cd", 1);
 
   if(sp[-args].type != T_STRING)
-    Pike_error("Bad argument 1 to cd()\n");
-
-  VALID_FILE_IO("cd","status");
+    SIMPLE_BAD_ARG_ERROR("cd", 1, "string");
 
   str = sp[-args].u.string;
 
@@ -1069,10 +1084,11 @@ void f_getcwd(INT32 args)
   }
 
   pop_n_elems(args);
-  push_string(make_shared_string(e));
+  push_text(e);
   free(e);
 }
 
+#ifdef HAVE_EXECVE
 /*! @decl int exece(string file, array(string) args)
  *! @decl int exece(string file, array(string) args, @
  *!                 mapping(string:string) env)
@@ -1085,13 +1101,15 @@ void f_getcwd(INT32 args)
  *!
  *! @returns
  *!   This function only returns if something went wrong during @tt{exece(2)@},
- *!   and in that case it returns @tt{0@} (zero).
+ *!   and in that case it returns @expr{0@} (zero).
  *!
  *! @note
  *!   The Pike driver _dies_ when this function is called. You must either
  *!   use @[fork()] or @[Process.create_process()] if you wish to execute a
  *!   program and still run the Pike runtime.
  *!
+ *!   This function is not available on all platforms.
+ *!
  *! @seealso
  *!   @[Process.create_process()], @[fork()], @[Stdio.File->pipe()]
  */
@@ -1108,7 +1126,7 @@ void f_exece(INT32 args)
   save_sp=sp-args;
 
   if(args < 2)
-    Pike_error("Too few arguments to exece().\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR("exece", 2);
 
 #ifdef PIKE_SECURITY
   if(!CHECK_SECURITY(SECURITY_BIT_SECURITY))
@@ -1122,26 +1140,26 @@ void f_exece(INT32 args)
   {
   default:
     if(sp[2-args].type != T_MAPPING)
-      Pike_error("Bad argument 3 to exece().\n");
+      SIMPLE_BAD_ARG_ERROR("exece", 3, "mapping(string:string)");
     en=sp[2-args].u.mapping;
     mapping_fix_type_field(en);
 
     if(m_ind_types(en) & ~BIT_STRING)
-      Pike_error("Bad argument 3 to exece().\n");
+      SIMPLE_BAD_ARG_ERROR("exece", 3, "mapping(string:string)");
     if(m_val_types(en) & ~BIT_STRING)
-      Pike_error("Bad argument 3 to exece().\n");
+      SIMPLE_BAD_ARG_ERROR("exece", 3, "mapping(string:string)");
 
   case 2:
     if(sp[1-args].type != T_ARRAY)
-      Pike_error("Bad argument 2 to exece().\n");
+      SIMPLE_BAD_ARG_ERROR("exece", 2, "array(string)");
     array_fix_type_field(sp[1-args].u.array);
 
     if(sp[1-args].u.array->type_field & ~BIT_STRING)
-      Pike_error("Bad argument 2 to exece().\n");
+      SIMPLE_BAD_ARG_ERROR("exece", 2, "array(string)");
 
   case 1:
     if(sp[0-args].type != T_STRING)
-      Pike_error("Bad argument 1 to exece().\n");
+      SIMPLE_BAD_ARG_ERROR("exece", 1, "string");
   }
 
   argv=(char **)xalloc((2+sp[1-args].u.array->size) * sizeof(char *));
@@ -1169,7 +1187,7 @@ void f_exece(INT32 args)
     for(e=0;e<i->size;e++)
     {
       push_string(ITEM(i)[e].u.string);
-      push_string(make_shared_string("="));
+      push_constant_text("=");
       push_string(ITEM(v)[e].u.string);
       f_add(3);
       env[e]=sp[-1].u.string->str;
@@ -1207,6 +1225,7 @@ void f_exece(INT32 args)
   pop_n_elems(sp-save_sp);
   push_int(0);
 }
+#endif
 
 /*! @decl int mv(string from, string to)
  *!
@@ -1222,7 +1241,7 @@ void f_exece(INT32 args)
  *! them.
  *!
  *! @returns
- *!   Returns @tt{0@} (zero) on failure, @tt{1@} otherwise. Call
+ *!   Returns @expr{0@} (zero) on failure, @expr{1@} otherwise. Call
  *!   @[errno()] to get more error info on failure.
  *!
  *! @seealso
@@ -1239,16 +1258,16 @@ void f_mv(INT32 args)
   PIKE_STAT_T st;
 #endif
 
+  VALID_FILE_IO("mv","write");
+
   if(args<2)
-    Pike_error("Too few arguments to mv()\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR("mv", 2);
 
   if(sp[-args].type != T_STRING)
-    Pike_error("Bad argument 1 to mv().\n");
+    SIMPLE_BAD_ARG_ERROR("mv", 1, "string");
 
   if(sp[-args+1].type != T_STRING)
-    Pike_error("Bad argument 2 to mv().\n");
-
-  VALID_FILE_IO("mv","write");
+    SIMPLE_BAD_ARG_ERROR("mv", 2, "string");
 
   str1 = sp[-args].u.string;
   str2 = sp[1-args].u.string;
@@ -1444,9 +1463,9 @@ void f_strerror(INT32 args)
   int err;
 
   if(!args) 
-    Pike_error("Too few arguments to strerror()\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR("strerror", 1);
   if(sp[-args].type != T_INT)
-    Pike_error("Bad argument 1 to strerror()\n");
+    SIMPLE_BAD_ARG_ERROR("strerror", 1, "int");
 
   err = sp[-args].u.integer;
   pop_n_elems(args);
@@ -1548,9 +1567,11 @@ void init_files_efuns(void)
   
 /* function(:string) */
   ADD_EFUN("getcwd",f_getcwd,tFunc(tNone,tStr),OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
-  
+
+#ifdef HAVE_EXECVE
 /* function(string,mixed*,void|mapping(string:string):int) */
   ADD_EFUN("exece",f_exece,tFunc(tStr tArr(tMix) tOr(tVoid,tMap(tStr,tStr)),tInt),OPT_SIDE_EFFECT); 
+#endif
 
 /* function(int:string) */
   ADD_EFUN("strerror",f_strerror,tFunc(tInt,tStr),0);
