@@ -2,12 +2,12 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: file.c,v 1.318 2004/04/14 11:34:02 grubba Exp $
+|| $Id: file.c,v 1.321 2005/05/18 12:36:54 mast Exp $
 */
 
 #define NO_PIKE_SHORTHAND
 #include "global.h"
-RCSID("$Id: file.c,v 1.318 2004/04/14 11:34:02 grubba Exp $");
+RCSID("$Id: file.c,v 1.321 2005/05/18 12:36:54 mast Exp $");
 #include "fdlib.h"
 #include "pike_netlib.h"
 #include "interpret.h"
@@ -188,7 +188,7 @@ static void debug_check_internals (struct my_file *f)
   for (ev = 0; ev < NELEM (f->event_cbs); ev++)
     if (f->event_cbs[ev].type == PIKE_T_INT &&
 	f->box.backend && f->box.events & (1 << ev))
-      Pike_fatal ("Got event flag but no callback for event %d.\n", ev);
+      Pike_fatal ("Got event flag but no callback for event %"PRINTSIZET"d.\n", ev);
 }
 #else
 #define debug_check_internals(f) do {} while (0)
@@ -2285,6 +2285,43 @@ static void file_query_fd(INT32 args)
   push_int(FD);
 }
 
+/*! @decl int release_fd()
+ *!
+ *! Returns the file descriptor number associated with this object, in
+ *! addition to releasing it so that this object behaves as if closed.
+ *! Other settings like callbacks and backend remain intact.
+ *! @[take_fd] can later be used to reinstate the file descriptor so
+ *! that the state is restored.
+ *!
+ *! @seealso
+ *!   @[query_fd()], @[take_fd()]
+ */
+static void file_release_fd(INT32 args)
+{
+  file_query_fd(args);
+  change_fd_for_box(&THIS->box, -1);
+}
+
+/*! @decl void take_fd(int fd)
+ *!
+ *! Rehooks the given file descriptor number to be associated with
+ *! this object. As opposed to using @[open] with a file descriptor
+ *! number, it will be closed by this object upon destruct or when
+ *! @[close] is called.
+ *!
+ *! @seealso
+ *!   @[release_fd()]
+ */
+static void file_take_fd(INT32 args)
+{
+  if (args < 1)
+    SIMPLE_TOO_FEW_ARGS_ERROR ("Stdio.File->take_fd", 1);
+  if (Pike_sp[-args].type != PIKE_T_INT)
+    SIMPLE_BAD_ARG_ERROR ("Stdio.File->take_fd", 0, "int");
+  change_fd_for_box(&THIS->box, Pike_sp[-args].u.integer);
+  pop_n_elems(args);
+}
+
 struct object *file_make_object_from_fd(int fd, int mode, int guess)
 {
   struct object *o=low_clone(file_program);
@@ -2343,6 +2380,7 @@ static void file_set_buffer(INT32 args)
   }else{
     flags=FILE_READ | FILE_WRITE;
   }
+  pop_n_elems(args);
 
 #ifdef SOCKET_BUFFER_MAX
 #if SOCKET_BUFFER_MAX
@@ -3063,7 +3101,7 @@ static void file_connect(INT32 args)
   struct svalue *dest_port = NULL;
   struct svalue *src_port = NULL;
 
-  int tmp;
+  int tmp, was_closed = FD < 0;
 
   if (args < 4) {
     get_all_args("Stdio.File->connect", args, "%S%*", &dest_addr, &dest_port);
@@ -3086,7 +3124,7 @@ static void file_connect(INT32 args)
 			   (dest_port->type == PIKE_T_INT?
 			    dest_port->u.integer : -1), 0);
 
-  if(FD < 0)
+  if(was_closed)
   {
     if (args < 4) {
       push_int(-1);
@@ -3122,6 +3160,11 @@ static void file_connect(INT32 args)
   {
     /* something went wrong */
     ERRNO=errno;
+    if (was_closed) {
+      while (fd_close (FD) && errno == EINTR) {}
+      change_fd_for_box (&THIS->box, -1);
+      errno = ERRNO;
+    }
     pop_n_elems(args);
     push_int(0);
   }else{
