@@ -1,8 +1,9 @@
-/*\
-||| This file a part of Pike, and is copyright by Fredrik Hubinette
-||| Pike is distributed as GPL (General Public License)
-||| See the files COPYING and DISCLAIMER for more information.
-\*/
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: sprintf.c,v 1.119 2004/11/02 14:38:45 aldem Exp $
+*/
 
 /* TODO: use ONERROR to cleanup fsp */
 
@@ -15,56 +16,7 @@
   deallocated at next call of sprintf. Most operators doesn't need this
   feature though as they allocate their buffers with alloca() or simply use
   pointers into other strings.
-  It also has a lot more features:
-
-  Modifiers:
-    0  Zero pad numbers (implies right justification)
-    !  Toggle truncation
-       pad positive integers with a space
-    +  pad positive integers with a plus sign
-    -  left adjusted within field size (default is right)
-    |  centered within field size
-    =  column mode if strings are greater than field size
-    /  Rough linebreak (break at exactly fieldsize instead of between words)
-    #  table mode, print a list of '\n' separated word (top-to-bottom order)
-    $  Inverse table mode (left-to-right order)
-    n  (where n is a number or *) a number specifies field size
-   .n  set precision
-   :n  set field size & precision
-   ;n  Set column width
-    *  if n is a * then next argument is used
-    ~  get pad string from argument list
-   'X'  Set a pad string. ' cannot be a part of the pad_string (yet)
-    <  Use same arg again
-    ^  repeat this on every line produced
-    @  do this format for each entry in argument array
-    > Put the string at the bottom end of column instead of top
-    _ Set width to size of length of data
-   [n] Select argument n. Use * to use the next arg as selector. 
-
- Operators:
-   %% percent
-   %d signed decimal int
-   %u unsigned decimal int
-   %o unsigned octal int
-   %x lowercase unsigned hexadecimal int
-   %X uppercase unsigned hexadecimal int
-   %c char
-   %f float
-   %g heuristically chosen representation of float
-   %e exponential notation float
-   %s string
-   %O any type (prettyprint)
-   %n nop
-   %t type of argument
-   %<modifiers>{format%}  do a format for every index in an array.
-
-   Most flags and operators are combinable in any fashion, but _really_
-   strange results can arise from things like:
-      sprintf("%+080#{%s\n%}",get_dir("/"))
-
- Callbacks:
-   string _sprintf(int operator, mapping modifiers);
+  It also has a lot more features.
 
  Ideas yet to be implemented:
    Line-break with fill? Lower-case? Upper case? Capitalize?
@@ -78,7 +30,7 @@
 
    A short 'people' function (without sort)
 
-   sprintf("%{%-14s %2d %-30s %s\n%}\n",map_array(users(),lambda(object x)
+   sprintf("%{%-14s %2d %-30s %s\n%}\n",map(users(),lambda(object x)
        {
            return ({x->query_real_name(),
 		x->query_level(),
@@ -102,8 +54,231 @@
 
 */
 
+/*! @decl string sprintf(string format, mixed ... args)
+ *!
+ *!   Print formated output to string.
+ *!
+ *!   The @[format] string is a string containing a description of how to
+ *!   output the data in @[args]. This string should generally speaking
+ *!   have one @tt{%@i{<modifiers>@}@i{<operator>@}@} format specifier
+ *!   (examples: @tt{%s@}, @tt{%0d@}, @tt{%-=20s@}) for each of the arguments.
+ *!
+ *!   The following modifiers are supported:
+ *!   @int
+ *!     @value '0'
+ *!       Zero pad numbers (implies right justification).
+ *!     @value '!'
+ *!       Toggle truncation.
+ *!     @value ' '
+ *!       Pad positive integers with a space.
+ *!     @value '+'
+ *!       Pad positive integers with a plus sign.
+ *!     @value '-'
+ *!       Left adjust within field size (default is right).
+ *!     @value '|'
+ *!       Centered within field size.
+ *!     @value '='
+ *!       Column mode if strings are greater than field size.
+ *!     @value '/'
+ *!       Rough line break (break at exactly field size instead of between
+ *!       words).
+ *!     @value '#'
+ *!       Table mode, print a list of '\n' separated word (top-to-bottom
+ *!       order).
+ *!     @value '$'
+ *!       Inverse table mode (left-to-right order).
+ *!     @value 'n'
+ *!       (Where n is a number or *) field size specifier.
+ *!     @value '.n'
+ *!       Precision specifier.
+ *!     @value ':n'
+ *!       Field size precision specifier.
+ *!     @value ';n'
+ *!       Column width specifier.
+ *!     @value '*'
+ *!       If n is a * then next argument is used for precision/field size.
+ *!     @value "'"
+ *!       Set a pad string. @tt{'@} cannot be a part of the pad string (yet).
+ *!     @value '~'
+ *!       Get pad string from argument list.
+ *!     @value '<'
+ *!       Use same argument again.
+ *!     @value '^'
+ *!       Repeat this on every line produced.
+ *!     @value '@'
+ *!       Repeat this format for each element in the argument array.
+ *!     @value '>'
+ *!       Put the string at the bottom end of column instead of top.
+ *!     @value '_'
+ *!       Set width to the length of data.
+ *!     @value '[n]'
+ *!       Select argument number @tt{@i{n@}@}. Use @tt{*@} to use the next
+ *!       argument as selector.
+ *!   @endint
+ *!
+ *!   The following operators are supported:
+ *!   @int
+ *!     @value '%'
+ *!       Percent.
+ *!     @value 'b'
+ *!       Signed binary integer.
+ *!     @value 'd'
+ *!       Signed decimal integer.
+ *!     @value 'u'
+ *!       Unsigned decimal integer.
+ *!     @value 'o'
+ *!       Signed octal integer.
+ *!     @value 'x'
+ *!       Lowercase signed hexadecimal integer.
+ *!     @value 'X'
+ *!       Uppercase signed hexadecimal integer.
+ *!     @value 'c'
+ *!       Character. If a fieldsize has been specified this will output
+ *!       the low-order bytes of the integer in network (big endian) byte
+ *!       order. To get little endian byte order, negate the field size.
+ *!     @value 'f'
+ *!       Float. (Locale dependent formatting.)
+ *!     @value 'g'
+ *!       Heuristically chosen representation of float.
+ *!       (Locale dependent formatting.)
+ *!     @value 'G'
+ *!       Like @tt{%g@}, but uses uppercase @tt{E@} for exponent.
+ *!     @value 'e'
+ *!       Exponential notation float. (Locale dependent output.)
+ *!     @value 'E'
+ *!       Like @tt{%e@}, but uses uppercase @tt{E@} for exponent.
+ *!     @value 'F'
+ *!       Binary IEEE representation of float (@tt{%4F@} gives 
+ *!       single precision, @tt{%8F@} gives double precision.)
+ *!     @value 's'
+ *!       String.
+ *!     @value 'O'
+ *!       Any value, debug style. Do not rely on the exact formatting;
+ *!       how the result looks can vary depending on locale, phase of
+ *!       the moon or anything else the _sprintf method implementor
+ *!       wanted for debugging.
+ *!     @value 'n'
+ *!       No operation (ignore the argument).
+ *!     @value 't'
+ *!       Type of the argument.
+ *!     @value '{'
+ *!     @value '}'
+ *!       Perform the enclosed format for every element of the argument array.
+ *!   @endint
+ *!
+ *!   Most modifiers and operators are combinable in any fashion, but some
+ *!   combinations may render strange results.
+ *!
+ *!   If an argument is an object that implements @[lfun::_sprintf()], that
+ *!   callback will be called with the operator as the first argument, and
+ *!   the current modifiers as the second. The callback is expected to return
+ *!   a string.
+ *!
+ *! @example
+ *! Pike v7.4 release 13 running Hilfe v3.5 (Incremental Pike Frontend)
+ *! > sprintf("The unicode character %c has character code %04X.", 'A', 'A');
+ *! (1) Result: "The unicode character A has character code 0041."
+ *! > sprintf("#%@@02X is the HTML code for purple.", Image.Color.purple->rgb());
+ *! (2) Result: "#A020F0 is the HTML code for purple."
+ *! > int n=4711;
+ *! > sprintf("%d = hexadecimal %x = octal %o = %b binary", n, n, n, n);
+ *! (3) Result: "4711 = hexadecimal 1267 = octal 11147 = 1001001100111 binary"
+ *!
+ *! @note
+ *!   sprintf style formatting is applied by many formatting functions, such
+ *!   @[write()] and @[werror].
+ *!
+ *! @example
+ *! > write(#"Formatting examples:
+ *! Left adjusted  [%-10d]
+ *! Centered       [%|10d]
+ *! Right adjusted [%10d]
+ *! Zero padded    [%010d]
+ *! ", n, n, n, n);
+ *! Formatting examples:
+ *! Left adjusted  [4711      ]
+ *! Centered       [   4711   ]
+ *! Right adjusted [      4711]
+ *! Zero padded    [0000004711]
+ *! (5) Result: 142
+ *! int screen_width=70;
+ *! > write("%-=*s\n", screen_width,
+ *! >> "This will wordwrap the specified string within the "+
+ *! >> "specified field size, this is useful say, if you let "+
+ *! >> "users specify their screen size, then the room "+
+ *! >> "descriptions will automagically word-wrap as appropriate.\n"+
+ *! >> "slosh-n's will of course force a new-line when needed.\n");
+ *! This will wordwrap the specified string within the specified field
+ *! size, this is useful say, if you let users specify their screen size,
+ *! then the room descriptions will automagically word-wrap as
+ *! appropriate.
+ *! slosh-n's will of course force a new-line when needed.
+ *! (6) Result: 355
+ *! > write("%-=*s %-=*s\n", screen_width/2,
+ *! >> "Two columns next to each other (any number of columns will "+
+ *! >> "of course work) independantly word-wrapped, can be useful.",
+ *! >> screen_width/2-1,
+ *! >> "The - is to specify justification, this is in addherence "+
+ *! >> "to std sprintf which defaults to right-justification, "+
+ *! >> "this version also supports centre and right justification.");
+ *! Two columns next to each other (any The - is to specify justification,
+ *! number of columns will of course    this is in addherence to std
+ *! work) independantly word-wrapped,   sprintf which defaults to
+ *! can be useful.                      right-justification, this version
+ *!                                     also supports centre and right
+ *!                                     justification.
+ *! (7) Result: 426
+ *! > write("%-$*s\n", screen_width,
+ *! >> "Given a\nlist of\nslosh-n\nseparated\n'words',\nthis option\n"+
+ *! >> "creates a\ntable out\nof them\nthe number of\ncolumns\n"+
+ *! >> "be forced\nby specifying a\npresision.\nThe most obvious\n"+
+ *! >> "use is for\nformatted\nls output.");
+ *! Given a          list of          slosh-n
+ *! separated        'words',         this option
+ *! creates a        table out        of them
+ *! the number of    columns          be forced
+ *! by specifying a  presision.       The most obvious
+ *! use is for       formatted        ls output.
+ *! (8) Result: 312
+ *! > write("%-#*s\n", screen_width,
+ *! >> "Given a\nlist of\nslosh-n\nseparated\n'words',\nthis option\n"+
+ *! >> "creates a\ntable out\nof them\nthe number of\ncolumns\n"+
+ *! >> "be forced\nby specifying a\npresision.\nThe most obvious\n"+
+ *! >> "use is for\nformatted\nls output.");
+ *! Given a          creates a        by specifying a
+ *! list of          table out        presision.
+ *! slosh-n          of them          The most obvious
+ *! separated        the number of    use is for
+ *! 'words',         columns          formatted
+ *! this option      be forced        ls output.
+ *! (9) Result: 312
+ *! > sample = ([ "align":"left", "valign":"middle" ]);
+ *! (10) Result: ([ @xml{/@}* 2 elements *@xml{/@}
+ *!          "align":"left",
+ *!          "valign":"middle"
+ *!        ])
+ *! > write("<td%{ %s='%s'%}>\n", (array)sample);
+ *! <td valign='middle' align='left'>
+ *! (11) Result: 34
+ *! >  write("Of course all the simple printf options "+
+ *! >> "are supported:\n %s: %d %x %o %c\n",
+ *! >> "65 as decimal, hex, octal and a char",
+ *! >> 65, 65, 65, 65);
+ *! Of course all the simple printf options are supported:
+ *!  65 as decimal, hex, octal and a char: 65 41 101 A
+ *! (12) Result: 106
+ *! > write("%[0]d, %[0]x, %[0]X, %[0]o, %[0]c\n", 75);
+ *! 75, 4b, 4B, 113, K
+ *! (13) Result: 19
+ *! > write("%|*s\n",screen_width, "THE END");
+ *!                                THE END
+ *! (14) Result: 71
+ *!
+ *! @seealso
+ *!   @[lfun::_sprintf()]
+ */
 #include "global.h"
-RCSID("$Id: sprintf.c,v 1.78 2003/10/16 16:44:10 grubba Exp $");
+RCSID("$Id: sprintf.c,v 1.119 2004/11/02 14:38:45 aldem Exp $");
 #include "pike_error.h"
 #include "array.h"
 #include "svalue.h"
@@ -120,19 +295,16 @@ RCSID("$Id: sprintf.c,v 1.78 2003/10/16 16:44:10 grubba Exp $");
 #include "builtin_functions.h"
 #include "operators.h"
 #include "opcodes.h"
+#include "cyclic.h"
+#include "module.h"
+#include "pike_float.h"
 #include <ctype.h>
 
-#ifdef PC
-#undef PC
-#endif /* PC */
+#include "config.h"
 
 #include <math.h>
-#ifdef HAVE_IEEEFP_H
-#include <ieeefp.h>
-#endif
-#ifdef HAVE_FP_CLASS_H
-#include <fp_class.h>
-#endif
+
+#define sp Pike_sp
 
 #define FORMAT_INFO_STACK_SIZE 200
 #define RETURN_SHARED_STRING
@@ -170,7 +342,7 @@ struct format_stack
 
 #define FIELD_LEFT 1
 #define FIELD_CENTER 2
-#define PAD_POSITIVE 4 
+#define PAD_POSITIVE 4
 #define LINEBREAK 8
 #define COLUMN_MODE 16
 #define ZERO_PAD 32
@@ -280,11 +452,9 @@ INLINE static void low_write_IEEE_float(char *b, double d, int sz)
     e = maxexp;
   else
 #endif
-#ifdef HAVE_ISNAN
-  if(isnan(d)) {
+  if(PIKE_ISNAN(d)) {
     e = maxexp; f = maxf;
   } else
-#endif
 #ifdef HAVE_ISZERO
   if(iszero(d))
     e = 0;
@@ -458,8 +628,8 @@ INLINE static void fix_field(struct string_builder *r,
   
       if(width>0)
       {
-  	d%=pad_length;
-  	string_builder_fill(r,width,pad_string,pad_length,d);
+	d%=pad_length;
+	string_builder_fill(r, width, pad_string, pad_length, d);
       }
       
       return;
@@ -472,7 +642,7 @@ INLINE static void fix_field(struct string_builder *r,
   e=width-len;
   if(e>0)
   {
-    string_builder_fill(r,e,pad_string, pad_length, 0);
+    string_builder_fill(r, e, pad_string, pad_length, 0);
     width-=e;
   }
 
@@ -507,7 +677,7 @@ static void sprintf_error(struct format_stack *fs,
   va_start(args,s);
   free_sprintf_strings(fs);
 
-  sprintf(buf,"Sprintf: %s",s);
+  sprintf(buf,"sprintf: %s",s);
   va_error(buf,args);
   va_end(args);
 }
@@ -745,7 +915,8 @@ INLINE static int do_one(struct format_stack *fs,
      fs->fsp->pad_string=MKPCHARP(" ",0);				\
      fs->fsp->pad_length=1;						\
      fs->fsp->column_width=0;						\
-     fs->fsp->pos_pad=fs->fsp->flags=0;					\
+     fs->fsp->pos_pad=0;						\
+     fs->fsp->flags=0;							\
      fs->fsp->width=fs->fsp->precision=SPRINTF_UNDECIDED;		\
      UNSET_ONERROR(_e);							\
      break;                                                             \
@@ -760,50 +931,58 @@ INLINE static int do_one(struct format_stack *fs,
 	  if(sv->type == T_OBJECT && sv->u.object->prog)		      \
 	  {                                                                   \
             ptrdiff_t fun=FIND_LFUN(sv->u.object->prog, LFUN__SPRINTF);	      \
-	    if (fun != -1)                                                    \
-            {                                                                 \
-              int n=0;							      \
-	      push_int(EXTRACT_PCHARP(a));				      \
-	      if (fs->fsp->precision!=SPRINTF_UNDECIDED)		      \
-	      {								      \
-		 push_constant_text("precision");			      \
-		 push_int(fs->fsp->precision);				      \
-                 n+=2;							      \
-	      }								      \
-	      if (fs->fsp->width!=SPRINTF_UNDECIDED)			      \
-	      {								      \
-		 push_constant_text("width");	           		      \
-		 push_int64(fs->fsp->width);				      \
-                 n+=2;							      \
-	      }								      \
-	      if ((fs->fsp->flags&FIELD_LEFT))				      \
-	      {								      \
-		 push_constant_text("flag_left");	       		      \
-		 push_int(1);						      \
-                 n+=2;							      \
-	      }								      \
-	      f_aggregate_mapping(n);					      \
+	    if (fun != -1) {						      \
+              DECLARE_CYCLIC();						      \
+              if (!BEGIN_CYCLIC(sv->u.object, fun))			      \
+              {                                                               \
+              	int n=0;						      \
+	      	push_int(EXTRACT_PCHARP(a));				      \
+	      	if (fs->fsp->precision!=SPRINTF_UNDECIDED)		      \
+	      	{							      \
+	      	   push_constant_text("precision");			      \
+	      	   push_int(fs->fsp->precision);			      \
+              	   n+=2;						      \
+	      	}							      \
+	      	if (fs->fsp->width!=SPRINTF_UNDECIDED)			      \
+	      	{							      \
+	      	   push_constant_text("width");	           		      \
+	      	   push_int64(fs->fsp->width);				      \
+              	   n+=2;						      \
+	      	}							      \
+	      	if ((fs->fsp->flags&FIELD_LEFT))			      \
+	      	{							      \
+	      	   push_constant_text("flag_left");	       		      \
+	      	   push_int(1);						      \
+              	   n+=2;						      \
+	      	}							      \
+	      	f_aggregate_mapping(n);					      \
+	      								      \
+	      	SET_CYCLIC_RET(1);					      \
+	      	apply_low(sv->u.object, fun, 2);                              \
 									      \
-	      apply_low(sv->u.object, fun, 2);                                \
-	      if(sp[-1].type == T_STRING)				      \
-	      {	                                                              \
-                DO_IF_DEBUG( if(fs->fsp->to_free_string)                      \
-                             fatal("OOps in sprintf\n"); )                    \
-                fs->fsp->to_free_string = (--sp)->u.string;	              \
-									      \
-		fs->fsp->b = MKPCHARP_STR(fs->fsp->to_free_string);           \
-		fs->fsp->len = fs->fsp->to_free_string->len;		      \
-	       								      \
-                /* We have to lift one argument from the format stack. */     \
-                GET_SVALUE(sv);                                               \
-		break;							      \
-	      }								      \
-              if(!IS_ZERO(sp-1))					      \
-	      {								      \
-		 sprintf_error(fs,"argument %d (object) returned "	      \
-			       "illegal value from _sprintf()\n",argument+1); \
-	      }								      \
-	      pop_stack();						      \
+	      	if(sp[-1].type == T_STRING)				      \
+	      	{	                                                      \
+              	  DO_IF_DEBUG( if(fs->fsp->to_free_string)                    \
+              		       Pike_fatal("OOps in sprintf\n"); )                  \
+              	  fs->fsp->to_free_string = (--sp)->u.string;	              \
+	      								      \
+	      	  fs->fsp->b = MKPCHARP_STR(fs->fsp->to_free_string);	      \
+	      	  fs->fsp->len = fs->fsp->to_free_string->len;		      \
+	      								      \
+              	  /* We have to lift one argument from the format stack. */   \
+              	  GET_SVALUE(sv);                                             \
+                  END_CYCLIC();						      \
+	      	  break;						      \
+	      	}							      \
+	      	if(!SAFE_IS_ZERO(sp-1))					      \
+	      	{							      \
+	      	   sprintf_error(fs,"argument %d (object) returned "	      \
+	      			 "illegal value from _sprintf()\n",	      \
+				 argument+1);				      \
+	      	}							      \
+	      	pop_stack();						      \
+              }								      \
+              END_CYCLIC();						      \
 	    }								      \
 	  }								      \
 	}
@@ -822,7 +1001,7 @@ static void low_pike_sprintf(struct format_stack *fs,
 			     int nosnurkel)
 {
   int argument=0;
-  int tmp,setwhat,pos,d,e;
+  int tmp,setwhat,d,e;
   char buffer[140];
   struct format_info *f,*start;
   double tf;
@@ -842,14 +1021,15 @@ static void low_pike_sprintf(struct format_stack *fs,
     fs->fsp++;
 #ifdef PIKE_DEBUG
     if(fs->fsp < fs->format_info_stack)
-      fatal("sprintf: fs->fsp out of bounds.\n");
+      Pike_fatal("sprintf: fs->fsp out of bounds.\n");
 #endif
     fs->fsp->pad_string=MKPCHARP(" ",0);
     fs->fsp->pad_length=1;
     fs->fsp->fi_free_string=0;
     fs->fsp->to_free_string=0;
     fs->fsp->column_width=0;
-    fs->fsp->pos_pad=fs->fsp->flags=0;
+    fs->fsp->pos_pad = 0;
+    fs->fsp->flags = 0;
     fs->fsp->width=fs->fsp->precision=SPRINTF_UNDECIDED;
 
     if(EXTRACT_PCHARP(a)!='%')
@@ -864,7 +1044,7 @@ static void low_pike_sprintf(struct format_stack *fs,
     }
     num_snurkel=0;
     arg=NULL;
-    setwhat=pos=0;
+    setwhat=0;
     begin=a;
 
     for(INC_PCHARP(a,1);;INC_PCHARP(a,1))
@@ -880,7 +1060,7 @@ static void low_pike_sprintf(struct format_stack *fs,
 	}else{
 	  sprintf_error(fs, "Error in format string, \\%o is not a format.\n",EXTRACT_PCHARP(a));
 	}
-	fatal("Foo, you shouldn't be here!\n");
+	Pike_fatal("Foo, you shouldn't be here!\n");
 
         /* First the modifiers */
 
@@ -1000,13 +1180,11 @@ static void low_pike_sprintf(struct format_stack *fs,
 	DO_OP();
 	for(e=1,tmp=1;tmp;e++)
 	{
-	  if(!INDEX_PCHARP(a,e))
-	  {
+	  if (!INDEX_PCHARP(a,e) &&
+	      !COMPARE_PCHARP(ADD_PCHARP(a,e),<,format_end)) {
 	    sprintf_error(fs, "Missing %%} in format string.\n");
 	    break;		/* UNREACHED */
-	  }
-	  if(INDEX_PCHARP(a,e)=='%')
-	  {
+	  } else if(INDEX_PCHARP(a,e)=='%') {
 	    switch(INDEX_PCHARP(a,e+1))
 	    {
 	    case '%': e++; break;
@@ -1046,9 +1224,9 @@ static void low_pike_sprintf(struct format_stack *fs,
 	  }
 #ifdef PIKE_DEBUG
 	  if(fs->fsp < fs->format_info_stack)
-	    fatal("sprintf: fs->fsp out of bounds.\n");
+	    Pike_fatal("sprintf: fs->fsp out of bounds.\n");
 	  if(fs->fsp!=fsp_save)
-	    fatal("sprintf: fs->fsp incorrect after recursive sprintf.\n");
+	    Pike_fatal("sprintf: fs->fsp incorrect after recursive sprintf.\n");
 #endif
 	  fs->fsp->b=MKPCHARP_STR(b.s);
 	  fs->fsp->len=b.s->len;
@@ -1084,7 +1262,7 @@ static void low_pike_sprintf(struct format_stack *fs,
 
       case 'c':
       {
-        INT32 tmp;
+        INT_TYPE tmp;
 	ptrdiff_t l,n;
 	char *x;
         DO_OP();
@@ -1204,9 +1382,9 @@ static void low_pike_sprintf(struct format_stack *fs,
 	  }
 	}
 	else if(mode == 'u')
-	  sprintf(x, "%u", val);
+	  sprintf(x, "%"PRINTPIKEINT"u", (unsigned INT_TYPE) val);
 	else
-	  sprintf(x, "%d", val);
+	  sprintf(x, "%"PRINTPIKEINT"d", val);
 
 	fs->fsp->len=strlen(x);
 	break;
@@ -1221,12 +1399,37 @@ static void low_pike_sprintf(struct format_stack *fs,
 	char *x;
 	DO_OP();
 	CHECK_OBJECT_SPRINTF()
+	GET_FLOAT(tf);
+
+	/* Special casing for infinity and not a number,
+	 * since many libc's forget about them...
+	 */
+	if (PIKE_ISNAN(tf)) {
+	  /* NaN */
+	  fs->fsp->b = MKPCHARP("nan", 0);
+	  fs->fsp->len = 3;
+	  break;
+	} else if (tf && (tf+tf == tf)) {
+	  /* Infinity. */
+	  if (tf > 0.0) {
+	    fs->fsp->b = MKPCHARP("inf", 0);
+	    fs->fsp->len = 3;
+	  } else {
+	    fs->fsp->b = MKPCHARP("-inf", 0);
+	    fs->fsp->len = 4;
+	  }
+	  break;
+	}
+
 	if (fs->fsp->precision==SPRINTF_UNDECIDED) fs->fsp->precision=3;
 
-	x=(char *)xalloc(100+MAXIMUM(fs->fsp->precision,3));
+	/* FIXME: The constant (320) is good for IEEE double precision
+	 * float, but will definitely fail for bigger precision! --aldem
+	 */
+	x=(char *)xalloc(320+MAXIMUM(fs->fsp->precision,3));
+	fs->fsp->fi_free_string=x;
 	fs->fsp->b=MKPCHARP(x,0);
 	sprintf(buffer,"%%*.*%c",EXTRACT_PCHARP(a));
-	GET_FLOAT(tf);
 
 	if(fs->fsp->precision<0) {
 	  double m=pow(10.0, (double)fs->fsp->precision);
@@ -1234,8 +1437,10 @@ static void low_pike_sprintf(struct format_stack *fs,
 	} else if (fs->fsp->precision==0) {
 	  tf = RINT(tf);
         }
-	
+
+	debug_malloc_touch(x);
 	sprintf(x,buffer,1,fs->fsp->precision<0?0:fs->fsp->precision,tf);
+	debug_malloc_touch(x);
 	fs->fsp->len=strlen(x);
 	
 	/* Make sure that the last digits really are zero. */
@@ -1254,8 +1459,6 @@ static void low_pike_sprintf(struct format_stack *fs,
 	      break;
 	    }
 	}
-
-	fs->fsp->fi_free_string=x;
 	break;
       }
 
@@ -1315,21 +1518,43 @@ static void low_pike_sprintf(struct format_stack *fs,
 
       case 'O':
       {
-	string s;
+	dynamic_buffer save_buf;
+	dynbuf_string s;
 	struct svalue *t;
 	DO_OP();
 	/* No need to do CHECK_OBJECT_SPRINTF() here,
 	   it is checked in describe_svalue. */
 	GET_SVALUE(t);
-	init_buf();
+	init_buf(&save_buf);
 	describe_svalue(t,0,0);
-	s=complex_free_buf();
+	s=complex_free_buf(&save_buf);
 	fs->fsp->b=MKPCHARP(s.str,0);
 	fs->fsp->len=s.len;
 	fs->fsp->fi_free_string=s.str;
 	break;
       }
 
+#if 0
+      /* This can be useful when doing low level debugging. */
+      case 'p':
+      {
+	dynamic_buffer save_buf;
+	dynbuf_string s;
+	char buf[50];
+	struct svalue *t;
+	DO_OP();
+	GET_SVALUE(t);
+	init_buf(&save_buf);
+	sprintf (buf, "%p", t->u.refs);
+	my_strcat (buf);
+	s=complex_free_buf(&save_buf);
+	fs->fsp->b=MKPCHARP(s.str,0);
+	fs->fsp->len=s.len;
+	fs->fsp->fi_free_string=s.str;
+	break;
+      }
+#endif
+
       case 's':
       {
 	struct pike_string *s;
@@ -1412,7 +1637,7 @@ static void low_pike_sprintf(struct format_stack *fs,
   {
 #ifdef PIKE_DEBUG
     if(fs->fsp < fs->format_info_stack)
-      fatal("sprintf: fsp out of bounds.\n");
+      Pike_fatal("sprintf: fsp out of bounds.\n");
 #endif
     
     if(fs->fsp->fi_free_string)
@@ -1429,9 +1654,14 @@ static void low_pike_sprintf(struct format_stack *fs,
 
 
 /* An C-callable pike_sprintf
-string pike_sprintf(char *format,struct svalue *argp,int num_arg)
+
+** This doesn't seem to work at all.  The prototype for
+** low_pike_sprintf has changed _a lot_ since this code was
+** written.  It should probably be removed.  // Marcus
+
+dynbuf_string pike_sprintf(char *format,struct svalue *argp,int num_arg)
 {
-  string prefix;
+  dynbuf_string prefix;
   prefix.str=0;
   prefix.len=0;
 
@@ -1442,7 +1672,7 @@ string pike_sprintf(char *format,struct svalue *argp,int num_arg)
  */
 
 /* The efun */
-void f_sprintf(INT32 num_arg)
+void f_sprintf(INT32 args)
 {
   ONERROR err_string_builder, err_format_stack;
   struct pike_string *ret;
@@ -1451,7 +1681,7 @@ void f_sprintf(INT32 num_arg)
 
   struct format_stack fs;
 
-  argp=sp-num_arg;
+  argp=sp-args;
   
   fs.fsp = fs.format_info_stack-1;
 
@@ -1469,9 +1699,10 @@ void f_sprintf(INT32 num_arg)
       /* Clean up the stack. */
       pop_stack();
     } else {
-      Pike_error("Bad argument 1 to sprintf.\n");
+      SIMPLE_BAD_ARG_ERROR("sprintf", 1, "string|object");
     }
   }
+/*  fprintf(stderr,"SPRINTF: %s\n",argp->u.string->str); */
 
   init_string_builder(&r,0);
   SET_ONERROR(err_format_stack, free_sprintf_strings, &fs);
@@ -1481,23 +1712,66 @@ void f_sprintf(INT32 num_arg)
 		   MKPCHARP_STR(argp->u.string),
 		   argp->u.string->len,
 		   argp+1,
-		   num_arg-1,
+		   args-1,
 		   0);
   UNSET_ONERROR(err_string_builder);
   UNSET_ONERROR(err_format_stack);
   ret=finish_string_builder(&r);
 
-  pop_n_elems(num_arg);
+  pop_n_elems(args);
   push_string(ret);
 }
 
-void pike_module_init(void)
+static node *optimize_sprintf(node *n)
 {
-  /* function(string|object, mixed ... : string) */
-  ADD_EFUN("sprintf", f_sprintf, tFuncV(tOr(tStr, tObj), tMix, tStr),
-	   OPT_TRY_OPTIMIZE);
+  node **arg0 = my_get_arg(&_CDR(n), 0);
+  node **arg1 = my_get_arg(&_CDR(n), 1);
+  node *ret;
+  int num_args=count_args(CDR(n));
+  if(arg0 && arg1 && num_args == 2 &&
+     (*arg0)->token == F_CONSTANT &&
+     (*arg0)->u.sval.type == T_STRING &&
+     (*arg0)->u.sval.u.string->size_shift == 0 &&
+     (*arg0)->u.sval.u.string->len == 2 &&
+     STR0((*arg0)->u.sval.u.string)[0]=='%')
+  {
+    switch(STR0((*arg0)->u.sval.u.string)[1])
+    {
+      case 'c':
+	ADD_NODE_REF2(*arg1,
+		      ret = mkefuncallnode("int2char",*arg1);
+	  );
+	return ret;
+
+      case 't':
+	ADD_NODE_REF2(*arg1,
+		      ret = mkefuncallnode("basetype",*arg1);
+	  );
+	return ret;
+
+      case 'x':
+	ADD_NODE_REF2(*arg1,
+		      ret = mkefuncallnode("int2hex",*arg1);
+	  );
+	return ret;
+
+      default: break;
+    }
+  }
+  return 0;
 }
 
-void pike_module_exit(void)
+PIKE_MODULE_INIT
+{
+  /* function(string|object, mixed ... : string) */
+  ADD_EFUN2("sprintf", 
+	    f_sprintf,
+	    tFuncV(tOr(tStr, tObj), tMix, tStr),
+	    OPT_TRY_OPTIMIZE,
+	    optimize_sprintf,
+	    0);
+}
+
+PIKE_MODULE_EXIT
 {
 }
