@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: peep.c,v 1.88 2004/05/11 15:47:39 grubba Exp $
+|| $Id: peep.c,v 1.99 2005/05/18 12:36:53 mast Exp $
 */
 
 #include "global.h"
@@ -26,7 +26,7 @@
 #include "interpret.h"
 #include "pikecode.h"
 
-RCSID("$Id: peep.c,v 1.88 2004/05/11 15:47:39 grubba Exp $");
+RCSID("$Id: peep.c,v 1.99 2005/05/18 12:36:53 mast Exp $");
 
 static void asm_opt(void);
 
@@ -179,8 +179,7 @@ INT32 assemble(int store_linenumbers)
       if (c[e].opcode == F_SYNCH_MARK) synch_depth++;
     }
     if (synch_depth) {
-      Pike_fatal("Unbalanced sync_mark/pop_sync_mark: %d\n",
-		 synch_depth);
+      Pike_fatal("Unbalanced sync_mark/pop_sync_mark: %d\n", synch_depth);
     }
   }
 #endif
@@ -422,8 +421,9 @@ INT32 assemble(int store_linenumbers)
   {
 #ifdef PIKE_DEBUG
     if (c != (((p_instr *)instrbuf.s.str)+e)) {
-      Pike_fatal("Instruction loop deviates. 0x%04x != 0x%04x\n",
-		 e, DO_NOT_WARN((INT32)(c - ((p_instr *)instrbuf.s.str))));
+      Pike_fatal("Instruction loop deviates. "
+		 "0x%04"PRINTPTRDIFFT"x != 0x%04"PRINTPTRDIFFT"x\n",
+		 e, c - ((p_instr *)instrbuf.s.str));
     }
     if((a_flag > 2 && store_linenumbers) || a_flag > 3)
     {
@@ -506,7 +506,7 @@ INT32 assemble(int store_linenumbers)
     default:
       switch(instrs[c->opcode - F_OFFSET].flags & I_IS_MASK)
       {
-      case I_ISJUMP:
+      case I_ISPTRJUMP:
 #ifdef INS_F_JUMP
 	tmp=INS_F_JUMP(c->opcode, (labels[c->arg] != -1));
 	if(tmp != -1)
@@ -528,13 +528,12 @@ INT32 assemble(int store_linenumbers)
 	jumps[c->arg]=tmp;
 	break;
 
-      case I_ISJUMPARGS:
+      case I_ISPTRJUMPARGS:
 #ifdef INS_F_JUMP_WITH_TWO_ARGS
 	tmp = INS_F_JUMP_WITH_TWO_ARGS(c->opcode, c->arg, c->arg2,
 				       (labels[c[1].arg] != -1));
 	if(tmp != -1)
 	{
-
 	  /* Step ahead to the pointer instruction, and inline it. */
 #ifdef PIKE_DEBUG
 	  if (c[1].opcode != F_POINTER) {
@@ -557,15 +556,15 @@ INT32 assemble(int store_linenumbers)
 	 */
 
       case I_TWO_ARGS:
+      case I_ISJUMPARGS:
 	ins_f_byte_with_2_args(c->opcode, c->arg, c->arg2);
 	break;
 
-      case I_ISJUMPARG:
+      case I_ISPTRJUMPARG:
 #ifdef INS_F_JUMP_WITH_ARG
 	tmp = INS_F_JUMP_WITH_ARG(c->opcode, c->arg, (labels[c[1].arg] != -1));
 	if(tmp != -1)
 	{
-
 	  /* Step ahead to the pointer instruction, and inline it. */
 #ifdef PIKE_DEBUG
 	  if (c[1].opcode != F_POINTER) {
@@ -588,10 +587,12 @@ INT32 assemble(int store_linenumbers)
 	 */
 
       case I_HASARG:
+      case I_ISJUMPARG:
 	ins_f_byte_with_arg(c->opcode, c->arg);
 	break;
 
       case 0:
+      case I_ISJUMP:
 	ins_f_byte(c->opcode);
 	break;
 
@@ -602,7 +603,6 @@ INT32 assemble(int store_linenumbers)
       }
     }
 
-
 #ifdef PIKE_DEBUG
     if (instrs[c->opcode - F_OFFSET].flags & I_HASPOINTER) {
       if ((e+1 >= length) || (c[1].opcode != F_POINTER)) {
@@ -619,7 +619,7 @@ INT32 assemble(int store_linenumbers)
     if(e+1 < length)
     {
       /* FIXME: Note that this code won't work for opcodes of type
-       *        I_ISJUMPARG or I_ISJUMPARGS, since c may already
+       *        I_ISPTRJUMPARG or I_ISPTRJUMPARGS, since c may already
        *        have been advanced to the corresponding F_POINTER.
        *        With the current opcode set this is a non-issue, but...
        * /grubba 2002-11-02
@@ -684,7 +684,7 @@ INT32 assemble(int store_linenumbers)
   if (a_flag > 6) {
     size_t len = (Pike_compiler->new_program->num_program - fun_start)*
       sizeof(PIKE_OPCODE_T);
-    fprintf(stderr, "Code at offset %d through %d:\n",
+    fprintf(stderr, "Code at offset %"PRINTSIZET"d through %"PRINTSIZET"d:\n",
 	    fun_start, Pike_compiler->new_program->num_program-1);
 #ifdef DISASSEMBLE_CODE
     DISASSEMBLE_CODE(Pike_compiler->new_program->program + fun_start, len);
@@ -961,6 +961,7 @@ static void do_optimization(int topop, ...)
   DO_OPTIMIZATION_POSTQUEL(q);
 }
 
+#include "peep_engine.c"
 
 static void asm_opt(void)
 {
@@ -986,9 +987,35 @@ static void asm_opt(void)
   }
 #endif
 
-#ifndef IN_TPIKE
-#include "peep_engine.c"
-#endif /* IN_TPIKE */
+  len=instrbuf.s.len/sizeof(p_instr);
+  instructions=(p_instr *)instrbuf.s.str;
+  instrbuf.s.str=0;
+  fifo_len=0;
+  init_bytecode();
+
+  for(eye=0;eye<len || fifo_len;)
+  {
+
+#ifdef PIKE_DEBUG
+    if(a_flag>6) {
+      int e;
+      fprintf(stderr, "#%ld,%d:",
+              DO_NOT_WARN((long)eye),
+              fifo_len);
+      for(e=0;e<4;e++) {
+        fprintf(stderr," ");
+        dump_instr(instr(e));
+      }
+      fprintf(stderr,"\n");
+    }
+#endif
+
+    low_asm_opt();
+    advance();
+  }
+
+  for(eye=0;eye<len;eye++) free_string(instructions[eye].file);
+  free((char *)instructions);
 
 #ifdef PIKE_DEBUG
   if(a_flag > 4)
