@@ -1,14 +1,14 @@
-#line 1 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 1 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 /* -*- c -*-
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: Shuffler.cmod,v 1.31 2004/08/27 02:24:28 nilsson Exp $
+|| $Id: Shuffler.cmod,v 1.32 2005/03/22 16:56:28 grubba Exp $
 */
 
 #include "global.h"
 #include "stralloc.h"
-RCSID("$Id: Shuffler.cmod,v 1.31 2004/08/27 02:24:28 nilsson Exp $");
+RCSID("$Id: Shuffler.cmod,v 1.32 2005/03/22 16:56:28 grubba Exp $");
 #include "pike_macros.h"
 #include "interpret.h"
 #include "threads.h"
@@ -83,8 +83,8 @@ extern struct program *Shuffler_program;
 
 #undef class_Shuffle_defined
 #define class_Shuffle_defined
-struct program *Shuffle_program=0;
-int Shuffle_program_fun_num=-1;
+struct program *Shuffle_program=NULL;
+static int Shuffle_program_fun_num=-1;
 
 #undef var_box_Shuffle_defined
 #define var_box_Shuffle_defined
@@ -149,6 +149,11 @@ int Shuffle_program_fun_num=-1;
 static ptrdiff_t Shuffle_storage_offset;
 struct Shuffle_struct {
 
+#ifdef var_box_Shuffle_defined
+#line 84 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+struct fd_callback_box box;
+#endif /* var_box_Shuffle_defined */
+
 #ifdef var_shuffler_Shuffle_defined
   struct object * shuffler;
 
@@ -169,48 +174,43 @@ struct Shuffle_struct {
 
 #endif /* var_request_arg_Shuffle_defined */
 
-#ifdef var_box_Shuffle_defined
-#line 84 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
-struct fd_callback_box box;
-#endif /* var_box_Shuffle_defined */
-
 #ifdef var_current_source_Shuffle_defined
-#line 102 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 102 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct source *current_source;
 #endif /* var_current_source_Shuffle_defined */
 
 #ifdef var_last_source_Shuffle_defined
-#line 103 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 103 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct source *last_source;
 #endif /* var_last_source_Shuffle_defined */
 
 #ifdef var_file_obj_Shuffle_defined
-#line 104 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 104 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct object *file_obj;
 #endif /* var_file_obj_Shuffle_defined */
 
 #ifdef var_callback_Shuffle_defined
-#line 105 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 105 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 int callback;
 #endif /* var_callback_Shuffle_defined */
 
 #ifdef var_write_callback_Shuffle_defined
-#line 106 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 106 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 int write_callback;
 #endif /* var_write_callback_Shuffle_defined */
 
 #ifdef var_sent_Shuffle_defined
-#line 108 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 108 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 int sent;
 #endif /* var_sent_Shuffle_defined */
 
 #ifdef var_state_Shuffle_defined
-#line 109 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 109 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 ShuffleState state;
 #endif /* var_state_Shuffle_defined */
 
 #ifdef var_leftovers_Shuffle_defined
-#line 111 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 111 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct data leftovers;
 #endif /* var_leftovers_Shuffle_defined */
 };
@@ -261,20 +261,20 @@ static void _send_more( struct Shuffle_struct *t );
   #define f_Shuffle_set_throttler_defined
 ptrdiff_t f_Shuffle_set_throttler_fun_num = 0;
 void f_Shuffle_set_throttler(INT32 args) {
-#line 158 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 158 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct object * t;
-#line 158 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 158 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 1) wrong_number_of_args_error("set_throttler",args,1);
-#line 158 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 158 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(Pike_sp[0-1].type != PIKE_T_OBJECT) SIMPLE_BAD_ARG_ERROR("set_throttler",1,"object");
-#line 158 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 158 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 debug_malloc_pass(t=Pike_sp[0-1].u.object);
 /*! @decl void set_throttler(Throttler t)
    *! Calling this function overrides the @[Shuffler] global throttler.
    *! 
   */
     
-#line 164 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 164 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     SHUFFLE_DEBUG2("set_throttler(%p)\n", THIS, t );
     if( THIS->throttler )
@@ -289,25 +289,25 @@ debug_malloc_pass(t=Pike_sp[0-1].u.object);
 #define f_Shuffle_sent_data_defined
 ptrdiff_t f_Shuffle_sent_data_fun_num = 0;
 void f_Shuffle_sent_data(INT32 args) {
-#line 174 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 174 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 0) wrong_number_of_args_error("sent_data",args,0);
 /*! @decl int sent_data()
    *! Returns the amount of data that has been sent so far.
    *! 
   */
     
-#line 180 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 180 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     SHUFFLE_DEBUG2("sent_data() --> %d\n", THIS, THIS->sent );
     do { INT_TYPE ret_=(THIS->sent);  push_int(ret_); return; }while(0);
-#line 183 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 183 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 }
 
   }
 #define f_Shuffle_state_defined
 ptrdiff_t f_Shuffle_state_fun_num = 0;
 void f_Shuffle_state(INT32 args) {
-#line 185 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 185 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 0) wrong_number_of_args_error("state",args,0);
 /*! @decl int state()
    *! Returns the current state of the shuffler.
@@ -322,11 +322,11 @@ if(args != 0) wrong_number_of_args_error("state",args,0);
    *!   
   */
     
-#line 199 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 199 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     SHUFFLE_DEBUG2("state() --> %d\n", THIS, THIS->state );
     do { INT_TYPE ret_=(THIS->state);  push_int(ret_); return; }while(0);
-#line 202 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 202 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 }
 
   }
@@ -337,7 +337,7 @@ if(args != 0) wrong_number_of_args_error("state",args,0);
 #undef Shuffle_event_handler_defined
 #define Shuffle_event_handler_defined
 static void init_Shuffle_struct(void)
-#line 205 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 205 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     THIS->leftovers.do_free = 0;
     THIS->shuffler = 0;
@@ -370,7 +370,7 @@ static void init_Shuffle_struct(void)
 #undef Shuffle_event_handler_defined
 #define Shuffle_event_handler_defined
 static void exit_Shuffle_struct(void)
-#line 231 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 231 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     SHUFFLE_DEBUG1("exit()\n", THIS );
 
@@ -533,11 +533,11 @@ static void exit_Shuffle_struct(void)
   #define f_Shuffle_set_done_callback_defined
 ptrdiff_t f_Shuffle_set_done_callback_fun_num = 0;
 void f_Shuffle_set_done_callback(INT32 args) {
-#line 390 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 390 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct svalue * cb;
-#line 390 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 390 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 1) wrong_number_of_args_error("set_done_callback",args,1);
-#line 390 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 390 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 cb=Pike_sp+0-1; dmalloc_touch_svalue(Pike_sp+0-1);
 /*! @decl void set_done_callback( function(Shuffle,int:void) cb )
     *! Sets the done callback. This function will be called when all
@@ -545,7 +545,7 @@ cb=Pike_sp+0-1; dmalloc_touch_svalue(Pike_sp+0-1);
     *! 
     */
     
-#line 397 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 397 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     SHUFFLE_DEBUG2("set_done_callback(%p)\n", THIS, cb->u.object );
     assign_svalue( &THIS->done_callback,cb);
@@ -555,11 +555,11 @@ cb=Pike_sp+0-1; dmalloc_touch_svalue(Pike_sp+0-1);
 #define f_Shuffle_set_request_arg_defined
 ptrdiff_t f_Shuffle_set_request_arg_fun_num = 0;
 void f_Shuffle_set_request_arg(INT32 args) {
-#line 402 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 402 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct svalue * arg;
-#line 402 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 402 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 1) wrong_number_of_args_error("set_request_arg",args,1);
-#line 402 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 402 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 arg=Pike_sp+0-1; dmalloc_touch_svalue(Pike_sp+0-1);
 /*! @decl void set_request_arg( mixed arg )
     *!
@@ -568,14 +568,14 @@ arg=Pike_sp+0-1; dmalloc_touch_svalue(Pike_sp+0-1);
     *! 
     */
     
-#line 410 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 410 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     SHUFFLE_DEBUG2("set_request_arg(%p)\n", THIS, arg->u.object );
     assign_svalue( &THIS->request_arg, arg );
   }
   
   }
-#line 415 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 415 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 static void _give_back( struct Shuffle_struct *t, int amount )
   {
     SHUFFLE_DEBUG2("_give_back(%d)\n", t, amount );
@@ -698,11 +698,11 @@ static void _give_back( struct Shuffle_struct *t, int amount )
   #define f_Shuffle_send_more_callback_defined
 ptrdiff_t f_Shuffle_send_more_callback_fun_num = 0;
 void f_Shuffle_send_more_callback(INT32 args) {
-#line 534 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 534 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 INT_TYPE amount;
-#line 534 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 534 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 1) wrong_number_of_args_error("send_more_callback",args,1);
-#line 534 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 534 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(Pike_sp[0-1].type != PIKE_T_INT) SIMPLE_BAD_ARG_ERROR("send_more_callback",1,"int");
 amount=Pike_sp[0-1].u.integer;
 {
@@ -722,12 +722,12 @@ amount=Pike_sp[0-1].u.integer;
   #define f_Shuffle_write_callback_defined
 ptrdiff_t f_Shuffle_write_callback_fun_num = 0;
 void f_Shuffle_write_callback(INT32 args) {
-#line 549 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 549 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct svalue * x;
-#line 549 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 549 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args > 1) wrong_number_of_args_error("write_callback",args,1);
 if (args > 0) {
-#line 549 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 549 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 x=Pike_sp+0-args; dmalloc_touch_svalue(Pike_sp+0-args);
 } else x=0;
 {
@@ -742,23 +742,23 @@ x=Pike_sp+0-args; dmalloc_touch_svalue(Pike_sp+0-args);
   #define f_Shuffle_create_defined
 ptrdiff_t f_Shuffle_create_fun_num = 0;
 void f_Shuffle_create(INT32 args) {
-#line 559 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 559 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct object * fd;
 struct object * shuffler;
 struct svalue * throttler;
 struct svalue * backend;
-#line 559 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 559 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 4) wrong_number_of_args_error("create",args,4);
-#line 559 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 559 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(Pike_sp[0-4].type != PIKE_T_OBJECT) SIMPLE_BAD_ARG_ERROR("create",1,"object");
-#line 559 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 559 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 debug_malloc_pass(fd=Pike_sp[0-4].u.object);
 if(Pike_sp[1-4].type != PIKE_T_OBJECT) SIMPLE_BAD_ARG_ERROR("create",2,"object");
-#line 560 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 560 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 debug_malloc_pass(shuffler=Pike_sp[1-4].u.object);
 throttler=Pike_sp+2-4; dmalloc_touch_svalue(Pike_sp+2-4);
 backend=Pike_sp+3-4; dmalloc_touch_svalue(Pike_sp+3-4);
-#line 564 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 564 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     struct Backend_struct*be = default_backend;
 
@@ -813,14 +813,14 @@ backend=Pike_sp+3-4; dmalloc_touch_svalue(Pike_sp+3-4);
 #define f_Shuffle_start_defined
 ptrdiff_t f_Shuffle_start_fun_num = 0;
 void f_Shuffle_start(INT32 args) {
-#line 614 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 614 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 0) wrong_number_of_args_error("start",args,0);
 /*! @decl void start();
     *! Start sending data from the sources.
     *! 
     */
     
-#line 620 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 620 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     if( !THIS->file_obj )
       Pike_error("Cannot start, no destination.\n");
@@ -833,14 +833,14 @@ if(args != 0) wrong_number_of_args_error("start",args,0);
 #define f_Shuffle_pause_defined
 ptrdiff_t f_Shuffle_pause_fun_num = 0;
 void f_Shuffle_pause(INT32 args) {
-#line 628 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 628 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 0) wrong_number_of_args_error("pause",args,0);
 /*! @decl void pause();
     *! Temporarily pause all data transmission
     *! 
     */
     
-#line 634 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 634 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     THIS->state = PAUSED;
     SHUFFLE_DEBUG1("pause()\n", THIS );
@@ -851,14 +851,14 @@ if(args != 0) wrong_number_of_args_error("pause",args,0);
 #define f_Shuffle_stop_defined
 ptrdiff_t f_Shuffle_stop_fun_num = 0;
 void f_Shuffle_stop(INT32 args) {
-#line 640 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 640 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 0) wrong_number_of_args_error("stop",args,0);
 /*! @decl void stop();
     *! Stop all data transmission, and then call the done callback
     *! 
     */
     
-#line 646 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 646 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     SHUFFLE_DEBUG1("stop()\n", THIS );
     _all_done( THIS, 2 );
@@ -868,24 +868,24 @@ if(args != 0) wrong_number_of_args_error("stop",args,0);
 #define f_Shuffle_add_source_defined
 ptrdiff_t f_Shuffle_add_source_fun_num = 0;
 void f_Shuffle_add_source(INT32 args) {
-#line 651 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 651 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct svalue * source;
-#line 651 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 651 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct svalue * start;
-#line 651 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 651 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct svalue * length;
-#line 651 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 651 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args < 1) wrong_number_of_args_error("add_source",args,1);
-#line 651 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 651 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args > 3) wrong_number_of_args_error("add_source",args,3);
-#line 651 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 651 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 source=Pike_sp+0-args; dmalloc_touch_svalue(Pike_sp+0-args);
 if (args > 1) {
-#line 651 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 651 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 start=Pike_sp+1-args; dmalloc_touch_svalue(Pike_sp+1-args);
 } else start=0;
 if (args > 2) {
-#line 651 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 651 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 length=Pike_sp+2-args; dmalloc_touch_svalue(Pike_sp+2-args);
 } else length=0;
 /*! @decl void add_source( mixed source, int|void start, int|void length );
@@ -1000,8 +1000,8 @@ static void Shuffle_event_handler(int ev) {
 
 #undef class_Shuffler_defined
 #define class_Shuffler_defined
-struct program *Shuffler_program=0;
-int Shuffler_program_fun_num=-1;
+struct program *Shuffler_program=NULL;
+static int Shuffler_program_fun_num=-1;
 
 #undef var_backend_Shuffler_defined
 #define var_backend_Shuffler_defined
@@ -1039,17 +1039,17 @@ struct Shuffler_struct {
 
 #endif /* var_throttler_Shuffler_defined */
 
+#ifdef var_paused_Shuffler_defined
+#line 749 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+int paused;
+#endif /* var_paused_Shuffler_defined */
+
 #ifdef var_sources_Shuffler_defined
   struct array * sources;
 
 #endif /* var_sources_Shuffler_defined */
-
-#ifdef var_paused_Shuffler_defined
-#line 749 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
-int paused;
-#endif /* var_paused_Shuffler_defined */
 };
-#line 753 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 753 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 static void update_sources()
   {
     push_array( THIS->sources );
@@ -1066,13 +1066,13 @@ static void update_sources()
   #define f_Shuffler_set_backend_defined
 ptrdiff_t f_Shuffler_set_backend_fun_num = 0;
 void f_Shuffler_set_backend(INT32 args) {
-#line 766 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 766 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct object * b;
-#line 766 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 766 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 1) wrong_number_of_args_error("set_backend",args,1);
-#line 766 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 766 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(Pike_sp[0-1].type != PIKE_T_OBJECT) SIMPLE_BAD_ARG_ERROR("set_backend",1,"object");
-#line 766 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 766 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 debug_malloc_pass(b=Pike_sp[0-1].u.object);
 /*! @decl void set_backend( Pike.Backend b );
      *! Set the backend that will be used by all @[Shuffle] objects created
@@ -1092,17 +1092,17 @@ debug_malloc_pass(b=Pike_sp[0-1].u.object);
 #define f_Shuffler_set_throttler_defined
 ptrdiff_t f_Shuffler_set_throttler_fun_num = 0;
 void f_Shuffler_set_throttler(INT32 args) {
-#line 781 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
-struct svalue * t;
-#line 781 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 781 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+struct object * t;
+#line 781 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args > 1) wrong_number_of_args_error("set_throttler",args,1);
 if ((args > 0) && 
     ((Pike_sp[0-args].type != PIKE_T_INT) ||
      (Pike_sp[0-args].u.integer))) {
-#line 781 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 781 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(Pike_sp[0-args].type != PIKE_T_OBJECT) SIMPLE_BAD_ARG_ERROR("set_throttler",1,"object|void");
-#line 781 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
-t=Pike_sp+0-args; dmalloc_touch_svalue(Pike_sp+0-args);
+#line 781 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+debug_malloc_pass(t=Pike_sp[0-args].u.object);
 } else t=0;
 /*! @decl void set_throttler( Throttler t );
      *! Set the throttler that will be used in all @[Shuffle] objects
@@ -1111,13 +1111,11 @@ t=Pike_sp+0-args; dmalloc_touch_svalue(Pike_sp+0-args);
      *! 
      */
   
-#line 788 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 788 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     if( THIS->throttler )
       free_object( THIS->throttler );
-    THIS->throttler = NULL;
-    if( t->type == PIKE_T_OBJECT )
-      THIS->throttler = t->u.object;
+    THIS->throttler = t;
     debug_malloc_touch( THIS->throttler );
     Pike_sp--;
     push_int(0);
@@ -1127,7 +1125,7 @@ t=Pike_sp+0-args; dmalloc_touch_svalue(Pike_sp+0-args);
 #define f_Shuffler_pause_defined
 ptrdiff_t f_Shuffler_pause_fun_num = 0;
 void f_Shuffler_pause(INT32 args) {
-#line 799 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 797 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 0) wrong_number_of_args_error("pause",args,0);
 /*! @decl void pause();
      *! Pause all @[Shuffle] objects associated with this @[Shuffler]
@@ -1149,7 +1147,7 @@ if(args != 0) wrong_number_of_args_error("pause",args,0);
 #define f_Shuffler_start_defined
 ptrdiff_t f_Shuffler_start_fun_num = 0;
 void f_Shuffler_start(INT32 args) {
-#line 816 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 814 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 0) wrong_number_of_args_error("start",args,0);
 /*! @decl void start();
      *! Unpause all @[Shuffle] objects associated with this @[Shuffler]
@@ -1171,13 +1169,13 @@ if(args != 0) wrong_number_of_args_error("start",args,0);
 #define f_Shuffler_cq____remove_shuffle_defined
 ptrdiff_t f_Shuffler_cq____remove_shuffle_fun_num = 0;
 void f_Shuffler_cq____remove_shuffle(INT32 args) {
-#line 833 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 831 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct object * so;
-#line 833 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 831 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 1) wrong_number_of_args_error("___remove_shuffle",args,1);
-#line 833 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 831 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(Pike_sp[0-1].type != PIKE_T_OBJECT) SIMPLE_BAD_ARG_ERROR("___remove_shuffle",1,"object");
-#line 833 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 831 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 debug_malloc_pass(so=Pike_sp[0-1].u.object);
 {
     f_aggregate(1);
@@ -1194,13 +1192,13 @@ debug_malloc_pass(so=Pike_sp[0-1].u.object);
 #define f_Shuffler_shuffle_defined
 ptrdiff_t f_Shuffler_shuffle_fun_num = 0;
 void f_Shuffler_shuffle(INT32 args) {
-#line 845 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 843 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 struct object * destination;
-#line 845 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 843 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(args != 1) wrong_number_of_args_error("shuffle",args,1);
-#line 845 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 843 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 if(Pike_sp[0-1].type != PIKE_T_OBJECT) SIMPLE_BAD_ARG_ERROR("shuffle",1,"object");
-#line 845 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 843 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 debug_malloc_pass(destination=Pike_sp[0-1].u.object);
 /*! @decl Shuffle shuffle( Stdio.File destination );
      *! Create a new @[Shuffle] object.
@@ -1236,7 +1234,7 @@ debug_malloc_pass(destination=Pike_sp[0-1].u.object);
 #undef Shuffler_event_handler_defined
 #define Shuffler_event_handler_defined
 static void init_Shuffler_struct(void)
-#line 873 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 871 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 {
     THIS->sources = allocate_array(0);
   }
@@ -1269,15 +1267,17 @@ static void Shuffler_event_handler(int ev) {
 /*! @endmodule
  */
 
-#line 893 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 891 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 PIKE_MODULE_INIT
 {
   
 #ifdef class_Shuffle_defined
 
 #ifdef PROG_SHUFFLE_ID
+#line 82 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   START_NEW_PROGRAM_ID(SHUFFLE);
 #else
+#line 82 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   start_new_program();
 
 #endif /* PROG_SHUFFLE_ID */
@@ -1321,89 +1321,91 @@ PIKE_MODULE_INIT
 #endif /* Shuffle_event_handler_defined */
 
 #ifdef f_Shuffle_set_throttler_defined
-#line 158 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffle_set_throttler_fun_num =
+#line 158 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("set_throttler", f_Shuffle_set_throttler, tFunc(tObj,tVoid), 0, OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffle_set_throttler_defined */
 
 #ifdef f_Shuffle_sent_data_defined
-#line 174 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffle_sent_data_fun_num =
+#line 174 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("sent_data", f_Shuffle_sent_data, tFunc(tNone,"\10\200\0\0\0\177\377\377\377"), 0, OPT_TRY_OPTIMIZE);
 
 #endif /* f_Shuffle_sent_data_defined */
 
 #ifdef f_Shuffle_state_defined
-#line 185 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffle_state_fun_num =
+#line 185 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("state", f_Shuffle_state, tFunc(tNone,"\10\200\0\0\0\177\377\377\377"), 0, OPT_TRY_OPTIMIZE);
 
 #endif /* f_Shuffle_state_defined */
 
 #ifdef f_Shuffle_set_done_callback_defined
-#line 390 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffle_set_done_callback_fun_num =
+#line 390 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("set_done_callback", f_Shuffle_set_done_callback, tFunc(tMix,tVoid), 0, OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffle_set_done_callback_defined */
 
 #ifdef f_Shuffle_set_request_arg_defined
-#line 402 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffle_set_request_arg_fun_num =
+#line 402 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("set_request_arg", f_Shuffle_set_request_arg, tFunc(tMix,tVoid), 0, OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffle_set_request_arg_defined */
 
 #ifdef f_Shuffle_send_more_callback_defined
-#line 534 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffle_send_more_callback_fun_num =
+#line 534 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("send_more_callback", f_Shuffle_send_more_callback, tFunc("\10\200\0\0\0\177\377\377\377",tVoid), 0, OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffle_send_more_callback_defined */
 
 #ifdef f_Shuffle_write_callback_defined
-#line 549 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffle_write_callback_fun_num =
+#line 549 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("write_callback", f_Shuffle_write_callback, tFunc(tOr(tMix,tVoid),tVoid), 0, OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffle_write_callback_defined */
 
 #ifdef f_Shuffle_create_defined
-#line 559 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffle_create_fun_num =
+#line 559 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("create", f_Shuffle_create, tFunc(tObj tObj tMix tMix,tVoid), ID_STATIC, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffle_create_defined */
 
 #ifdef f_Shuffle_start_defined
-#line 614 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffle_start_fun_num =
+#line 614 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("start", f_Shuffle_start, tFunc(tNone,tVoid), 0, OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffle_start_defined */
 
 #ifdef f_Shuffle_pause_defined
-#line 628 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffle_pause_fun_num =
+#line 628 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("pause", f_Shuffle_pause, tFunc(tNone,tVoid), 0, OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffle_pause_defined */
 
 #ifdef f_Shuffle_stop_defined
-#line 640 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffle_stop_fun_num =
+#line 640 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("stop", f_Shuffle_stop, tFunc(tNone,tVoid), 0, OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffle_stop_defined */
 
 #ifdef f_Shuffle_add_source_defined
-#line 651 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffle_add_source_fun_num =
+#line 651 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("add_source", f_Shuffle_add_source, tFunc(tMix tOr(tMix,tVoid) tOr(tMix,tVoid),tVoid), 0, OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffle_add_source_defined */
+#line 82 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   Shuffle_program=end_program();
+#line 82 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   Shuffle_program_fun_num=add_program_constant("Shuffle",Shuffle_program,0);
 
 #endif /* class_Shuffle_defined */
@@ -1411,8 +1413,10 @@ PIKE_MODULE_INIT
 #ifdef class_Shuffler_defined
 
 #ifdef PROG_SHUFFLER_ID
+#line 744 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   START_NEW_PROGRAM_ID(SHUFFLER);
 #else
+#line 744 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   start_new_program();
 
 #endif /* PROG_SHUFFLER_ID */
@@ -1451,51 +1455,53 @@ PIKE_MODULE_INIT
 #endif /* Shuffler_event_handler_defined */
 
 #ifdef f_Shuffler_set_backend_defined
-#line 766 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffler_set_backend_fun_num =
+#line 766 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("set_backend", f_Shuffler_set_backend, tFunc(tObj,tVoid), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffler_set_backend_defined */
 
 #ifdef f_Shuffler_set_throttler_defined
-#line 781 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffler_set_throttler_fun_num =
+#line 781 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("set_throttler", f_Shuffler_set_throttler, tFunc(tOr(tObj,tVoid),tVoid), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffler_set_throttler_defined */
 
 #ifdef f_Shuffler_pause_defined
-#line 799 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffler_pause_fun_num =
+#line 797 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("pause", f_Shuffler_pause, tFunc(tNone,tVoid), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffler_pause_defined */
 
 #ifdef f_Shuffler_start_defined
-#line 816 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffler_start_fun_num =
+#line 814 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("start", f_Shuffler_start, tFunc(tNone,tVoid), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffler_start_defined */
 
 #ifdef f_Shuffler_cq____remove_shuffle_defined
-#line 833 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffler_cq____remove_shuffle_fun_num =
+#line 831 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("___remove_shuffle", f_Shuffler_cq____remove_shuffle, tFunc(tObj,tVoid), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffler_cq____remove_shuffle_defined */
 
 #ifdef f_Shuffler_shuffle_defined
-#line 845 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   f_Shuffler_shuffle_fun_num =
+#line 843 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     ADD_FUNCTION2("shuffle", f_Shuffler_shuffle, tFunc(tObj,tObj), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_Shuffler_shuffle_defined */
+#line 744 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   Shuffler_program=end_program();
+#line 744 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
   Shuffler_program_fun_num=add_program_constant("Shuffler",Shuffler_program,0);
 
 #endif /* class_Shuffler_defined */
-#line 895 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 893 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 ;
   sources_init();
   add_integer_constant( "INITIAL", INITIAL, 0 );
@@ -1512,6 +1518,7 @@ PIKE_MODULE_EXIT
   
 #ifdef class_Shuffle_defined
   if(Shuffle_program) {
+#line 82 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     free_program(Shuffle_program);
     Shuffle_program=0;
   }
@@ -1520,12 +1527,13 @@ PIKE_MODULE_EXIT
 
 #ifdef class_Shuffler_defined
   if(Shuffler_program) {
+#line 744 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
     free_program(Shuffler_program);
     Shuffler_program=0;
   }
 
 #endif /* class_Shuffler_defined */
-#line 908 "/tmp/pikedeb.a8fa1cdb35/7.6/src/post_modules/Shuffler/Shuffler.cmod"
+#line 906 "/tmp/pikedeb.470504af38/7.6/src/post_modules/Shuffler/Shuffler.cmod"
 ;
   sources_exit();
 }
