@@ -2,13 +2,13 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: split.c,v 1.3 2002/10/11 01:40:00 nilsson Exp $
+|| $Id: split.c,v 1.4 2004/04/11 18:51:10 per Exp $
 */
 
 #include "global.h"
 #include "stralloc.h"
 #include "global.h"
-RCSID("$Id: split.c,v 1.3 2002/10/11 01:40:00 nilsson Exp $");
+RCSID("$Id: split.c,v 1.4 2004/04/11 18:51:10 per Exp $");
 #include "pike_macros.h"
 #include "interpret.h"
 #include "program.h"
@@ -21,11 +21,11 @@ RCSID("$Id: split.c,v 1.3 2002/10/11 01:40:00 nilsson Exp $");
 #include "buffer.h"
 #include "split.h"
 
-struct words *uc_words_mkspace( struct words *d, int n )
+static inline struct words *uc_words_mkspace( struct words *d, int n )
 {
   while( d->size+n > d->allocated_size )
   {
-    d->allocated_size += 256;
+    d->allocated_size *= 2;
     d = realloc( d, d->allocated_size*8+(sizeof(struct words)-8) );
   }
   return d;
@@ -57,29 +57,73 @@ void uc_words_free( struct words *w )
 
 #include "wordbits.h"
 
-int unicode_is_wordchar( int c )
+static inline int _unicode_is_wordchar( int c )
 {
   /* Ideographs */
   unsigned int i;
   if( c>=0x5000 && c<= 0x9fff ) /* CJK */
     return 2;
-  
   for( i = 0; i<sizeof(ranges)/sizeof(ranges[0]); i++ )
     if( c <= ranges[i].end )
       return c>=ranges[i].start;
   return 0;
 }
 
+int unicode_is_wordchar( int c )
+{
+  return _unicode_is_wordchar(c);
+}
+
+struct words *unicode_split_words_pikestr0( struct pike_string *data )
+{
+  unsigned int i;
+  unsigned int in_word = 0;
+  unsigned int last_start = 0;
+  struct words *res = uc_words_new();
+  unsigned char *ptr = data->str;
+  unsigned int sz = data->len;
+  
+  for( i=0; i<sz; i++, ptr++ )
+  {
+    switch( _unicode_is_wordchar( *ptr ) )
+    {
+      case 1: /* normal  */
+	if( *ptr > 127 )
+	{
+	  uc_words_free( res );
+	  return NULL;
+	}
+	if( !in_word )
+	{
+	  last_start = i;
+	  in_word=1;
+	}
+	break;
+      case 0: /* not */
+	if( in_word )
+	{
+	  in_word=0;
+	  res = uc_words_write( res, last_start, i-last_start );
+	}
+	break;
+    }
+  }
+  if( in_word )
+    return uc_words_write( res, last_start, i-last_start );
+  return res;
+}
+
 struct words *unicode_split_words_buffer( struct buffer *data )
 {
   unsigned int i;
   unsigned int in_word = 0;
   unsigned int last_start = 0;
   struct words *res = uc_words_new();
-
-  for( i=0; i<data->size; i++ )
+  unsigned int *ptr = data->data;
+  unsigned int sz = data->size;
+  for( i=0; i<sz; i++ )
   {
-    switch( unicode_is_wordchar( data->data[i] ) )
+    switch( _unicode_is_wordchar( *ptr++ ) )
     {
       case 1: /* normal  */
 	if( !in_word )
