@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: stralloc.c,v 1.155 2004/05/18 08:13:15 grubba Exp $
+|| $Id: stralloc.c,v 1.169 2005/01/19 13:06:57 grubba Exp $
 */
 
 #include "global.h"
@@ -24,7 +24,7 @@
 #include <ctype.h>
 #include <math.h>
 
-RCSID("$Id: stralloc.c,v 1.155 2004/05/18 08:13:15 grubba Exp $");
+RCSID("$Id: stralloc.c,v 1.169 2005/01/19 13:06:57 grubba Exp $");
 
 /* #define STRALLOC_USE_PRIMES */
 
@@ -42,6 +42,15 @@ unsigned INT32 htable_mask;
 
 #endif
 
+#if (SIZEOF_LONG == 4) && defined(_LP64)
+/* Kludge for gcc and the system header files not using the same model... */
+#undef LONG_MIN
+#undef LONG_MAX
+#undef ULONG_MAX
+#define LONG_MIN	INT_MIN
+#define LONG_MAX	INT_MAX
+#define ULONG_MAX	UINT_MAX
+#endif
 
 #if PIKE_RUN_UNLOCKED
 /* Make this bigger when we get lightweight threads */
@@ -152,31 +161,39 @@ static INLINE unsigned INT32 generic_extract (const void *str, int size, ptrdiff
   return 0;
 }
 
-PMOD_EXPORT INLINE unsigned INT32 index_shared_string(struct pike_string *s,
-						      ptrdiff_t pos)
+PMOD_EXPORT unsigned INT32 index_shared_string(struct pike_string *s,
+					       ptrdiff_t pos)
 {
 #ifdef PIKE_DEBUG
   if(pos > s->len || pos<0) {
     if (s->len) {
-      Pike_fatal("String index %ld is out of range [0 - %ld]!\n",
-	    DO_NOT_WARN((long)pos),
-	    DO_NOT_WARN((long)s->len-1));
+      Pike_fatal("String index %"PRINTPTRDIFFT"d is out of "
+		 "range 0..%"PRINTPTRDIFFT"d.\n",
+		 pos, s->len-1);
     } else {
-      Pike_fatal("Attempt to index the empty string with %ld!\n",
-	    DO_NOT_WARN((long)pos));
+      Pike_fatal("Attempt to index the empty string with %"PRINTPTRDIFFT"d.\n",
+		 pos);
     }
   }
 #endif
   return generic_extract(s->str,s->size_shift,pos);
 }
 
-PMOD_EXPORT INLINE void low_set_index(struct pike_string *s, ptrdiff_t pos,
-				      int value)
+PMOD_EXPORT void low_set_index(struct pike_string *s, ptrdiff_t pos,
+			       int value)
 {
 #ifdef PIKE_DEBUG
-  if(pos > s->len || pos<0)
-    Pike_fatal("string index out of range!\n");
-  
+  if(pos > s->len || pos<0) {
+    if (s->len) {
+      Pike_fatal("String index %"PRINTPTRDIFFT"d is out of "
+		 "range 0..%"PRINTPTRDIFFT"d.\n",
+		 pos, s->len-1);
+    } else {
+      Pike_fatal("Attempt to index the empty string with %"PRINTPTRDIFFT"d.\n",
+		 pos);
+    }
+  }
+
   if(pos == s->len && value)
     Pike_fatal("string zero termination foul!\n");
 #endif
@@ -191,7 +208,8 @@ PMOD_EXPORT INLINE void low_set_index(struct pike_string *s, ptrdiff_t pos,
 }
 
 #ifdef PIKE_DEBUG
-PMOD_EXPORT INLINE struct pike_string *debug_check_size_shift(struct pike_string *a,int shift)
+PMOD_EXPORT struct pike_string *debug_check_size_shift(struct pike_string *a,
+						       int shift)
 {
   if(a->size_shift != shift)
     Pike_fatal("Wrong STRX macro used!\n");
@@ -200,9 +218,9 @@ PMOD_EXPORT INLINE struct pike_string *debug_check_size_shift(struct pike_string
 #endif
 
 #define CONVERT(FROM,TO) \
-INLINE void PIKE_CONCAT4(convert_,FROM,_to_,TO)(PIKE_CONCAT(p_wchar,TO) *to, const PIKE_CONCAT(p_wchar,FROM) *from, ptrdiff_t len) \
+void PIKE_CONCAT4(convert_,FROM,_to_,TO)(PIKE_CONCAT(p_wchar,TO) *to, const PIKE_CONCAT(p_wchar,FROM) *from, ptrdiff_t len) \
 {  while(--len>=0) *(to++)=*(from++); } \
-INLINE INT32 PIKE_CONCAT4(compare_,FROM,_to_,TO)(const PIKE_CONCAT(p_wchar,TO) *to, const PIKE_CONCAT(p_wchar,FROM) *from, ptrdiff_t len) \
+INT32 PIKE_CONCAT4(compare_,FROM,_to_,TO)(const PIKE_CONCAT(p_wchar,TO) *to, const PIKE_CONCAT(p_wchar,FROM) *from, ptrdiff_t len) \
 { int tmp; while(--len>=0) if((tmp=*(to++)-*(from++))) return tmp; return 0; }
 
 
@@ -276,8 +294,7 @@ PMOD_EXPORT void generic_memcpy(PCHARP to,
   }
 }
 
-PMOD_EXPORT INLINE void pike_string_cpy(PCHARP to,
-			    struct pike_string *from)
+PMOD_EXPORT void pike_string_cpy(PCHARP to, struct pike_string *from)
 {
   generic_memcpy(to,MKPCHARP_STR(from),from->len);
 }
@@ -444,12 +461,26 @@ static struct pike_string *propagate_shared_string(const struct pike_string *s,
 
 static void rehash_string_backwards(struct pike_string *s)
 {
-  ptrdiff_t h;
+  struct pike_string *prev = NULL;
+  struct pike_string *next;
+
   if(!s) return;
-  rehash_string_backwards(s->next);
-  h = HMODULO(s->hval);
-  s->next=base_table[h];
-  base_table[h]=s;
+
+  /* Reverse the hash list. */
+  while ((next = s->next)) {
+    s->next = prev;
+    prev = s;
+    s = next;
+  }
+  s->next = prev;
+
+  /* Rehash the strings for this list. */
+  do {
+    ptrdiff_t h = HMODULO(s->hval);
+    next = s->next;
+    s->next = base_table[h];
+    base_table[h] = s;
+  } while ((s = next));
 }
 
 static void stralloc_rehash(void)
@@ -956,7 +987,7 @@ PMOD_EXPORT void really_free_string(struct pike_string *s)
 
 PMOD_EXPORT void debug_free_string(struct pike_string *s)
 {
-  if(--s->refs<=0)
+  if(!sub_ref(s))
     really_free_string(s);
 }
 
@@ -966,9 +997,10 @@ PMOD_EXPORT void debug_free_string(struct pike_string *s)
  */
 struct pike_string *add_string_status(int verbose)
 {
+  dynamic_buffer save_buf;
   char b[200];
 
-  init_buf();
+  init_buf(&save_buf);
 
   if (verbose)
   {
@@ -1013,7 +1045,7 @@ struct pike_string *add_string_status(int verbose)
       (long)num_str_searches, (double)search_len / num_str_searches);
   my_strcat(b);
 */
-  return free_buf();
+  return free_buf(&save_buf);
 }
 
 /*** PIKE_DEBUG ***/
@@ -1287,7 +1319,15 @@ PMOD_EXPORT int c_compare_string(struct pike_string *s, char *foo, int len)
   return s->len == len && s->size_shift == 0 && !MEMCMP(s->str,foo,len);
 }
 
-#if defined(HAVE_STRCOLL) && !defined(DONT_USE_SYSTEM_LOCALE)
+#ifndef HAVE_STRCOLL
+/* No locale function available */
+static int low_binary_strcmp(char *a, ptrdiff_t alen,
+			     char *b, ptrdiff_t blen)
+{
+  low_quick_binary_strcmp(a,alen,b,blen);
+}
+#else
+
 /* takes locale into account */
 static int low_binary_strcmp(char *a, ptrdiff_t alen,
 			     char *b, ptrdiff_t blen)
@@ -1323,9 +1363,6 @@ PMOD_EXPORT ptrdiff_t my_quick_strcmp(struct pike_string *a,
 /* Does take locale into account */
 PMOD_EXPORT ptrdiff_t my_strcmp(struct pike_string *a,struct pike_string *b)
 {
-#if !defined(HAVE_STRCOLL) || defined(DONT_USE_SYSTEM_LOCALE)
-  return my_quick_strcmp(a, b);
-#else /* HAVE_STRCOLL && !DONT_USE_SYSTEM_LOCALE */
   if(a==b) return 0;
 
   switch(TWO_SIZES(a->size_shift,b->size_shift))
@@ -1341,6 +1378,7 @@ PMOD_EXPORT ptrdiff_t my_strcmp(struct pike_string *a,struct pike_string *b)
 	INT32 ac=index_shared_string(a,e);
 	INT32 bc=index_shared_string(b,e);
 
+#ifdef HAVE_STRCOLL
 	if(ac < 256 && bc < 256)
 	{
 	  char atmp[2],btmp[2];
@@ -1352,12 +1390,12 @@ PMOD_EXPORT ptrdiff_t my_strcmp(struct pike_string *a,struct pike_string *b)
 	  if((tmp=strcoll(atmp,btmp)))
 	     return tmp;
 	}else
+#endif
 	  if(ac-bc) return ac-bc;
       }
       return a->len - b->len;
     }
   }
-#endif /* !HAVE_STRCOLL || DONT_USE_SYSTEM_LOCALE */
 }
 
 PMOD_EXPORT struct pike_string *realloc_unlinked_string(struct pike_string *a,
@@ -1375,7 +1413,7 @@ PMOD_EXPORT struct pike_string *realloc_unlinked_string(struct pike_string *a,
   } else if (size > SHORT_STRING_THRESHOLD) {
     r=(struct pike_string *)realloc((char *)a,
 				    sizeof(struct pike_string)+
-				    ((size+1)<<a->size_shift)); /* FIXME !! */
+				    ((size+1)<<a->size_shift));
   }
 	
   if(!r)
@@ -1547,8 +1585,18 @@ PMOD_EXPORT struct pike_string *modify_shared_string(struct pike_string *a,
 
     if((((unsigned int)index) >= HASH_PREFIX) && (index < a->len-8))
     {
+      struct pike_string *old;
       /* Doesn't change hash value - sneak it in there */
       low_set_index(a,index,c);
+      unlink_pike_string(a);
+      old = internal_findstring(a->str, a->len, a->size_shift, a->hval);
+      if (old) {
+	/* The new string is equal to some old string. */
+	really_free_pike_string(a);
+	add_ref(a = old);
+      } else {
+	link_pike_string(a, a->hval);
+      }
       return a;
     }else{
       unlink_pike_string(a);
@@ -1618,9 +1666,9 @@ PMOD_EXPORT ptrdiff_t string_search(struct pike_string *haystack,
 			   haystack->len,
 			   needle);
 
-  r=mojt.vtab->funcN(mojt.data,
-		     ADD_PCHARP(MKPCHARP_STR(haystack), start),
-		     haystack->len - start).ptr;
+  r = (char *)mojt.vtab->funcN(mojt.data,
+			       ADD_PCHARP(MKPCHARP_STR(haystack), start),
+			       haystack->len - start).ptr;
 
   mojt.vtab->freeme(mojt.data);
 
@@ -1762,7 +1810,7 @@ PMOD_EXPORT struct pike_string *string_replace(struct pike_string *str,
   {
 #ifdef PIKE_DEBUG
     if(tmp + (del->len << str->size_shift) > end)
-      Pike_fatal("generic_memory_search found a match beyond end of string!!!\n");
+      Pike_fatal("SearchMojt found a match beyond end of string!!!\n");
 #endif
     generic_memcpy(r,MKPCHARP(s,str->size_shift),(tmp-s)>>str->size_shift);
     INC_PCHARP(r,(tmp-s)>>str->size_shift);
@@ -1794,11 +1842,9 @@ void init_shared_string_table(void)
   }
 #endif
   empty_pike_string = make_shared_string("");
-
-  initialize_global_buf();
 }
 
-#ifdef DEBUG_MALLOC
+#ifdef DO_PIKE_CLEANUP
 struct shared_string_location *all_shared_string_locations;
 #endif
 
@@ -1813,7 +1859,7 @@ void cleanup_shared_string_table(void)
     empty_pike_string = 0;
   }
 
-#if defined(PIKE_DEBUG) && defined(DEBUG_MALLOC)
+#ifdef DO_PIKE_CLEANUP
   while(all_shared_string_locations)
   {
     struct shared_string_location *x=all_shared_string_locations;
@@ -1822,14 +1868,16 @@ void cleanup_shared_string_table(void)
     x->s=0;
   }
 
-  if(verbose_debug_exit)
+  if (exit_with_cleanup)
   {
     INT32 num,size;
     count_memory_in_strings(&num,&size);
     if(num)
     {
       fprintf(stderr,"Strings left: %d (%d bytes) (zapped)\n",num,size);
+#ifdef PIKE_DEBUG
       dump_stralloc_strings();
+#endif
     }
   }
 #endif
@@ -1948,8 +1996,33 @@ PMOD_EXPORT void init_string_builder_alloc(struct string_builder *s, ptrdiff_t l
   s->known_shift=0;
 }
 
-static void string_build_mkspace(struct string_builder *s,
-				 ptrdiff_t chars, int mag)
+PMOD_EXPORT void init_string_builder_copy(struct string_builder *to,
+					  struct string_builder *from)
+{
+  to->malloced = from->malloced;
+  to->s = begin_wide_shared_string (from->s->len, from->s->size_shift);
+  MEMCPY (to->s->str, from->s->str, from->s->len << from->s->size_shift);
+  to->known_shift = from->known_shift;
+}
+
+/* str becomes invalid if successful (i.e. nonzero returned),
+ * otherwise nothing happens. */
+PMOD_EXPORT int init_string_builder_with_string (struct string_builder *s,
+						 struct pike_string *str)
+{
+  if (str->refs == 1 && str->len > SHORT_STRING_THRESHOLD) {
+    /* Unlink the string and use it as buffer directly. */
+    unlink_pike_string (str);
+    s->s = str;
+    s->malloced = str->len;
+    s->known_shift = str->size_shift;
+    return 1;
+  }
+  return 0;
+}
+
+PMOD_EXPORT void string_build_mkspace(struct string_builder *s,
+				      ptrdiff_t chars, int mag)
 {
   if(mag > s->s->size_shift)
   {
@@ -1963,10 +2036,10 @@ static void string_build_mkspace(struct string_builder *s,
     s->malloced=l;
     s->s=n;
   }
-  else if(((size_t)s->s->len+chars) > ((size_t)s->malloced))
+  else if(s->s->len+chars > s->malloced)
   {
-    size_t newlen = MAXIMUM((size_t)(s->malloced*2),
-			    (size_t)(s->s->len + chars));
+    ptrdiff_t newlen = MAXIMUM(s->malloced*2,
+			       s->s->len + chars);
     ptrdiff_t oldlen = s->s->len;
 
     s->s->len = s->malloced;	/* Restore the real length */
@@ -2217,13 +2290,13 @@ PMOD_EXPORT long STRTOL_PCHARP(PCHARP str, PCHARP *ptr, int base)
     c = EXTRACT_PCHARP(str);		/* skip over leading "0x" or "0X" */
   }
 
+  mul_limit = ((unsigned long)LONG_MAX)/base;
+  add_limit = (int) (LONG_MAX % base);
   if (neg) {
-    mul_limit = (unsigned long) LONG_MIN / base;
-    add_limit = (int) ((unsigned long) LONG_MIN % base);
-  }
-  else {
-    mul_limit = LONG_MAX / base;
-    add_limit = (int) (LONG_MAX % base);
+    if (++add_limit == base) {
+      add_limit = 0;
+      mul_limit++;
+    }
   }
 
   val=DIGIT(c);
@@ -2232,9 +2305,9 @@ PMOD_EXPORT long STRTOL_PCHARP(PCHARP str, PCHARP *ptr, int base)
     INC_PCHARP(str,1);
     c=EXTRACT_PCHARP(str);
     if(!(WIDE_ISALNUM(c)  && (xx=DIGIT(c)) < base)) break;
-    if (val > mul_limit || (val == mul_limit && xx > add_limit))
+    if (val > mul_limit || (val == mul_limit && xx > add_limit)) {
       overflow = 1;
-    else
+    } else
       val = base * val + xx;
   }
 
@@ -2245,9 +2318,7 @@ PMOD_EXPORT long STRTOL_PCHARP(PCHARP str, PCHARP *ptr, int base)
   }
   else {
     if (neg)
-      return val > (unsigned long) LONG_MAX ?
-	-(long) (val - (unsigned long) LONG_MAX) - LONG_MAX :
-	-(long) val;
+      return (long)(~val + 1);
     else
       return (long) val;
   }
@@ -2271,7 +2342,7 @@ PMOD_EXPORT int string_to_svalue_inumber(struct svalue *r,
 
 PMOD_EXPORT int wide_string_to_svalue_inumber(struct svalue *r,
 					      void * str,
-					      void **ptr,
+					      void *ptr,
 					      int base,
 					      ptrdiff_t maxlength,
 					      int shift)
@@ -2282,7 +2353,7 @@ PMOD_EXPORT int wide_string_to_svalue_inumber(struct svalue *r,
 				   &tmp,
 				   base,
 				   maxlength);
-  if(ptr) *ptr=(char *)tmp.ptr;
+  if(ptr) *(p_wchar0 **)ptr=tmp.ptr;
   return ret;
 }
 
@@ -2523,9 +2594,16 @@ PMOD_EXPORT double STRTOD_PCHARP(PCHARP nptr, PCHARP *endptr)
 	/* The exponent overflowed a `long int'.  It is probably a safe
 	   assumption that an exponent that cannot be represented by
 	   a `long int' exceeds the limits of a `double'.  */
+	/* NOTE: Don't trust the value returned from STRTOL.
+	 * We need to find the sign of the exponent by hand.
+	 */
+	p_wchar2 c;
+	while(WIDE_ISSPACE(c = EXTRACT_PCHARP(s))) {
+	  INC_PCHARP(s, 1);
+	}
 	if (endptr != NULL)
 	  *endptr = end;
-	if (exp < 0)
+	if (c == '-')
 	  goto underflow;
 	else
 	  goto overflow;
